/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <sys/file.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <math.h>
#include <strings.h>
#include <stdio.h>
#include <ctype.h>

/* returns NULL if file cannot be opened for reading.  Tries to expand ~ 
   to user's home directory.  Returns
   the FILE pointer if file successfully opened. */

FILE *
open_fl(fname, forwhat)
	char *fname;
	char *forwhat;
{
	FILE *simfile;
	char *expanded_filenm, *expando_filenm();

	expanded_filenm = expando_filenm(fname);

	simfile = fopen(expanded_filenm, forwhat);

	return(simfile);
}

char *
expando_filenm(filenm)
	char *filenm;
{
	static char home_dir[256];
	extern char *getenv();

	/* check to see if tilde is the first char in the file name.  If so,
	   then replace it with the path to the user's home directory.
	*/
	if ((filenm[0] == '~') && (filenm[1] == '/')) {
		strcpy(home_dir, getenv("HOME"));
		if (strcmp(home_dir, "") == 0) {
			printf("expando_filenm: HOME environment var undefined\n");
			return(filenm);
		}
		return(strcat(home_dir, &filenm[1]));
	}
	else return(filenm);
}


/* produces a unique name, replacing all X's in pattern with the number
   which is generated.  If there are more X's than there are digits in
   the number generated, then string is truncated. Thus, the X's should
   be trailing X's.

   Example: for the argument "pattern" you send in:
		/usr/tmp/grogXXXXXX

	It will then return a string like this:
		/usr/tmp/grog123890   (or some such hopefully unique #)

   Why not just use mktemp? Once upon a time, it started returning
   non-unique names, although I don't remember why.  Anyhow, this  fixed
   the whole mess.
*/
unique_tempnm(filenm_ptr, pattern)
	char *filenm_ptr, *pattern;
{
	struct timeval tp;
#ifdef VAX
	uid_t uid;
#else
	int uid;
#endif VAX
	long t;
	char holder[25];
	int i,j;
	char name[512];

	/* SIG_TMP_PATH is defined via  -D during compilation. */
	strcpy(name, SIG_TMP_PATH); 
	strcat(name, pattern);

	/* produce a "unique" number based on user's id and the time */
	uid = getuid();
	gettimeofday(&tp,(struct timezone*)0);
	t = (tp.tv_sec / tp.tv_usec) + (long)uid;

	sprintf(holder, "%d", t);
	i = 0;
	while (i < 25 && holder[i] != '\0')
		i++;
	/* i now points to first \0 in holder (the end of "t") */
	i--; /* now points to last char in "t" */

	j = 0;
	while ((name[j] != 'X') && (name[j] != '\0')) {
		filenm_ptr[j] = name[j];
		j++;
	}	
	/* j now points to first X in pattern -- maybe.  It might be at
	   end of pattern (no X's in pattern). */
	while ((name[j] == 'X') && (i >= 0)) {
		filenm_ptr[j] = holder[i];
		j++, i--;
	}
	filenm_ptr[j] = '\0';
}


delete_file(filenm_ptr)
	char *filenm_ptr;
{
	unlink(filenm_ptr);
}


/* Find the power of two which is closest to the number without being
   less than it. 
*/
int
least_power_of_two(number)
	int number;
{
	double answer;

	answer = 0.0;
	while (pow(2.0, answer) < (double)number)
		answer++;

	return((int) answer);
}

remove_substr(str, start, substr_len)
	char *str;
	int start, substr_len;
{
	int keep, toss;

	keep = start + substr_len;
	toss = start;

	while (str[keep] != '\0') 
		str[toss++] = str[keep++];
	str[toss] = '\0';
}

/* Returns beginning position of pattern in str, between the indices
   startsearch to endsearch.  Returns -1 if pattern not in string 
   within that range.
*/
int
substr(str, pattern, startsearch, endsearch)
	char *str, *pattern;
	int startsearch, endsearch;
{
	int patlen,strptr,patptr;
	char firstch;

	patlen = strlen(pattern);
	strptr = startsearch; 
	firstch = pattern[0];

	while (strptr <= endsearch) {
		/* find the first char of the pattern in the str*/
		while((strptr <= endsearch) && (str[strptr] != firstch))
			strptr++;

		/* Hopefully, strptr now points to char in str which matches
		   firstch in pattern. Check to make sure we have enough left
		   of string so that pattern could possibly match.
		*/	
		if ((strptr + patlen - 1) > endsearch)
			return(-1);

		/* we already know the first chars match, so advance each ptr */
		patptr = 1;
		strptr++;
		while((patptr < patlen)&&(pattern[patptr]==str[strptr])) {
			patptr++;
			strptr++;
		}	
		if (patptr == patlen)
			return(strptr-patlen);
	}

	return(-1);
}

/* Returns beginning position of pattern in str, between the indices
   endsearch to startsearch.  Returns -1 if pattern not in string 
   within that range.
*/
int
substr_backwards(str, pattern, startsearch, endsearch)
	char *str, *pattern;
	int startsearch, endsearch;
{
	int patlen,strptr,patptr;
	char firstch;

	patlen = strlen(pattern);
	strptr = startsearch - patlen + 1;
	firstch = pattern[0];

	while (strptr >= endsearch) {
		/* find the first char of the pattern in the str*/
		while(str[strptr] != firstch)
			if (strptr == endsearch)
				return(-1);
			else strptr--;

		/* now see if rest of pattern matches this position in the str */ 
		patptr = 1;
		while((patptr < patlen)&&(pattern[patptr]==str[strptr+patptr])) 
			patptr++;
			
		if (patptr == patlen)
			return(strptr);
		else strptr--;
	}
	return(-1);
}

char *
date_and_time()
{
	struct timeval tp;
	struct tm *localt;

	gettimeofday(&tp,(struct timezone*)0);
	localt = localtime(&tp.tv_sec);
	return(asctime(localt));
}


char *
user_id()
{
#ifdef VAX
	uid_t uid;
#else
	int uid;
#endif VAX
	struct passwd *pwptr;

	uid = getuid();
	pwptr = getpwuid((int)uid);
	return(pwptr->pw_name);
}

/* appends a line stating name of machine, name of user and current date 
   and time at end of file "filename".
*/
note_usage(filename)
	char *filename;
{
	FILE *use_file, *fopen();
	char name[64];

	use_file = fopen(filename, "a");
	if (use_file == NULL)
		return;
	
	name[0] = '\0';
	gethostname(name, 64);
	fprintf(use_file, "%s %s %s\n", user_id(), name, date_and_time());
	fclose(use_file);
}

/*  Given an array X of size numelts and a value, return the index 
	where X[index] <= value < X[index + 1], if X increasing.  Else
	return the index where X[index] >= value > X[index + 1], if
	X decreasing.  X must be monotonic increasing or decreasing.  
	We start by searching around the index "low" for the value.  
	This is useful when you sorta know where the value will be found 
	in X.  -1 as a resulting index indicates that value < X[0], if
	X increasing.  -1 as resulting index when X decreasing indicates
	that value > X[0].

	I made a slight change in that if value == X[high] after the
	final binary search, I return high.

	Taken from Numerical Recipes by Press, Flannery, etc.
*/

int
hunt(X, numelts, value, low)
	double *X;
	int numelts, low;
	double value;
{
	int ascending, high, mid;
	int inc, done;
	int returnval;

	if (X[numelts - 1] > X[0])
		ascending = 1;
	else ascending = 0;
	
	if ((low < 0) || (low >= numelts)) {
		low = -1;
		high = numelts;
	}
	else {

		inc = 1;
		if ((value >= X[low]) == ascending) {
			done = 0;
			while (! done) {
				high = low + inc;
				if (high >= numelts) {
					high = numelts;
					done = 1;
				}
				else if ((value >= X[high]) == ascending) {
					low = high;
					inc = inc + inc;
				}
				else done = 1;
			}
		}
		else {
			high = low;
			done = 0;
			while (! done) {
				low = high - inc;
				if (low < 0) {
					low = -1;
					done = 1;
				}
				else if ((value < X[low]) == ascending) {
					high = low;
					inc = inc + inc;
				}
				else done = 1;
			}
		}
	}

	while ((high - low) != 1) {
		mid = (high + low)/2;
		if ((value > X[mid]) == ascending)
			low = mid;
		else high = mid;
	}	

	if ((high < numelts) && (high > -1) && (X[high] == value))
		returnval = high;
	else returnval = low;
	
	if (returnval == -1)
		return(0);
	else return(returnval);	
}

double
screen_dist(x1, y1, x2, y2, xspan, yspan)
	double x1, y1, x2, y2, xspan, yspan;
{
	double xdist, ydist;

	xdist = (x2 - x1)/xspan;
	ydist = (y2 - y1)/yspan;
	return(sqrt((xdist * xdist) + (ydist * ydist)));
/*	return(sqrt(((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1)))); */
}
