/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "sigview.h"
#include <strings.h>

extern comm_file_t *Sig_files;
extern int Sig_precision;
extern int Sig_decimals;
extern axis_t *Sig_axes;
extern int Sig_axis_ct; /* number of axes actually on screen as opposed 
						to scrolled off, above or below. */
extern axis_t *Sig_top_axis, *Sig_bottom_axis;
extern double Sig_timestart;
extern double Sig_units;
extern int Sig_interactive_mode, Sig_notype;


create(axnm, signms, runnms)
	char *axnm, *signms, *runnms;
{
	sigT *signals, *get_signals();
	axis_t *add_axis();
	int i;

	if (Sig_notype && ((axnm == NULL) || (*axnm == '\0'))) {
		i= 0;
		while ((signms[i] != ' ') && (signms[i] != '\0')) {
			axnm[i] = signms[i];
			i++;
		}
		axnm[i] = '\0';
	}

	signals = get_signals(signms, runnms);
	if (signals != NULL) {
		add_axis(axnm, Sig_timestart, signals, NORMAL);
		if (Sig_interactive_mode)
			draw_screen();
	}
	else sig_msg("Create:  no matching signals found.","","");
}

change_mode(axnm, mode, conversion)
	char *axnm, *mode, *conversion;
{
	axis_t *the_axis, *find_axis();

	if ((the_axis = find_axis(axnm)) == NULL) {
		sig_msg("Mode: axis ", axnm, " not found.");
		return;
	}
	if (strcmp(mode, "normal") == 0)
		the_axis->axistype = NORMAL;
	else if (strcmp(mode, "bus") == 0) {
		if (strcmp(conversion, "hex") == 0)
			the_axis->axistype = HEX;
		else if (strcmp(conversion, "octal") == 0)
			the_axis->axistype = OCTAL;
		else if (strcmp(conversion, "decimal") == 0)
			the_axis->axistype = DECIMAL;
		else if (strcmp(conversion, "binary") == 0)
			the_axis->axistype = BINARY;
	}
	if (Sig_interactive_mode)
		draw_screen();
}

/* returns a linked list of all signals in all runs in all files. */
sigT *
all_signals()
{
	comm_file_t *fileptr;
	run_t *runptr;
	sigT *signals, *sig_ptr;
	int i;

	signals = sig_ptr = NULL;
	fileptr = Sig_files;
	while (fileptr != NULL) {
		runptr = fileptr->runs;
		while (runptr != NULL) {
			if (runptr->run_type == DIGITL)
				i = 0;
			else i = 1;
			while (i < runptr->num_Y) {
				if (sig_ptr == NULL) {
					sig_ptr = (sigT *) malloc(sizeof(sigT));
					signals = sig_ptr;
				}
				else {
					sig_ptr->next = (sigT *) malloc(sizeof(sigT));
					sig_ptr = sig_ptr->next;
				}
				sig_ptr->run = runptr;
				sig_ptr->next = NULL;
				if (runptr->run_type == DIGITL) {
					sig_ptr->dig_Y = runptr->data.digital.Y[i];
					sig_ptr->anal_Y = NULL;
				}
				else {
					sig_ptr->anal_Y = runptr->data.analog[i];
					sig_ptr->dig_Y = NULL;
				}
				i++;
			}
			runptr = runptr->next;
		}
		fileptr = fileptr->next;
	}
	return(signals);
}

/* returns a linked list of all signals which are listed for each run in
   runnms. */
sigT *
all_signals_run(runnms)
	char *runnms;
{
	run_t *run, *find_run();
	sigT *sig_ptr, *signals;
	int ptr, i;

	ptr = 0;
	signals = sig_ptr = NULL;

	if (runnms == NULL)
		return(NULL);

	while (runnms[ptr] != '\0') {
		run = find_run(&(runnms[ptr]));
		if (run != NULL) {
			if (run->run_type == DIGITL)
				i = 0;
			else i = 1;
			while (i < run->num_Y) {
				if (sig_ptr == NULL) {
					sig_ptr = (sigT *) malloc(sizeof(sigT));
					signals = sig_ptr;
				}
				else {
					sig_ptr->next = (sigT *) malloc(sizeof(sigT));
					sig_ptr = sig_ptr->next;
				}
				sig_ptr->run = run;
				sig_ptr->next = NULL;
				if (run->run_type == DIGITL) {
					sig_ptr->dig_Y = run->data.digital.Y[i];
					sig_ptr->anal_Y = NULL;
				}
				else {
					sig_ptr->anal_Y = run->data.analog[i];
					sig_ptr->dig_Y = NULL;
				}
				i++;
			}
		}
		while ((runnms[ptr] != ' ') && (runnms[ptr] != '\0'))
			ptr++;
		if (runnms[ptr] == ' ')
			ptr++;
	}
	return(signals);
}

/* returns a pointer to the run with the matching name. */

run_t *
find_run(name)
	char *name;
{
	comm_file_t *fileptr;
	run_t *runptr;

	fileptr = Sig_files;
	while (fileptr != NULL) {
		runptr = fileptr->runs;
		while (runptr != NULL)  {
			if (string_equal(runptr->name, name))
				return(runptr);
			runptr = runptr->next;
		}
		fileptr = fileptr->next;
	}
	return(NULL);
}

/* returns a linked list of signals from any run which has a name in the
   signms list . */

sigT *
all_signals_sigs(signms)
	char *signms;
{
	comm_file_t *fileptr;
	run_t *runptr;
	sigT *signals, *sig_ptr;
	int ptr, index;

	ptr = 0;
	signals = sig_ptr = NULL;

	if ((signms == NULL) || (signms[0] == '\0'))
		return(NULL);

	while (signms[ptr] != '\0') {
		fileptr = Sig_files;
		while (fileptr != NULL) {
			runptr = fileptr->runs;
			while (runptr != NULL) {
				if ((index = find_signm(&(signms[ptr]), runptr)) != -1) {
					if (sig_ptr == NULL) {
						sig_ptr = (sigT *) malloc(sizeof(sigT));
						signals = sig_ptr;
					}
					else {
						sig_ptr->next = (sigT *) malloc(sizeof(sigT));
						sig_ptr = sig_ptr->next;
					}
					sig_ptr->run = runptr;
					sig_ptr->next = NULL;
					if (runptr->run_type == DIGITL) {
						sig_ptr->dig_Y = runptr->data.digital.Y[index];
						sig_ptr->anal_Y = NULL;
					}
					else {
						sig_ptr->anal_Y = runptr->data.analog[index];
						sig_ptr->dig_Y = NULL;
					}
				}
				runptr = runptr->next;
			}
			fileptr = fileptr->next;
		}
		while ((signms[ptr] != ' ') && (signms[ptr] != '\0'))
			ptr++;
		if (signms[ptr] == ' ')
			ptr++;
	}
	return(signals);
}

int
find_signm(nm, run)
	char *nm;
	run_t *run;
{
	int i;

	for (i = 0; i < run->num_Y; i++)
		if (string_equal(nm, run->Ynames[i])) 
			return(i);
		
	return(-1);
}

sigT *
crossproduct(signms, runnms)
	char * signms, *runnms;
{
	run_t *runptr, *find_run();
	sigT *signals, *sig_ptr;
	int sigindex;
	int signmptr, runnmptr;

	runnmptr = 0;
	signals = sig_ptr = NULL;

	if ((runnms == NULL) || (runnms[0] == '\0'))
		return(NULL);

	while (runnms[runnmptr] != '\0') {
		runptr = find_run(&(runnms[runnmptr]));
		if (runptr != NULL) {
			signmptr = 0;
			while (signms[signmptr] != '\0') {
				sigindex = find_signm(&(signms[signmptr]), runptr);
				if (sigindex != -1) {
					if (sig_ptr == NULL) {
						sig_ptr = (sigT *) malloc(sizeof(sigT));
						signals = sig_ptr;
					}
					else {
						sig_ptr->next = (sigT *) malloc(sizeof(sigT));
						sig_ptr = sig_ptr->next;
					}
					sig_ptr->run = runptr;
					sig_ptr->next = NULL;
					if (runptr->run_type == DIGITL) {
						sig_ptr->dig_Y = runptr->data.digital.Y[sigindex];
						sig_ptr->anal_Y = NULL;
					}
					else {
						sig_ptr->anal_Y = runptr->data.analog[sigindex];
						sig_ptr->dig_Y = NULL;
					}
				}
				while ((signms[signmptr] != ' ') && (signms[signmptr] != '\0'))
					signmptr++;
				if (signms[signmptr] == ' ')
					signmptr++;
			}
		}
		while ((runnms[runnmptr] != ' ') && (runnms[runnmptr] != '\0'))
			runnmptr++;
		if (runnms[runnmptr] == ' ')
			runnmptr++;
	}
	return(signals);
}

sigT *
get_signals(signms, runnms)
	char *signms, *runnms;
{
	sigT *signals;
	sigT *all_signals(), *all_signals_sigs(), *all_signals_run(),
			*crossproduct();

	if ((runnms == NULL) || (*runnms == '\0')) {
		if ((signms == NULL) || (*signms == '\0'))
			signals = all_signals();
		else signals = all_signals_sigs(signms);
	}
	else if ((signms == NULL) || (*signms == '\0'))
			signals = all_signals_run(runnms);
		else signals = crossproduct(signms, runnms);

	return(signals);
}

/* Given a buffer with blank or null terminated strings, compare the
   first such string with string.  Return 1 if equal, else return 0. */
static int
string_equal(buffer, string)
	char *buffer, *string;
{
	int i;

	i = 0;
	while ((buffer[i] != ' ') && (buffer[i] != '\0')) {
		if (buffer[i] != string[i])
			return(0);
		i++;
	}
	if ((string[i] == ' ') || (string[i] == '\0'))
		return(1);
	else return(0);
}

axis_t *
find_axis(axnm)
	char *axnm;
{
	axis_t *axptr;

	axptr = Sig_axes;
	while ((axptr != NULL) && (strcmp(axptr->name, axnm) != 0))
		axptr = axptr->next;
	return(axptr);
}

free_signals(signals)
	sigT *signals;
{
	sigT *sig_ptr;

	while (signals != NULL) {
		sig_ptr = signals->next;
		free ((char *) signals);
		signals = sig_ptr;
	}
}

collect_all_axnms(buffer)
	char *buffer;
{
	axis_t *axptr;

	buffer[0] = '\0';
	axptr = Sig_axes;
	while (axptr != NULL) {
		strcat(buffer, axptr->name);		
		strcat(buffer, " ");
		axptr = axptr->next;
	}
}
