/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "gp.h"
#include <ctype.h>

extern comm_file_t *Sig_files;
extern int Sig_precision;
extern int Sig_decimals;
extern vert_t Sig_leftvert, Sig_rightvert;
extern axis_t *Sig_axes;
extern int Sig_axis_ct; /* number of axes actually on screen as 
						opposed to scrolled off, above or below. */
extern axis_t *Sig_top_axis, *Sig_bottom_axis;
extern double Sig_timestart, Sig_timeend, Sig_interval;
extern double Sig_units, Sig_multiplier;
extern int Sig_interactive_mode;
extern int Sig_main;

extern char **Sig_signms;
extern int Sig_signm_ct;

extern int Sig_notype, Sig_nums;
extern int Sig_yaxes;

extern int GP_mode, Global_quit, GP_currwin_id;
extern gp_win_t  *GP_currwin;
extern gp_win_t  *GP_wins;

addsigs(axnms, signms, runnms)
	char *axnms, *signms, *runnms;
{
	sigT *signals, *get_signals();
	char *axnm, *next_token();

	signals = get_signals(signms, runnms);
	if (signals != NULL) {
		if ((axnms != NULL) && (*axnms != '\0')) {
			axnm = next_token(&axnms);
			while (*axnm != '\0') {
				add_sigs_to_axis(axnm, signals);
				axnm = next_token(&axnms);
			}
		}
		else add_sigs_to_all_axes(signals);
	}
	free_signals(signals);
	if (Sig_interactive_mode)
		draw_screen();
}

combine(axnm, axnms)
	char *axnm, *axnms;
{
	char *otheraxis;
	sigT *signals, *axis_signals();
	axis_t *newaxis;
	int got_some_signals;
	axis_t *add_axis(), *find_axis(), *oldaxis;
	char buffer[BUFSIZE];
	int i;

	/* pick the first name out of axnms and use this name for our
	   new axis.  Add_axis will make sure it is unique and add subscripts
	   as necessary. */
	if (Sig_notype && (*axnm == '\0')) {
		i= 0;
		while ((axnms[i] != ' ') && (axnms[i] != '\0')) {
			axnm[i] = axnms[i];
			i++;
		}
		axnm[i] = '\0';
	}

	if ((axnms == NULL) || (*axnms == '\0')) { 
		collect_all_axnms(buffer);
		axnms = buffer;
	}

	newaxis = add_axis(axnm, Sig_timestart, NULL, NORMAL);
	oldaxis = find_axis(axnm);
	if (oldaxis != NULL)
		newaxis->yaxis.type = oldaxis->yaxis.type;
	else newaxis->yaxis.type = LINEAR;

	otheraxis = next_token(&axnms);
	got_some_signals = 0;

	while (*otheraxis != '\0') {
		if ((signals = axis_signals(otheraxis)) != NULL) {
			oldaxis = find_axis(otheraxis);
			/* Note that we prejudice the type of the axis to be LINEAR--
			   if even one LINEAR axis is found, then the type of the new
			   axis we're making will be linear. */
			if (oldaxis->yaxis.type == LINEAR)
				newaxis->yaxis.type = LINEAR;
			no_dup_add(signals, newaxis);
			delaxis(otheraxis);
			got_some_signals = 1;
		}
		otheraxis = next_token(&axnms);
	}
	if (!got_some_signals) 
		delaxis(axnm);
	else {
		/* set yaxis span to min, max voltages for all signals herein */
		set_min_max_yscale(newaxis->signals, &(newaxis->yaxis.minv), 
			&(newaxis->yaxis.maxv));

		if (Sig_interactive_mode)
			draw_screen();
	}
}

compare(signms, runnms)
	char *signms, *runnms;
{
	sigT *signals, *all_signals_sigs(), *crossproduct();
	int global, more_signms;
	char *signm, *next_token();
	axis_t *add_axis();

	if ((signms == NULL) || (*signms == '\0')) {
		if (Sig_signm_ct == 0)
			return;
		global = 0;
		signm = Sig_signms[0];
	}
	else {
		global = -1;
		signm = next_token(&signms);
	}

	more_signms = 1;
	while (more_signms) {
		if ((runnms == NULL) || (*runnms == '\0'))
			signals = all_signals_sigs(signm);
		else signals = crossproduct(signm, runnms);
		
		if (signals != NULL)
			add_axis(signm, Sig_timestart, signals, NORMAL);

		if (global == -1) {
			if ((signms == NULL) || (*signms == '\0'))
				more_signms = 0;
			else signm = next_token(&signms);
		}
		else {
			global++;
			if (global == Sig_signm_ct)
				more_signms = 0;
			else signm = Sig_signms[global];
		}
	}
	if (Sig_interactive_mode) {
		draw_screen();
		runnames();
	}
}

convert_axis(axnm, conversion)
	char *axnm, *conversion;
{
	axis_t *axptr, *find_axis();
	sigT *sigptr;
	int all_analog;

	axptr = find_axis(axnm);
	if (axptr == NULL)
		return;

	/* if all the signals on this axis are analog, then doing a conversion
	   on it not only makes no sense, BUT it's dangerous later when we go
	   to plot and think we have a bus axis here.  So, we prevent a totally
	   analog axis from being converted. */
	sigptr = axptr->signals;
	all_analog = 1;
	while ((sigptr != NULL) && (all_analog)) {
		if (sigptr->dig_Y != NULL)
			all_analog = 0;
		sigptr = sigptr->next;
	}
	if (all_analog)
		return;

	if (strcmp(conversion, "binary") == 0)
		axptr->axistype = BINARY;
	else if	(strcmp(conversion, "decimal") == 0)
		axptr->axistype = DECIMAL;
	else if	(strcmp(conversion, "hex") == 0)
		axptr->axistype = HEX;
	else if	(strcmp(conversion, "octal") == 0)
		axptr->axistype = OCTAL;

	if (Sig_interactive_mode)
		draw_screen();
}

logy(axnms)
	char *axnms;
{
	char *axnm, *next_token();
	axis_t *axptr, *find_axis();

	if ((axnms == NULL) || (*axnms == '\0')) { 
		axptr = Sig_axes;
		while (axptr != NULL) {
			axptr->yaxis.type =  1 - axptr->yaxis.type;
			axptr = axptr->next;
		}
	}
	else {
		axnm = next_token(&axnms);
		while (*axnm != '\0') {
			axptr = find_axis(axnm);
			if (axptr != NULL)  
				axptr->yaxis.type =  1 - axptr->yaxis.type;
			axnm = next_token(&axnms);
		}
	}
	if (Sig_interactive_mode)
		draw_screen();
}

copy(axnms)
	char *axnms;
{
	char *axnm, *next_token();
	axis_t *axptr, *find_axis();
	sigT *oldsigs, *newsigs, *signals;
	axis_t *newaxis;
	char buffer[BUFSIZE];

	if ((axnms == NULL) || (*axnms == '\0')) { 
		collect_all_axnms(buffer);
		axnms = buffer;
	}
	axnm = next_token(&axnms);
	while (*axnm != '\0') {
		axptr = find_axis(axnm);
		if (axptr != NULL)  {

			/* create copy of axptr's signals */
			oldsigs = axptr->signals;
			signals = (sigT *) malloc(sizeof(sigT));
			newsigs = signals;
			while (oldsigs != NULL) {
				newsigs->run = oldsigs->run;
				newsigs->dig_Y = oldsigs->dig_Y;
				newsigs->anal_Y = oldsigs->anal_Y;

				oldsigs = oldsigs->next;
				if(oldsigs != NULL) {
					newsigs->next = (sigT *) malloc(sizeof(sigT));
					newsigs = newsigs->next;
				}
				else newsigs->next = NULL;
			}

			newaxis = (axis_t *)malloc(sizeof(axis_t));
			unique_name(axnm, newaxis->name);
			newaxis->yaxis.type = axptr->yaxis.type;
			newaxis->yaxis.minv = axptr->yaxis.minv;
			newaxis->yaxis.maxv = axptr->yaxis.maxv;
			newaxis->yaxis.defawlt = axptr->yaxis.defawlt;
			newaxis->axistype = axptr->axistype;
			newaxis->timebegin = axptr->timebegin;
			newaxis->signals = signals;
			/* place new axis under it's parent. */
			insert_axis(newaxis, axptr);
		}
		axnm = next_token(&axnms);
	}
	if (Sig_interactive_mode)
		draw_screen();
}

deallocate_axptr(axptr)
	axis_t *axptr;
{
	sigT *sigptr, *next;

	sigptr = axptr->signals;
	while (sigptr != NULL) {
		next = sigptr->next;
		free((char *)sigptr);
		sigptr = next;
	}
	free((char *)axptr);
}

delete_axes(axnms)
	char *axnms;
{
	char *axnm, *next_token();

	if ((axnms == NULL) || (*axnms == '\0'))
		delete_all_axes();
	else {
		axnm = next_token(&axnms);
		while (axnm[0] != '\0') {
			delaxis(axnm);
			axnm = next_token(&axnms);
		}
	}
	if (Sig_interactive_mode)
		draw_screen();
}

delete_all_axes()
{
	axis_t *axptr, *next;

	axptr = Sig_axes;

	while (axptr != NULL) {
		next = axptr->next;
		deallocate_axptr(axptr);
		axptr = next;
	}
	Sig_axis_ct = 0;
	Sig_top_axis = Sig_axes = Sig_bottom_axis = NULL;
}

delaxis(axnm)
	char *axnm;
{
	axis_t *axptr, *find_axis();

	axptr = find_axis(axnm);
	if (axptr == NULL)
		return;
	delaxisptr(axptr);
}

/* Whenever we remove an axis, either from the visible or off-screen area,
   we call this (hopefully).  We try to maintain the current Sig_axis ct
   by pulling axes from offscreen.  The only way we could have an empty
   screen is if Sig_axes is NULL, i.e. there are no axes defined.  We assume
   the axptr is a valid ptr to an existing axis. */

delaxisptr(axptr)
	axis_t *axptr;
{
	/* first, readjust pointers */

	if (Sig_axes->next == NULL)  { /* then we're about to delete only axis */
		Sig_axis_ct = 0;
		Sig_top_axis = Sig_axes = Sig_bottom_axis = NULL;
		deallocate_axptr(axptr);
		return;
	}
	else {	/* we know there are at least two axes defined */
		if ((axptr == Sig_top_axis) && (axptr == Sig_bottom_axis)) {
			if (axptr->previous != NULL)
				Sig_top_axis = Sig_bottom_axis = axptr->previous;
			else Sig_top_axis = Sig_bottom_axis = axptr->next;
		}
		else if (axptr == Sig_top_axis) {
			if (Sig_top_axis->previous != NULL)
				Sig_top_axis = Sig_top_axis->previous;
			else {
				Sig_top_axis = Sig_top_axis->next;
				if (Sig_bottom_axis->next != NULL)
					Sig_bottom_axis = Sig_bottom_axis->next;
				else Sig_axis_ct--; /* couldn't pull any from offscreen */
			}
		}
		else {
			if (Sig_bottom_axis->next != NULL)
				Sig_bottom_axis = Sig_bottom_axis->next;
			else {
				if (axptr == Sig_bottom_axis)
					Sig_bottom_axis = Sig_bottom_axis->previous;
				if (Sig_top_axis->previous != NULL)
					Sig_top_axis = Sig_top_axis->previous;
				else Sig_axis_ct--;  /* couldn't pull any from offscreen */
			}
		}
	}

	/* now readjust the Sig_axes pointer */
	if (axptr == Sig_axes) {
		Sig_axes = axptr->next;
		Sig_axes->previous = NULL;
	}

	if (axptr->previous != NULL) 
		axptr->previous->next = axptr->next;
	if (axptr->next != NULL)
		axptr->next->previous = axptr->previous;

	/* finally, actually delete the damn thing */
	deallocate_axptr(axptr);
}

delsigs(axnms, signms, runnms)
	char *axnms, *signms, *runnms;
{
	sigT *signals, *get_signals();
	char *axnm, *next_token();

	signals = get_signals(signms, runnms);
	if (signals != NULL) {
		if ((axnms != NULL) && (*axnms != '\0')) {
			axnm = next_token(&axnms);
			while (*axnm != '\0') {
				del_sigs_from_axis(axnm, signals);
				axnm = next_token(&axnms);
			}
		}
		else del_sigs_from_all_axes(signals);
	}
	free_signals(signals);
	if (Sig_interactive_mode)
		draw_screen();
}

/* both Sigview and General Plot parsers call this routine */
help(cmdname)
	char *cmdname;
{
	graph_set_layer(2);
	graph_char_just_set("ul"),
	graph_draw_mode(0),
	graph_font_set("medium");

	while (isspace(*cmdname))
		cmdname++;

	help_msg(cmdname);
}


/* If max, then set to max, else set to n. */
do_interval(n, max)
	double n;
	int max;
{
	double newinterval, maxtime_of_all_runs();

	if (max) 
		newinterval = maxtime_of_all_runs();
	else newinterval = (n / Sig_multiplier);

	if (newinterval > 0.0) {
		Sig_interval = newinterval;
		Sig_timeend = Sig_timestart + Sig_interval;
	}
	if (Sig_interactive_mode)
		draw_screen();
}

notype()
{
	Sig_notype = 1 - Sig_notype;
}

nums()
{
	Sig_nums = 1 - Sig_nums;
	if (Sig_interactive_mode)
		draw_screen();
}

/* This is called by both transient mode and general mode parsers */
precision()
{
	Sig_precision = 1 - Sig_precision;
	if (Sig_interactive_mode)
		if (GP_mode)
			gp_draw_screen(GP_currwin_id);
		else draw_screen();
}

sigview_quit()
{
	Global_quit = 1;
}

show(signms, runnms)
	char *signms, *runnms;
{
	sigT *temp, *signals, *get_signals();
	char *runnm, *signm, *next_token();
	run_t *runptr, *find_run();
	comm_file_t *fileptr;
	int i;

	/* for every signal, for every run, get the appropriate signal
	   structure and add an axis with this signal on it. */
	if ((signms != NULL) && (*signms != '\0')) {
		signm = next_token(&signms);
		while (*signm != '\0') {
			if ((runnms == NULL) || (*runnms == '\0'))
				signals = all_signals_sigs(signm);
			else signals = crossproduct(signm, runnms);
			temp = signals;
			while (temp != NULL) {
				signals = temp->next;
				temp->next = NULL;
				add_axis(signm, Sig_timestart, temp, NORMAL);
				temp = signals;
			}
			signm = next_token(&signms);
		}
	}
	else {  /* no signals named, so get a list of all signms */
		if ((runnms != NULL) && (*runnms != '\0')) {
			runnm = next_token(&runnms);
			while (*runnm != '\0') {
				runptr = find_run(runnm);
				if (runptr != NULL) {
					if (runptr->run_type == DIGITL) 
						i = 0;
					else i = 1; /* we don't want the first column to be
								plotted on its own axis */
					while (i < runptr->num_Y) {
						signals = crossproduct(runptr->Ynames[i], runnm);	
						add_axis(runptr->Ynames[i],Sig_timestart,
							signals,NORMAL);
						i++;
					}
				}
				runnm = next_token(&runnms);
			}
		}
		else { /* no signals named, and no runs named, so do everything. */
			fileptr = Sig_files;
			while (fileptr != NULL) {
				runptr = fileptr->runs;
				while (runptr != NULL) {
					if (runptr->run_type == DIGITL)
						i = 0;
					else i = 1;
					while (i < runptr->num_Y) {
						signals =crossproduct(runptr->Ynames[i],runptr->name);	
						add_axis(runptr->Ynames[i],Sig_timestart,
							signals,NORMAL);
						i++;
					}
					runptr = runptr->next;
				}
				fileptr = fileptr->next;
			}
		}
	}
	if (Sig_interactive_mode)
		draw_screen();
}


snapshot(devname)
	char *devname;
{
	char name[64];
	char *env_name;
	extern char *getenv();

	if (Sig_interactive_mode) { 
		if ((GP_mode) && (GP_currwin->plot_list == NULL))  {
			sig_msg("Cannot take snapshot of this GP window ",
				"-- it contains no plots.","");
			return;
		}
	}

	if (*devname == '\0') {
		if ((env_name = getenv("SIGDEVICE")) == NULL) {
			sig_msg("Environment variable SIGDEVICE not found.  ",
					"Not executing snapshot.","");
			return;
		}
		strcpy(name, env_name);
	}
	else strcpy(name, devname);

	if (hardcopy_on(name))  {
		if (GP_mode) {
			gp_runs_area(GP_currwin_id);
			gp_draw_pic(GP_currwin_id);
		}
		else draw_screen();
		hardcopy_off();
	}
}

/* change starting time of axes in axnms to timebegin.  If axnms is null,
   change all axes and Sig_timebegin.  It is assumed that timebegin is in
   the units currently displayed so we must adjust by Sig_multiplier to get
   seconds. */

start_time(timebegin, axnms)
	char *axnms;
	double timebegin;
{
	axis_t *axptr, *find_axis();
	char *axnm, *next_token();

	timebegin = timebegin/Sig_multiplier;

	if ((axnms == NULL) || (*axnms == '\0')) {
		axptr = Sig_axes;
		while (axptr != NULL) {
			axptr->timebegin = timebegin;
			axptr = axptr->next;
		}
		Sig_timestart = timebegin;
		Sig_timeend = Sig_timestart + Sig_interval;
	}
	else {
		axnm = next_token(&axnms);
		while (*axnm != '\0') {
			axptr = find_axis(axnm);
			if (axptr != NULL) 
				axptr->timebegin = timebegin;
			axnm = next_token(&axnms);
		}
	}
	if (Sig_interactive_mode)
		axis_area();
}

/*  if called when in transient mode, then we need to check to make sure
	that at least one general mode window is open.  If not, let's open one.  
	If in general mode, then check to make sure the transient mode window is
	open.  If not, then open it.  We know a window is not open if it's
	structure contains window id -1.  */

open_other_mode() {
	if (GP_mode) {
		GP_mode = 0;
		if ((Sig_main == -1) && (Sig_interactive_mode)) {
			Sig_main = graph_create_free_win(0.0, 0.0, 600.0, 500.0, 
				0.9, 0.8, 'c', 0, 0, "sigview");
			draw_screen();
			graph_set_input(Sig_main, 0, "bk");
		}
	}
	else {
		GP_mode = 1;
		/* if one general win not yet opened, then open one (provided we
		   are in interactive mode). */

		if (GP_wins->id == -1) {
			if (Sig_interactive_mode) {
				open_gp_win(GP_wins);
				GP_currwin = GP_wins;
				GP_currwin_id = GP_wins->id;
			}
			else GP_currwin = GP_wins;
		}
	}
}

xscale(scale)
	double scale;
{
	if (scale == 0.0) {
		sig_msg("Cannot set scale factor to 0.0","","");
		return;
	}
	Sig_multiplier = scale;
	if (Sig_interactive_mode) 
		axis_area();
}

units(units)
	double units;
{
	Sig_units = units/Sig_multiplier;
	if (Sig_interactive_mode)
		axis_area();
}

yaxes()
{
	if (Sig_yaxes)
		Sig_yaxes = 0;
	else Sig_yaxes = 1;
	if (Sig_interactive_mode)
		axis_area();
}

void
yrange(axnms, yminspecified, ymaxspecified, ymin, ymax)
	char *axnms;
	int yminspecified, ymaxspecified;
	double ymin, ymax;
{
	axis_t *axptr, *find_axis();
	char *axnm, *next_token();

	if ((yminspecified && ymaxspecified) && (ymax <= ymin)) {
		sig_msg("Ymin must be less than ymax","","");
		return;
	}
	if ((axnms == NULL) || (*axnms == '\0')) {
		axptr = Sig_axes;
		while (axptr != NULL) {
			if (yminspecified) {
				if (ymin < axptr->yaxis.maxv) {
					axptr->yaxis.minv = ymin;
					axptr->yaxis.defawlt = 0;
				}
				else sig_msg("Ymin must be less than ymax for axis ",
					axptr->name,"");
			}
			if (ymaxspecified) {
				if (ymax > axptr->yaxis.minv) {
					axptr->yaxis.maxv = ymax;
					axptr->yaxis.defawlt = 0;
				}
				else sig_msg("Ymax must be greater than ymin for axis ",
					axptr->name,"");
			}
			axptr = axptr->next;
		}
	}
	else {
		axnm = next_token(&axnms);
		while (*axnm != '\0') {
			axptr = find_axis(axnm);
			if (axptr != NULL) {
				if (yminspecified) {
					if (ymin < axptr->yaxis.maxv) {
						axptr->yaxis.minv = ymin;
						axptr->yaxis.defawlt = 0;
					}
					else sig_msg("Ymin must be less than ymax for axis ",
						axptr->name,"");
				}
				if (ymaxspecified) {
					if (ymax > axptr->yaxis.minv) {
						axptr->yaxis.maxv = ymax;
						axptr->yaxis.defawlt = 0;
					}
					else sig_msg("Ymax must be greater than ymin for axis ",
						axptr->name,"");
				}
			}
			axnm = next_token(&axnms);
		}
	}
	if (Sig_interactive_mode)
		draw_screen();
}

del_sigs_from_axis(axnm, signals)
	char *axnm;
	sigT *signals;
{
	axis_t *axptr, *find_axis();

	axptr = find_axis(axnm);
	if (axptr == NULL)
		return;
	delete_sigs(signals, axptr);
	if (axptr->signals == NULL)
		delaxisptr(axptr);
}

del_sigs_from_all_axes(signals)
	sigT *signals;
{
	axis_t *axptr;

	axptr = Sig_axes;
	while (axptr != NULL) {
		delete_sigs(signals, axptr);
		if (axptr->signals == NULL)
			delaxisptr(axptr);
		axptr = axptr->next;
	}
}

add_sigs_to_axis(axnm, signals)
	char *axnm;
	sigT *signals;
{
	axis_t *axptr, *find_axis();

	axptr = find_axis(axnm);
	if (axptr == NULL)
		return;
	no_dup_add(signals, axptr);
}

add_sigs_to_all_axes(signals)
	sigT *signals;
{
	axis_t *axptr;

	axptr = Sig_axes;
	while (axptr != NULL) {
		no_dup_add(signals, axptr);
		axptr = axptr->next;
	}
}

no_dup_add(newsignals, axptr)
	sigT *newsignals;
	axis_t *axptr;
{
	sigT *ax_sigptr, *newsigs_sigptr, *temp;
	int already_there;

	/* for each new signal we propose to add to axptr, check and see if it
	   is already there.  If not, add it in. */
	newsigs_sigptr = newsignals;
	while (newsigs_sigptr != NULL) {
		ax_sigptr = axptr->signals;
		already_there = 0;
		while ((ax_sigptr != NULL) && (!already_there)) {
			if ((ax_sigptr->dig_Y == newsigs_sigptr->dig_Y) &&
				(ax_sigptr->anal_Y == newsigs_sigptr->anal_Y))
				already_there = 1;
			ax_sigptr = ax_sigptr->next;
		}
		if (!already_there) {
			temp = axptr->signals; /* hold onto old first signal */
			axptr->signals = (sigT *) malloc(sizeof(sigT));
			axptr->signals->run = newsigs_sigptr->run;
			axptr->signals->dig_Y = newsigs_sigptr->dig_Y;
			axptr->signals->anal_Y = newsigs_sigptr->anal_Y;
			axptr->signals->next = temp;
		}
		newsigs_sigptr = newsigs_sigptr->next;
	}
}

delete_sigs(signals, axptr)
	sigT *signals;
	axis_t *axptr;
{
	sigT *previous, *delsigs, *axsigs;
	int found;

	/* for each signal in "signals", if it is in axptr, then get rid
	   of it. */
	delsigs = signals;
	while (delsigs != NULL) {
		axsigs = axptr->signals;
		previous = NULL;
		found = 0;
		while ((axsigs != NULL) && (!found)) {
			if ((axsigs->dig_Y == delsigs->dig_Y) &&
				(axsigs->anal_Y == delsigs->anal_Y)) {
				found = 1;
				if (previous != NULL) {
					previous->next = axsigs->next;
					free((char *) axsigs);
					axsigs = previous->next;
				}
				else  {
					axptr->signals = axsigs->next;
					free((char *) axsigs);
					axsigs = axptr->signals;
				}
			}
			else { 
				previous = axsigs;
				axsigs = axsigs->next;
			}
		}
		delsigs = delsigs->next;
	}
}

sigT *
axis_signals(axnm)
	char *axnm;
{
	axis_t *axptr, *find_axis();

	axptr = find_axis(axnm);
	if (axptr == NULL)
		return(NULL);
	return(axptr->signals);
}

/* place axis "picked up" above axis "insert above".  NOTE that we assume
   that neither axis is ever null.  */

swap_axes(picked_up, insert_above)
	axis_t *picked_up, *insert_above;
{
	int i;
	axis_t *temp;

	/* nothing to do in these two cases.  Get outta here before they
	   cause trouble. */
	if (picked_up == insert_above)
		return;
	if (picked_up->next == insert_above)
		return;

	if (Sig_axes == picked_up) /* about to move head of the list down*/
		Sig_axes = picked_up->next;
	else if (Sig_axes == insert_above) /* about to supercede current head */
		Sig_axes = picked_up;

	if (Sig_top_axis == picked_up) /* about to move head of screen down*/
		Sig_top_axis = picked_up->next;
	else if (Sig_top_axis == insert_above) /* about to supercede current head */
		Sig_top_axis = picked_up;


	/* YOU WANT A MESS?  HERE IT IS!! */
	if (picked_up->previous != NULL)
		picked_up->previous->next = picked_up->next;
	if (picked_up->next != NULL)
		picked_up->next->previous = picked_up->previous;

	picked_up->next = insert_above;
	picked_up->previous = insert_above->previous;

	if (insert_above->previous != NULL)
		insert_above->previous->next = picked_up;
	insert_above->previous = picked_up;

	temp = Sig_top_axis;
	for (i = 0; i < Sig_axis_ct-1; i++) 
		temp = temp->next;
	Sig_bottom_axis = temp;
}

ytics(arglist)
	char *arglist;
{
	int ynumtics;
	char *numtics;
	char *next_token(), *namelist();

	numtics = next_token(&arglist);
	if (numtics[0] == '\0') {
		sig_msg("Ytics:  Number of yaxis tics is required.","","");
		return;
	}	

	sscanf(numtics,"%d",&ynumtics);
	GP_currwin->yscale.numtics = ynumtics;
	if (Sig_interactive_mode)
		if (GP_mode)
			gp_draw_screen(GP_currwin_id);
		else draw_screen();

}

xtics(arglist)
	char *arglist;
{
	int xnumtics;
	char *numtics;
	char *next_token(), *namelist();

	numtics = next_token(&arglist);
	if (numtics[0] == '\0') {
		sig_msg("Xtics:  Number of xaxis tics is required.","","");
		return;
	}	

	sscanf(numtics,"%d",&xnumtics);
	GP_currwin->xscale.numtics = xnumtics;
	if (Sig_interactive_mode)
		if (GP_mode)
                        gp_draw_screen(GP_currwin_id);
             	else draw_screen();

}
