/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <ctype.h>
#include <sys/wait.h>
#include "sigview.h"

int
create_and_compile_program(numvars, expr, var_start, var_len, eval_prog, eval_filenm) 
	int numvars;
	char *expr, *eval_prog, *eval_filenm;
	int var_start[], var_len[];
{
	int j;
	char newvar[17]; /* room for "vxx" where xx is 0 to 99 */
	FILE *open_fl(), *eval_file;
	char temp[256];
	char temp_expr[256];

	strcpy(temp_expr, expr);
	for (j = numvars; j > 0; j--) {
		sprintf(newvar, "v%d", j);
		replace_string(expr, var_start[j-1], var_len[j-1], newvar);
	}

	unique_tempnm(eval_prog, "#Sig_evalXXXXXX");
	strcpy(eval_filenm, eval_prog);
	strcat(eval_filenm, ".c");

	sig_file_msg(0, "Compiling evalution file: ",eval_prog);
	if ((eval_file = open_fl(eval_filenm, "w")) == NULL) {
		sig_file_msg(1,"Cannot open temp file ",eval_filenm);
		return(0);
	}
	fprintf(eval_file, "#include <math.h>\n#define STDIN 0\n#define STDOUT 1\nmain()\n{\n");
	fprintf(eval_file, "\tdouble res;\n\tchar buf[256];\n\tint stat, i;\n\tchar c;\n\tdouble ");
	for (j = 1; j < numvars; j++)
		fprintf(eval_file, "v%d, ", j);
	fprintf(eval_file, "v%d;", j);
	fprintf(eval_file, "\n\n\tstat = 1;\n\twhile (stat > 0) { /* EOF */\n\t\ti = 0;\n");
	fprintf(eval_file, "\t\tstat = read(STDIN, &c, 1);\n\t\twhile ((stat != 0) && (stat != -1) && (c != '~')) {\n");
	fprintf(eval_file, "\t\t\tbuf[i++] = c;\n\t\t\tstat = read(STDIN, &c, 1);\n\t\t}");
	fprintf(eval_file, "\n\t\tbuf[i] = '%c%c';\n\n\t\tif (stat > 0) {\n\t\t\tsscanf(buf, \"", '\\', '0');
	for (j = 1; j < numvars; j++)
		fprintf(eval_file, "%s ", "%lf");
	fprintf(eval_file, "%s\", ", "%lf");
	for (j = 1; j < numvars; j++)
		fprintf(eval_file, "&v%d, ", j);
	fprintf(eval_file, "&v%d);", j);
	fprintf(eval_file, "\n\t\t\tres = %s;\n\t\t\tsprintf(buf, \"%s \", res);", expr, "%g");
	fprintf(eval_file, "\n\t\t\twrite(STDOUT, buf, strlen(buf));\n\t\t}\n\t}\n\texit();\n}");
	fclose(eval_file);

	sprintf(temp, "cc %s -o %s -lm", eval_filenm, eval_prog); 
	system(temp);
	strcpy(expr, temp_expr);

	return(1);
}

replace_string(expr, replacee_start, replacee_len, replacer)
	char *expr, *replacer;
	int replacee_start, replacee_len;
{
	char temp[256];
	int i, temp_ptr;

	temp_ptr = 0;
	for (i = 0; i < replacee_start; i++) /* assumes replacee_start < exprlen */
		temp[temp_ptr++] = expr[i];

	i = 0;
	while (replacer[i] != '\0')
		temp[temp_ptr++] = replacer[i++];

	i = replacee_start + replacee_len;
	while (expr[i] != '\0')
		temp[temp_ptr++] = expr[i++];
	temp[temp_ptr] = '\0';

	strcpy(expr, temp);
}

int
fork_and_execute(eval_prog, var_data, numdata, numvars, res)
	char *eval_prog;
	double *var_data[];
	int numvars, numdata;
	double *res;
{
    int done;
    int parent_to_child[2];
    int child_to_parent[2];
	int send, receive;
	int i, j, n, stat;
	char c;
	char buf[256];
	char number[25];

	/* Create two pipes and establish where the channels are going. */
	pipe(parent_to_child);
 	send = parent_to_child[1];
	pipe(child_to_parent);
	receive = child_to_parent[0];

    if (fork() == 0) { 
	
		/* The child never uses the other ends of the pipes. Since
		 * everything is duplicated in fork, the parent's ends of 
		 * the pipes must be closed in the child.  */
		close(parent_to_child[1]);
		close(child_to_parent[0]);

		close(0);
		dup2(parent_to_child[0], 0);  /* child stdout goes into pipe2 */

		close(1);
		dup2(child_to_parent[1], 1);  /* child stdin come from pipe1 */

		/* The child program will read the data written to the parent's
		 * pipe1[1], it's output will appear on the parent's pipe2[0].  */

		execl(eval_prog, eval_prog, NULL );
		sig_file_msg(1,"addplot: couldn't exec prog to figure expr:",eval_prog);
		exit(1);  
    }
	/* The parent never uses the child's ends of the pipes. Since
	 * everything is duplicated in fork, the child's ends of the
	 * pipes must be closed in the parent.  */
	close(parent_to_child[0]);
	close(child_to_parent[1]);

	for (i = 0; i < numdata; i++) { 
		buf[0] = '\0';
		for (n = 0; n < numvars; n++) {
			sprintf(number, " %g", var_data[n][i]);
			strcat(buf, number);
		}
		strcat(buf, "~");
		write(send, buf, strlen(buf));

		done = 0;
		j = 0;
		stat = read(receive, &c, 1);
		if (stat < 1) 
			return(0);

		while (! done) {
			if ((stat < 1) || (c == ' ')) {
				buf[j] = '\0';
				done = 1;
			}
			else {
				buf[j++] = c;
				stat = read(receive, &c, 1);
			}
		}
		sscanf(buf, "%lf", &(res[i]));
/*fprintf(stderr,"Parent: last line with res[i] %lf\n", res[i]); */
	}
	close(send);
	return(1);
}
