/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "gp.h"

extern gp_win_t *GP_currwin, *GP_wins;
extern int GP_mode;
extern int GP_currwin_id;
extern int Sig_interactive_mode;

/* given value we desire, and the points above and below it, 
   figure what the interpolated value must be. */
double
linear_interp(x1, x2, y1, y2, xvalue)
	double x1, y1, x2, y2, xvalue;
{
	if (x1 == x2)
		return(y1);
	else return(y1 + (((xvalue - x1)/(x2 - x1)) * (y2 - y1)));
}

double
quad_interp(x1, x2, x3, y1, y2, y3, value)
	double x1, x2, x3, y1, y2, y3, value;
{
	double result;

	result = (
		(((((value-x2) * y1) + ((x1-value) * y2)) / (x1 - x2)) * (value-x3)) +
		(((((value-x3) * y2) + ((x2-value) * y3)) / (x2 - x3)) * (x1-value))) /
		(x1 - x3);

	return(result);
}

#define ARRAYSIZE	200

do_ft(vecname, oldX, oldY, n, nfreq, inverse, fund_freq, quadratic, create_file)
	char *vecname;
	double *oldX, *oldY;
	int n, nfreq, quadratic, create_file;
	int inverse; /* NOT IMPLEMENTED */
	double fund_freq;
{
	int i;
	double *X, *Y;
	double delta;
	int x1, x2, x3;
	double interp();
	double time_span, wavelength, min;
	double *freq, *mag, *phase, *nmag, *nphase;

	if (oldX[n -1] < oldX[0]) {
		sig_msg("Ft: xscale is decreasing", "","");
		return;
	}

	/* If no fundamental freq was specified, we will use all the data */
	time_span = oldX[n - 1] - oldX[0];
	if (fund_freq == 0.0) {
		wavelength = time_span;
		fund_freq = 1.0 / time_span;
	}
	else wavelength = 1.0 / fund_freq;

	/* Note that we look only at the last period (which, hopefully is
	   1/fund_freq).  We will ignore all other data. */
	if (time_span < wavelength) {
		sig_msg("Ft: wavelength based on fund_freq longer than time span",
				"","");
		return;
	}
	else min = oldX[n-1] - wavelength;

	delta = (oldX[n-1] - min) / ARRAYSIZE;

	X = (double *) calloc(ARRAYSIZE, sizeof(double));
	Y = (double *) calloc(ARRAYSIZE, sizeof(double));

	freq = (double *) calloc(nfreq, sizeof (double));
	mag = (double *) calloc(nfreq, sizeof (double));
	phase = (double *) calloc(nfreq, sizeof (double));
	nmag = (double *) calloc(nfreq, sizeof (double));
	nphase = (double *) calloc(nfreq, sizeof (double));

	for (i = 0; i < ARRAYSIZE; i++) {
		X[i] = min + (i * delta);

		x1 = hunt(oldX, n, X[i], i-1);
		if (quadratic) {
			if (x1 < (n - 2)) {
				x2 = x1 + 1;
				x3 = x2 + 1;
			}
			else {
				x1 = n-3;
				x2 = n-2;
				x3 = n-1;
			}
			Y[i] = quad_interp(oldX[x1], oldX[x2], oldX[x3], 
					oldY[x1], oldY[x2], oldY[x3], X[i]);
		}
		else {
			if (x1 < (n - 1))
				x2 = x1 + 1;
			else {
				x1 = n - 2;
				x2 = n - 1;
			}
			Y[i] = linear_interp(oldX[x1], oldX[x2], oldY[x1], oldY[x2], X[i]);
		}
	}

	dft(Y, ARRAYSIZE, nfreq, fund_freq, freq, mag, phase, nmag, nphase);

	write_and_load_dft(freq, mag, phase, nmag, nphase, 
			nfreq, vecname, create_file);

	free((char *) X);
	free((char *) Y);
	free((char *) freq);
	free((char *) mag);
	free((char *) phase);
	free((char *) nmag);
	free((char *) nphase);
}

/* This routine adapted from code in U. C. Berkeley's spice3.  It is
   assumed that the array Y will contain the data for exactly one full 
   period satisfying "fund_freq" */

#ifndef PI
#define PI 3.14159265358979323846
#endif /*PI*/

dft(Y, n, nfreq, fund_freq, freq, mag, phase, nmag, nphase)
	double *Y, *freq, *mag, *phase, *nmag, *nphase;
	int n, nfreq;
	double fund_freq;
{
	int i, j;
	double temp;
	double atan2(), sin(), cos();

	for(i = 0; i < nfreq; i++) {
		mag[i] = 0.0;
		phase[i] = 0.0;
	}

	for(i = 0; i < n; i++) {
		for(j = 0; j < nfreq; j++) {
			mag[j] += (Y[i] * sin(j * 2.0 * PI * i /((double) n)));
			phase[j] += (Y[i] * cos(j * 2.0 * PI * i/((double) n)));
		}
	}

	mag[0] = phase[0] / ((double)n);
	phase[0] = nmag[0] = nphase[0] = freq[0] = 0.0;

    for(i = 1; i < nfreq; i++) {
		temp = mag[i] * 2.0 / ((double)n);
		phase[i] *= 2.0 / ((double)n);
		freq[i] = i * fund_freq;
		mag[i] = sqrt((temp * temp) + (phase[i] * phase[i]));
                phase[i] = atan2(phase[i], temp) * 180.0 / PI;
		nmag[i] = mag[i] / mag[1];
		nphase[i] = phase[i] - phase[1];
	}
}

write_and_load_dft(freq, mag, phase, nmag, nphase, nfreq, vecname, create_file)
	double *freq, *mag, *phase, *nmag, *nphase;
	char *vecname;
	int nfreq, create_file;
{
	FILE *ft_file, *open_fl();
	char filename[SIGNMSIZE];
	char signm[SIGNMSIZE];
	int i;
	gp_win_t *define_gp_win(), *win;
	run_t *last_run();

	if (! create_file)
		sprintf(filename, "/usr/tmp/%s.ft", vecname);
	else sprintf(filename, "%s.ft", vecname);

	if ((ft_file = open_fl(filename, "w")) == NULL) {
		sig_msg("Unable to open file",filename,"to store ft info.");
		return;
	}

	fprintf(ft_file, "AC ANALYSIS\n\n");
	fprintf(ft_file, "Frequency mag(%s) phase(%s) nmag(%s) nphase(%s)\n",
		vecname, vecname, vecname, vecname);

	for (i = 0; i < nfreq; i++)
		fprintf(ft_file, "%e %e %e %e %e\n",
			freq[i], mag[i], phase[i], nmag[i], nphase[i]);

	fclose(ft_file);

	/* now we need to open a new genwin and plot the magnitude */
	if (Sig_interactive_mode)  {
		win = define_gp_win();
		GP_mode = 1;
		load_simfile(filename);
		if (! create_file)
			delete_file(filename);

		/* since we created this file of name "filename" we know it
		   only has one run in it.  Locate the run in the last file 
		   in our list of Sig_files and we will find the run we want. */
		
		win->curr_run = last_run();
		gp_xaxis("Frequency");

/*		sprintf(signm, "%s/mag(%s)", win->curr_run->name, vecname); */
		sprintf(signm, "\'mag(%s)\'", vecname); 
		addplot_p(signm, 0);
		win->bar_plot = 1;
		open_gp_win(win);
		gp_draw_screen(GP_currwin_id);
	}
	else {
	}
}
