/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "sigview.h"
#include <strings.h>

typedef struct gp_xpt_s gp_xpt_t;
struct gp_xpt_s {
	run_t *run; /* run from which this list of x points comes */
	double *xpts; /* list of xpts.  DOES NOT point into run_t.data.analog!
					  We allocate space for a copy. */
	gp_xpt_t *next;
};

typedef struct gp_xscale_s gp_xscale_t;
struct gp_xscale_s {
	double data_left, data_right; /* min, max values found in all scales */
	char *expression;
	int numtics; /* number of xscale tics */
	int log; /* is the xscale currently rep'd as log? */
	int increasing; /* does x axis increase from left to right? */
	double axis_left, axis_right; /* left and right of current x axis */
	gp_xpt_t *scales; /* for each run, point to or create the list of
			x points.  A new one of these is figured each time
			a run is loaded, if we can satisfy 'expression' 
			with the run. */
};

typedef struct gp_yscale_s gp_yscale_t;
struct gp_yscale_s {
	int log;
	int numtics; /* number of yscale tics */
	double min, max; /* of all plots defined.  When a plot added or
						deleted, we update these two. */
	double save_min, save_max;
	double currmin, currmax; /* as seen on screen.  Updated whenever we
						zoom or unzoom */
	double save_currmin, save_currmax;
	double logmin, logmax;
};

typedef struct gp_plot_s gp_plot_t;
struct gp_plot_s {
	double ymin, ymax;
	double save_ymin, save_ymax;
	double *ypts; /* list of ypts.  DOES NOT point into run_t.data.analog. */
	double *save_ypts; /* when the yscale is LOG, this will point to the
					un"log"ged pts.  when yscale is linear, this will be null*/
	gp_xpt_t *xpts; /* a pointer to the appropriate scale in xscales->scales*/
	char *expression;
	int numpts; /* the number of pts in ypts.  This is NOT necessarily
			the same as number of pts in xpts (i.e. when the plot
			is a constant plot). */
	int layer;
	int marked_for_death; /*is this plot marked for deletion? */
	int closest_to_zero; /* init to -1. If logy cmd is given, we'll find and
		fill it with the index into ypts of that value which is closest
		to zero without actually being zero. */
	gp_plot_t * next;
};

typedef struct gp_label_s gp_label_t;
struct gp_label_s {
	char *text; /* null terminated */
	double x1, y1, x2, y2; /* bbox of label in terms of 0,0,600,500 */
	double cx, cy; /* graph_text(label->cx, label->cy, label->text); when
					 graph_char_just_set("cc"); */
	gp_label_t *prev, *next;
};

typedef struct gp_win_s gp_win_t;
struct gp_win_s {
	int id;
	run_t *curr_run;
	gp_xscale_t xscale;
	gp_yscale_t yscale;
	gp_plot_t *plot_list;
	gp_label_t *label_list;
	int precision; /* print doubles in scientific notation? */
	int bar_plot;  /* 1 means yes, 0 means connect the dots (default) */
	gp_win_t *next;
};

#define XLEFT	65.0
#define XRIGHT	580.0
#define YBOT	75.0
#define YTOP	480.0

#define YPTS	0
#define XPTS	1

#define MIDBUTTON	0
#define RTBUTTON	3
#define LEFTBUTTON	1

/* stucture used to pass results back from yacc parser to our world */
struct vvec_s {
	double *values;
	int num_vals;
	run_t *run;
	int manufactured;
};
