/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "values.h"
#include "gp.h"

extern int Sig_interactive_mode;
extern comm_file_t *Sig_files;

extern int GP_mode, Global_quit;
extern int GP_currwin_id; /* if 0, then we got this command from the GP
	main window.  Else we got this command from a GP child window. */
extern int GP_grid_on;
extern gp_win_t *GP_currwin;

gp_analyze()
{
	int win_id, val;
	double x, y;

	graph_draw_mode(0);
        graph_set_layer(0);
        graph_rect(0.0, 0.0, 600.0, 44.0);
        graph_set_layer(2); /*textoverbkgd*/
        graph_char_just_set("ll");
        graph_font_set("medium");
        graph_text(10.0, 10.0, "Click any button to continue.");
	while ( graph_await_event(&win_id, &val, &x, &y) != 'b' )
           graph_text(10.0, 10.0, "Click any button to continue.");
}

gp_logy()
{
	/* toggle value */
	if (GP_currwin->yscale.log)
		GP_currwin->yscale.log = 0;
	else GP_currwin->yscale.log = 1;

	transform_yscale(&(GP_currwin->yscale), GP_currwin->plot_list);

	/* now, if we're interactive, redraw the screen */
	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

gp_logx()
{
	if (GP_currwin->xscale.log)
		GP_currwin->xscale.log = 0;
	else GP_currwin->xscale.log = 1;

	transform_xscale(&(GP_currwin->xscale));

	/* now, if we're interactive, redraw the screen */
	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

gp_grid()
{
	if (GP_grid_on)
		GP_grid_on = 0;
	else GP_grid_on = 1;

	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

/* this is to set the low and high values of the y axis.  It's kind of a 
   non-interactive zoom just for the y axis.  Note that we assume the
   person gave the new min, max values in log form if yscale is currently
   log.  The first parameter is not used, but it is used in another 
   procedure "yrange".  (see "range_p" in parse.c) */
void
gp_yrange(garbage, minspecified, maxspecified, min, max)
	char *garbage;
	int minspecified, maxspecified;
	double min, max;
{
	gp_yscale_t *yscale;

	yscale = &(GP_currwin->yscale);
	if (minspecified) {
		if (yscale->log) {
			yscale->currmin = min;
			if (yscale->save_currmin < 0.0)
				yscale->save_currmin = -pow(10.0, min);
			else yscale->save_currmin = pow(10.0, min);
		}
		else yscale->currmin = min;
	}

	if (maxspecified) {
		if (yscale->log) {
			yscale->currmax = max;
			if (yscale->save_currmax < 0.0)
				yscale->save_currmax = -pow(10.0, max);
			else yscale->save_currmax = pow(10.0, max);
		}
		else yscale->currmax = max;
	}

	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

/* this is to set the low and high values of the x axis.  It's kind of a 
   non-interactive zoom just for the x axis.  */
void
gp_xrange(garbage, minspecified, maxspecified, min, max)
	char *garbage;
	int minspecified, maxspecified;
	double min, max;
{
	if (minspecified)
		GP_currwin->xscale.axis_left = min;
	if (maxspecified)
		GP_currwin->xscale.axis_right = max;
	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

gp_xaxis(new_expr)
	char *new_expr;
{
	gp_xscale_t *curr_xscale;
	char nospace_expr[256];
	comm_file_t *file_ptr;
	run_t *run_ptr, *run_used;
	int old_increasing;
	double *new_xpts;
	int numpts, made_up;
	int i, found_suitable, top;
	gp_xpt_t *scale, *previous;

	curr_xscale = &(GP_currwin->xscale);
	remove_whitespace(new_expr, nospace_expr); 
	if (curr_xscale->expression != NULL) {
		if(strcmp(curr_xscale->expression, nospace_expr) == 0)
			return;
	}

	/* we may change direction in the following */
	old_increasing = curr_xscale->increasing;
	curr_xscale->increasing = -1; 

	/* Now for every loaded run, try to create or locate a vector of xpts
	   satisfying new_expr. */
	found_suitable = 0;
	file_ptr = Sig_files;
	while (file_ptr != NULL) {
		run_ptr = file_ptr->runs;
		while (run_ptr != NULL) {
		  if (run_ptr->run_type != DIGITL) {

			if (gp_parse_expression(run_ptr, &run_used, new_expr, &new_xpts,
					&numpts, &made_up, 0) &&
				valid_scale(run_ptr, new_xpts, numpts, curr_xscale, 
					nospace_expr, made_up)){
				
				found_suitable = 1;

				/* first, search for the scale which goes with this run*/
				scale = GP_currwin->xscale.scales;
				previous = NULL;
				while ((scale != NULL)&&(scale->run != run_used)){
					previous = scale;
					scale = scale->next;
				}

				if (scale == NULL) {
					if (previous == NULL)
						scale = GP_currwin->xscale.scales = 
							(gp_xpt_t *) malloc (sizeof(gp_xpt_t));
					else scale = previous->next = 
							(gp_xpt_t *) malloc (sizeof(gp_xpt_t));
					scale->next = NULL;
					if (made_up)
						scale->xpts = new_xpts;
					else {
						scale->xpts = (double *)calloc(numpts, sizeof(double));
						for (i = 0; i < numpts; i++)
							scale->xpts[i] = new_xpts[i];
					}
					scale->run = run_used;
				}
				else {
					free((char *) scale->xpts);
					if (made_up)
						scale->xpts = new_xpts;
					else {
						scale->xpts = (double *)calloc(numpts, sizeof(double));
						for (i = 0; i < numpts; i++)
							scale->xpts[i] = new_xpts[i];
					}
					scale->run = run_used;
				}
				if (curr_xscale->log) {
					for (i = 0; i < numpts; i++)
						scale->xpts[i] = LOGTEN(scale->xpts[i]);
					curr_xscale->data_left = LOGTEN(curr_xscale->data_left);
					curr_xscale->data_right = LOGTEN(curr_xscale->data_right);
					curr_xscale->axis_left = LOGTEN(curr_xscale->axis_left);
					curr_xscale->axis_right = LOGTEN(curr_xscale->axis_right);
				}
			}
		  }
		  run_ptr = run_ptr->next;
		}
		file_ptr = file_ptr->next;
	}

	if (!found_suitable) { /* didn't find anything suitable for new_expr */
		curr_xscale->increasing = old_increasing;
		return;
	}

	if (curr_xscale->expression == NULL) 
		curr_xscale->expression = (char *) calloc(strlen(new_expr) + 1, 
			sizeof(char));
	else curr_xscale->expression = (char *) 
			realloc(curr_xscale->expression, strlen(new_expr) + 1);
	strcpy(curr_xscale->expression, new_expr);

	/* now figure new data_left and right.  */
	scale = curr_xscale->scales;
	curr_xscale->data_left = scale->xpts[0];
	curr_xscale->data_right = scale->xpts[scale->run->num_X - 1];
	scale = curr_xscale->scales->next;

	while (scale != NULL) {
		top = scale->run->num_X - 1;
		if (curr_xscale->increasing) { 
			if (scale->xpts[0] < curr_xscale->data_left)
				curr_xscale->data_left = scale->xpts[0];
			if (scale->xpts[top] > curr_xscale->data_right)
				curr_xscale->data_right = scale->xpts[top];
		}
		else {
			if (scale->xpts[0] > curr_xscale->data_left)
				curr_xscale->data_left = scale->xpts[0];
			if (scale->xpts[top] < curr_xscale->data_right)
				curr_xscale->data_right = scale->xpts[top];
		}
		scale = scale->next;
	}
	curr_xscale->axis_left = curr_xscale->data_left;	
	curr_xscale->axis_right = curr_xscale->data_right;
 }

transform_yscale(yscale, plot_list)
	gp_yscale_t *yscale;
	gp_plot_t *plot_list;
{
	gp_plot_t *plot;
	int i;
	double screen_min, min, y;
	int c_t_z;

	if (yscale->log) { 
		/* then we were sent here to take the log of all values */
		yscale->save_min = yscale->min;
		yscale->save_max = yscale->max;
		yscale->save_currmin = yscale->currmin;
		yscale->save_currmax = yscale->currmax;
		
		screen_min = (double)MAXFLOAT;

		plot = plot_list;
		while (plot != NULL) {
			plot->save_ymin = plot->ymin;
			plot->save_ymax = plot->ymax;
			plot->save_ypts = plot->ypts;
			plot->ypts = (double *)calloc(plot->numpts, sizeof(double));

			/* If we haven't already done so (i.e. closest to zero is a
			   nonsense index), find the value which is closest to zero
			   and save index to it in the field of the same name */
			if (plot->closest_to_zero == -1) {
				min = LOGTEN(plot->save_ypts[0]);
				c_t_z = 0;
				for (i = 0; i < plot->numpts; i++) {
					plot->ypts[i] = y = LOGTEN(plot->save_ypts[i]);
					if (y < min) {
						min = y;
						c_t_z = i;
					}
				}
				plot->closest_to_zero = c_t_z;
			}
			else {
				for (i = 0; i < plot->numpts; i++)
					plot->ypts[i] = LOGTEN(plot->save_ypts[i]);
			}
			screen_min = SMIN(screen_min, plot->ypts[plot->closest_to_zero]); 
			plot->ymax = SMAX(LOGTEN(plot->save_ymax), LOGTEN(plot->save_ymin));
			plot->ymin = plot->ypts[plot->closest_to_zero];
			plot = plot->next;
		}

		/* Now reset the mins, maxes as log vals */
		yscale->max = 
			SMAX(LOGTEN(yscale->save_min), LOGTEN(yscale->save_max));
		yscale->min = screen_min;
		yscale->currmax = 
			SMAX(LOGTEN(yscale->save_currmin), LOGTEN(yscale->save_currmax));
		if ((yscale->save_currmin <= 0.0) && (yscale->save_currmax > 0.0))
			yscale->currmin = screen_min; 
		else yscale->currmin = 
			SMIN(LOGTEN(yscale->save_currmin), LOGTEN(yscale->save_currmax));
		/* wrr */
		yscale->currmin = floor( yscale->currmin );
		if( yscale->currmax / floor( yscale->currmax ) > 1.001 ) {
		   yscale->currmax = floor( yscale->currmax ) + 1;
		} else {
		   yscale->currmax = floor( yscale->currmax );
		}
	}
	else {
		/* Now return the saved ypts back to their original place of
		   honor, trashing the logged ypts. */
		yscale->min = yscale->save_min;
		yscale->max = yscale->save_max;
		yscale->currmin = yscale->save_currmin;
		yscale->currmax = yscale->save_currmax;

		plot = plot_list;
		while (plot != NULL) {
			plot->ymin = plot->save_ymin;
			plot->ymax = plot->save_ymax;

			free ((char *)plot->ypts);
			plot->ypts = plot->save_ypts;
			plot = plot->next;
		}
	}
}

transform_xscale(xscale)
	gp_xscale_t *xscale;
{
	gp_xpt_t *scale;
	int i;
	char holder[256];

	if (xscale->log) {
		xscale->data_left = LOGTEN(xscale->data_left);
		xscale->data_right = LOGTEN(xscale->data_right);
		xscale->axis_left = LOGTEN(xscale->axis_left);
		xscale->axis_right = LOGTEN(xscale->axis_right);

		strcpy(holder, "log10(");
		strcat(holder, xscale->expression);
		strcat(holder, ")");

		xscale->expression = (char *) realloc(xscale->expression, 
			strlen(xscale->expression) + 8);
		strcpy(xscale->expression, holder);

		scale = xscale->scales;
		while (scale != NULL) {
			for (i = 0; i < scale->run->num_X; i++)
				scale->xpts[i] = LOGTEN(scale->xpts[i]);
			scale = scale->next;
		}
		xscale->axis_left = floor( xscale->axis_left );
		if( xscale->axis_right / floor( xscale->axis_right) > 1.001 ) {
		   xscale->axis_right = floor( xscale->axis_right ) + 1;
		} else {
		   xscale->axis_right = floor( xscale->axis_right );
		}
	}
	else {
		strcpy(holder, &(xscale->expression[6]));
		holder[strlen(holder) -1] = '\0';
		gp_xaxis(holder);
	/*	strcpy(xscale->expression, holder); */
	}
}

gp_precision()
{
	GP_currwin->precision =  1 - GP_currwin->precision;
	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}
