/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "gp.h"

int GP_currwin_id; /* window from which we are currently processing a command */
gp_win_t *GP_currwin; /* pointer to current window's info */
gp_win_t *GP_wins;  /* list of pointers to our children windows */

extern comm_file_t *Sig_files;
extern int Sig_decimals;

gp_window_menu(x, y)
	double x, y;
{
	if (y > 60.0)
		return;

	if (y >= 50.0) { 
		change_curr_run(x);
		gp_runs_area(GP_currwin_id);
		return;
	}	

	if (y > 44.0)
		return;

	if (y < 22.0) {
		if (x <= 100.0) 
			track();
		else if (x <= 200.0)
			delete_from_picture();
		else if (x <= 300.0)
			slope();
		else if (x <= 400.0)
			zoom_reset();
		else if (x <= 500.0)
			def_and_open_gp_win();
		else sigview_quit();
	}
	else {
		if (x <= 100.0) 
			gp_label();
		else if (x <= 200.0)
			gp_do_show();
		else if (x <= 300.0) 
			do_snapshot();
		else if (x <= 400.0)
			zoom();
		else if (x <= 500.0)
			gp_draw_screen(GP_currwin_id);
		else destroy_gp_win();
	}
}

change_curr_run(x)
	double x;
{
	comm_file_t *fileptr;
	run_t *runptr;
	double begin, end, graph_str_width();

	if (x < 22.0)
		return;

	graph_char_just_set("ll");
	graph_font_set("medium");

	fileptr = Sig_files;
	begin = 22.0;
	end = 22.0 + graph_str_width("Runs: ");
	while (fileptr != NULL) {
		if (! DIGITAL(fileptr->type)) {
			runptr = fileptr->runs;
			while (runptr != NULL) {
				if (runptr == GP_currwin->curr_run) 
					end = end + graph_str_width("[");
				end = end + graph_str_width(runptr->name);
				if (runptr == GP_currwin->curr_run) 
					end = end + graph_str_width("]");
				
				if ((x >= begin) && (x <= end)) {
					GP_currwin->curr_run = runptr;
					return;
				}

				end = end + graph_str_width(" ");
				begin = end;
				runptr = runptr->next;
			}
		}
		fileptr = fileptr->next;
	}
}

track() 
{
	int win_id, val;
	double x, y, lastx;
	double lastxval, xunit, axisleft;
	double *lastyval, gp_interp_y();
	int i, count;
	gp_plot_t *plot;
/*	gp_xpt_t *scale; */
	int decreasing;
	double namepos, dummy;
	int where;
	int set_size, set_number, char_curves;
	int precision;
	double x1, y1; /* dummies, not used */
	gp_plot_t *which_plot();
	gp_xscale_t xscale;

	if (GP_currwin->plot_list == NULL)
		return;

	/* detect if there are any char curves plots */
	char_curves = 0;
	plot = GP_currwin->plot_list;
	count = 0;
	while (plot != NULL) {
		if ((plot->xpts != NULL) && (plot->xpts->run->run_type == CHAR_CURVES))
			char_curves = 1;
		count++;
		plot = plot->next;
	}

	lastx = XLEFT;
	xunit = (GP_currwin->xscale.axis_right - GP_currwin->xscale.axis_left) /
			(XRIGHT - XLEFT);
	axisleft = GP_currwin->xscale.axis_left;
	lastxval = axisleft;
	lastyval = (double *) calloc (count, sizeof(double));
	if (GP_currwin->xscale.increasing)
		decreasing = 0;
	else decreasing = 1;

	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("lr");
	graph_font_set("medium");

	/* if there was at least one char curves set plotted, we need to
	   determine which of the set(s) we are going to track */
	if (char_curves) {
		graph_text(300.0, 10.0, "Click any button over desired curve.");
		graph_set_input(GP_currwin_id, 1, "b");
		set_gp_win_scale(GP_currwin_id,GP_currwin->xscale,GP_currwin->yscale);
		graph_await_event(&win_id, &val, &x, &y);
		which_plot(x, y, &x1, &y1, &set_number, decreasing);
		if (set_number == -1) {
			sig_msg("Could not determine which curve of curve set desired.",
				"  Will track curve sets randomly","");
			char_curves = 0;
		}
		graph_window_set(GP_currwin_id, 0.0, 0.0, 600.0, 500.0);
	}

	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_text(300.0, 10.0, "Click any button to stop tracking.");

	precision = GP_currwin->precision;

	graph_set_layer(10);
	graph_draw_mode(0);
	graph_line(lastx, 60.0, lastx, 495.0);
	if( GP_currwin->xscale.log ) {
	   graph_text_double(lastx, 34.0, pow(10.0,lastxval), precision);
	} else {
	   graph_text_double(lastx, 34.0, lastxval, precision);
	}
	namepos = YTOP - 15.0;
	for (i = 0; i < count; i++) {
	   if( GP_currwin->yscale.log ) {
		graph_text_double(460.0, namepos, pow(10.0,lastyval[i]), precision);
	   } else {
		graph_text_double(460.0, namepos, lastyval[i], precision);
	   }
		namepos = namepos - 15.0;
	}

	graph_set_input(GP_currwin_id, 1, "bm");
	while (graph_await_event(&win_id, &val, &x, &y) == 'm') {

		graph_draw_mode(1);
		graph_line(lastx, 60.0, lastx, 495.0);
		if( GP_currwin->xscale.log ) {
		   graph_text_double(lastx, 34.0, pow(10.0,lastxval),precision);
		} else {
		   graph_text_double(lastx, 34.0, lastxval, precision);
		}
		namepos = YTOP - 15.0;
		for (i = 0; i < count; i++) {
		   if( GP_currwin->yscale.log ) {
			graph_text_double(460.0, namepos, pow(10.0,lastyval[i]), precision);
		   } else {
			graph_text_double(460.0, namepos, lastyval[i], precision);
		   }
			namepos = namepos - 15.0;
		}

		graph_draw_mode(0);
		if (x < XLEFT) {
			graph_line(XLEFT, 60.0, XLEFT, 495.0);
			lastx = XLEFT;
		}
		else if (x > XRIGHT) {
			graph_line(XRIGHT, 60.0, XRIGHT, 495.0);
			lastx = XRIGHT;
		}
		else {
			graph_line(x, 60.0, x, 495.0);
			lastx = x;
		}
		lastxval = axisleft + ((lastx - XLEFT) * xunit);
		if( GP_currwin->xscale.log ) {
		   graph_text_double(lastx, 34.0, pow(10.0,lastxval),precision);
		} else {
		   graph_text_double(lastx, 34.0, lastxval, precision);
		}

		plot = GP_currwin->plot_list;
		i = 0;
		where = 0;
		while (plot != NULL) {
			if (plot->xpts == NULL)  /* a constant plot */
				lastyval[i] = plot->ypts[0];
			else {
				set_size = plot->xpts->run->num_X / plot->xpts->run->num_sets;
				where = hunt(plot->xpts->xpts, set_size, lastxval, where);

				if (char_curves)
					lastyval[i] = gp_interp_y(lastxval, &dummy, where, 
						&(plot->xpts->xpts[set_size * set_number]),
						&(plot->ypts[set_size * set_number]), 
						set_size, decreasing);
				else lastyval[i] = gp_interp_y(lastxval, &dummy, where, 
						plot->xpts->xpts, plot->ypts, 
						plot->numpts, decreasing);
			}
			i++;
			plot = plot->next;
		}	
		namepos = YTOP - 15.0;
		for (i = 0; i < count; i++) {
		   if ( GP_currwin->yscale.log ) {
		      graph_text_double(460.0, namepos, pow(10.0,lastyval[i]), precision);
		   } else {
		      graph_text_double(460.0, namepos, lastyval[i], precision);
		   }
			namepos = namepos - 15.0;
		}
	}
/* wrr
	graph_draw_mode(1);
	graph_line(lastx, 60.0, lastx, 495.0);
	if( GP_currwin->xscale.log ) {
	   graph_text_double(lastx, 34.0, pow(10.0,lastxval), precision);
	} else {
	   graph_text_double(lastx, 34.0, lastxval, precision);
	}
	namepos = YTOP - 15.0;
	for (i = 0; i < count; i++) {
	   if( GP_currwin->yscale.log ) {
		graph_text_double(460.0, namepos, pow(10.0,lastyval[i]), precision);
	   } else {
		graph_text_double(460.0, namepos, lastyval[i], precision);
	   }
		namepos = namepos - 15.0;
	}
*/
	if( GP_currwin->xscale.log ) {
           graph_text_double(lastx, 80.0, pow(10.0,lastxval), precision);
        } else {
           graph_text_double(lastx, 80.0, lastxval, precision);
        }
	graph_set_input(GP_currwin_id, 0, "bk");
}

static
zoom()
{
	int win_id, val;
	double x1, y1, x2, y2;
	double lastx2, lasty1;
	double xunit, yunit;
	double temp;

	if (GP_currwin->plot_list == NULL) {
		sig_msg("No plots have yet been added to zoom in on","","");
		return;
	}
	
	/* draw over menu here to show mouse buttons */
	graph_set_drawing(GP_currwin_id);
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(300.0, 33.0, "Zoom.");
	graph_text(300.0, 22.0, "Use rubber-band box to set focus.");
	graph_text(300.0, 11.0, "Place upper left corner now.");

	/* first give upper left cursor so that user can set new upper left */
	graph_cursor("upperleft");
	graph_set_input(GP_currwin_id, 1, "b");
	/*wait for buttonhit that places corner*/
	graph_await_event(&win_id, &val, &x1, &y2);

	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_text(300.0, 33.0, "Zoom.");
	graph_text(300.0, 22.0, "Now place lower right corner.");

	graph_set_layer(10); /* leftvert */
	graph_cursor("lowerright");
	graph_set_input(GP_currwin_id, 1, "bm");
	lastx2 = x1; lasty1 = y2;

	while (graph_await_event(&win_id, &val, &x2, &y1) == 'm') {
		graph_draw_mode(1);
		graph_line_rect(x1, lasty1, lastx2, y2);
		graph_draw_mode(0);
		graph_line_rect(x1, y1, x2, y2);
		lastx2 = x2; lasty1 = y1;
	}
	graph_draw_mode(1);
	graph_line_rect(x1, lasty1, lastx2, y2);

	/* if the bounding box doesn't bound anything, then get the
	   heck out of here. */
	if ((y1 == y2) || (x1 == x2)) {
		graph_cursor("pointer");
		graph_set_input(GP_currwin_id, 0, "bk");
		return;
	}

	/* next recalculate the new time interval */
	if (y1 > y2) {
		temp = y2; y2 = y1; y1 = temp; }
	if (x1 > x2) {
		temp = x2; x2 = x1; x1 = temp; }
	xunit = (GP_currwin->xscale.axis_right - GP_currwin->xscale.axis_left) / 
				(XRIGHT - XLEFT);
	yunit = (GP_currwin->yscale.currmax - GP_currwin->yscale.currmin) / 
				(YTOP - YBOT);
	if( GP_currwin->xscale.log ) {
		GP_currwin->xscale.axis_left = 
		   floor( GP_currwin->xscale.axis_left + (x1 - XLEFT) * xunit);
		GP_currwin->xscale.axis_right = 
		   floor( GP_currwin->xscale.axis_right - (XRIGHT - x2) * xunit) + 1;
	} else {
		GP_currwin->xscale.axis_left = 
		   GP_currwin->xscale.axis_left + ((x1 - XLEFT) * xunit); 
		GP_currwin->xscale.axis_right = 
		   GP_currwin->xscale.axis_right - ((XRIGHT - x2) * xunit);
	}

	if( GP_currwin->yscale.log ) {
		GP_currwin->yscale.currmin = 
		   floor( GP_currwin->yscale.currmin + (y1 - YBOT) * yunit); 
		GP_currwin->yscale.currmax = 
		   floor( GP_currwin->yscale.currmax - (YTOP - y2) * yunit) + 1;
		if (GP_currwin->yscale.save_currmin < 0.0)
			GP_currwin->yscale.save_currmin = 
				-(pow(10.0, GP_currwin->yscale.currmin));
		else GP_currwin->yscale.save_currmin = 
				(pow(10.0, GP_currwin->yscale.currmin));

		if (GP_currwin->yscale.save_currmax < 0.0)
			GP_currwin->yscale.save_currmax = 
				-(pow(10.0, GP_currwin->yscale.currmax));
		else GP_currwin->yscale.save_currmax = 
				(pow(10.0, GP_currwin->yscale.currmax));
	} else {
		GP_currwin->yscale.currmin = 
		   GP_currwin->yscale.currmin + ((y1 - YBOT) * yunit); 
		GP_currwin->yscale.currmax = 
		   GP_currwin->yscale.currmax - ((YTOP - y2) * yunit);
	}
	gp_menu_area(GP_currwin_id);
	gp_draw_screen(GP_currwin_id);
	graph_cursor("pointer");
	graph_set_input(GP_currwin_id, 0, "bk");
}

static
zoom_reset()
{
	if( GP_currwin->xscale.log ) {
	   GP_currwin->xscale.axis_left = floor( GP_currwin->xscale.data_left );
	   if( GP_currwin->xscale.data_right / floor( GP_currwin->xscale.data_right) > 1.001 ) {
	      GP_currwin->xscale.axis_right = floor( GP_currwin->xscale.data_right ) + 1;
	   } else {
              GP_currwin->xscale.axis_right = floor( GP_currwin->xscale.data_right );
	   }
	} else {
	   GP_currwin->xscale.axis_left = GP_currwin->xscale.data_left;
	   GP_currwin->xscale.axis_right = GP_currwin->xscale.data_right;
	}

	if( GP_currwin->yscale.log ) {
	   GP_currwin->yscale.currmin = floor( GP_currwin->yscale.min ); 
	   if( GP_currwin->yscale.max / floor( GP_currwin->yscale.max ) > 1.001 ) {
	      GP_currwin->yscale.currmax = floor( GP_currwin->yscale.max ) + 1;
	   } else {
	      GP_currwin->yscale.currmax = floor( GP_currwin->yscale.max );
	   }
	} else {
	   GP_currwin->yscale.currmin = GP_currwin->yscale.min; 
	   GP_currwin->yscale.currmax = GP_currwin->yscale.max;
	}
	
	GP_currwin->yscale.save_currmin = GP_currwin->yscale.save_min;
	GP_currwin->yscale.save_currmax = GP_currwin->yscale.save_max;

	gp_menu_area(GP_currwin_id);
	gp_draw_screen(GP_currwin_id);
}


static int All_deleted;

static
draw_plot_expr_popup(popup, plotlist)
	int popup;
	gp_plot_t *plotlist;
{
	double ypos;

	graph_set_drawing(popup);
	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 23.0, 60.0, 0.0);
	graph_set_layer(5);
	graph_rect(0.0, 21.0, 60.0, 20.0);
	graph_set_layer(8);
	graph_rect(0.0, 22.0, 60.0, 21.0);
	graph_set_layer(4);
	graph_rect(0.0, 23.0, 60.0, 22.0);
	graph_set_layer(3);
	graph_char_just_set("cc");
	if (All_deleted)
		graph_text(30.0, 20.5, "RESTORE ALL");
	else graph_text(30.0, 20.5, "DELETE ALL");
	graph_text(15.0, 21.5, "BACK");
	graph_text(45.0, 21.5, "FORWARD");
	graph_text(30.0, 22.5, "DO IT");
	graph_set_layer(6);
	graph_line_rect(0.0, 23.0, 60.0, 0.0);

	/* now list the plot expressions */
	graph_char_just_set("ll");
	ypos = 1.0;
	while ((ypos < 21.0) && (plotlist != NULL)) {
		graph_set_layer(plotlist->layer);
		graph_text(7.0, ypos, plotlist->expression);
		if (plotlist->marked_for_death) {
			graph_set_layer(2);
			graph_text(1.0, ypos, "X");
		}

		plotlist = plotlist->next;
		ypos = ypos + 1.0;
	}
}

static
mark_all_plots(top_of_list, popup, mark)
	gp_plot_t *top_of_list;
	int popup;
	int mark;
{
	gp_plot_t *plot;

	graph_set_drawing(GP_currwin_id);
	if (mark)
		graph_draw_mode(1);
	else graph_draw_mode(0);

	plot = GP_currwin->plot_list;
	while (plot != NULL) {
		plot->marked_for_death = mark;
		draw_plot(plot, GP_currwin->xscale, 
				GP_currwin->yscale, GP_currwin->bar_plot);
		plot = plot->next;
	}

	draw_plot_expr_popup(popup, top_of_list);
}

delete_from_picture()
{
	gp_plot_t *this_plot, *last_hilited_plot;
	gp_plot_t *top_of_list;
	double namepos, last_namepos;
	int win_id, button;
	double x, y;
	char event;
	gp_plot_t *which_name(), **every_20th;
	int popup, count, done;
	int every_20th_ct, which_top;
	gp_plot_t *plot;
	gp_xpt_t *scale;

	if (GP_currwin->plot_list == NULL)
		return;

	popup = graph_create_fixed_win(425.0, 50.0, 595.0, 495.0, 
				0.0, 23.0, 60.0, 0.0);
	if (popup == -1) 
		return;

	if (! set_gp_win_scale(GP_currwin_id, GP_currwin->xscale,
			GP_currwin->yscale))
		return;

	/* place pointers to every 20th element in plot list, to allow
	   easy scrolling. */
	every_20th = (gp_plot_t **) calloc(1, sizeof(gp_plot_t *));
	every_20th[0] = GP_currwin->plot_list;
	every_20th_ct = 1;

	plot = GP_currwin->plot_list;
	while (plot != NULL) {
		count = 0;

		while ((count < 20)  && (plot != NULL)) {
			count++;
			plot = plot->next;
		}
		if (plot != NULL) {
			every_20th_ct++;
			every_20th = (gp_plot_t **) realloc((char *)every_20th,
				sizeof(gp_plot_t *) * every_20th_ct);
			every_20th[every_20th_ct - 1] = plot;
		}
	}
	which_top = 0;
	
	graph_set_input(popup, 1, "bm");
	graph_set_drawing(popup);
	graph_cursor("pointer");

	All_deleted = 0;
	draw_plot_expr_popup(popup, every_20th[which_top]);

	last_hilited_plot = NULL;
	top_of_list = every_20th[which_top];
	done = 0;
	last_namepos = 0.0;
	while (! done) {
		event = graph_await_event(&win_id, &button, &x, &y);

		if ((event == 'm') && (y < 20.0)) {
			this_plot = which_name(y, top_of_list, &namepos);
			if ((this_plot != NULL) && 
				(! this_plot->marked_for_death) &&
				(this_plot != last_hilited_plot)) {
				if (last_hilited_plot != NULL) {
					graph_set_drawing(GP_currwin_id);
					graph_draw_mode(3);
					draw_plot(last_hilited_plot, GP_currwin->xscale,
						GP_currwin->yscale, GP_currwin->bar_plot);
					graph_set_drawing(popup);
					graph_set_layer(last_hilited_plot->layer);
					graph_text(7.0, last_namepos, 
								last_hilited_plot->expression);
				}
				graph_set_drawing(GP_currwin_id);
				graph_draw_mode(2);
				draw_plot(this_plot,GP_currwin->xscale,
						GP_currwin->yscale, GP_currwin->bar_plot);
				graph_set_drawing(popup);
				graph_set_layer(this_plot->layer);
				graph_text(7.0, namepos, this_plot->expression);
				last_hilited_plot = this_plot;
				last_namepos = namepos;
			}
		}
		else if (event == 'b') {
			if (y > 22.0)
				done = 1;

			else if (y > 21.0) {
				if (x > 30.0)
					which_top = ((which_top + 1) == every_20th_ct) ? 
						which_top : which_top + 1;
				else which_top = (which_top == 0) ? 
						which_top : which_top - 1;
				top_of_list = every_20th[which_top];
				draw_plot_expr_popup(popup, top_of_list);
			}

			else if (y > 20.0) {
				if (All_deleted) 
					All_deleted = 0;
				else All_deleted = 1;
				mark_all_plots(top_of_list, popup, All_deleted); 
			}
			else {
				this_plot = which_name(y, top_of_list, &namepos);
				if (this_plot != NULL) {
					if (this_plot->marked_for_death) { /* reincarnate it */
						this_plot->marked_for_death = 0;
						graph_set_drawing(GP_currwin_id);
						graph_draw_mode(0);
						draw_plot(this_plot, GP_currwin->xscale,
								GP_currwin->yscale, GP_currwin->bar_plot);
						graph_draw_mode(1);
						graph_set_drawing(popup);
						graph_set_layer(2);
						graph_text(1.0, namepos, "X");
					}
					else { /* mark this one for possible deletion */
						this_plot->marked_for_death = 1;
						graph_set_drawing(GP_currwin_id);
						graph_draw_mode(1);
						draw_plot(this_plot, GP_currwin->xscale,
								GP_currwin->yscale, GP_currwin->bar_plot);
						graph_draw_mode(0);
						graph_set_drawing(popup);
						graph_set_layer(2);
						graph_text(1.0, namepos, "X");
					}
				}
			}
		}
	}

	delete_plots();

	if (GP_currwin->plot_list == NULL) { /* then undefine the xscale */

		if (GP_currwin->xscale.expression != NULL) {
			free ((char *)GP_currwin->xscale.expression);
			GP_currwin->xscale.expression = NULL;
		}
		while (GP_currwin->xscale.scales != NULL) {
			scale = GP_currwin->xscale.scales;
			free ((char *)scale->xpts);
			GP_currwin->xscale.scales = scale->next;
			free ((char *)scale);
		}
	}
	else { /* update the yscale so that max, min are correct */
		plot = GP_currwin->plot_list;
		adjust_yscale(1, plot->ymin, plot->ymax, plot->save_ymin, plot->save_ymax);
		plot = plot->next;
		while (plot != NULL) {
			adjust_yscale(0, plot->ymin, plot->ymax, plot->save_ymin, plot->save_ymax);
			plot = plot->next;
		}
	}

	graph_end_win(popup);
	graph_set_drawing(GP_currwin_id);
	graph_window_set(GP_currwin_id, 0.0, 0.0, 600.0, 500.0);
	graph_set_input(GP_currwin_id, 0, "bk");
	graph_cursor("pointer");
	gp_draw_screen(GP_currwin_id);
}

delete_plots()
{
	gp_plot_t *plot, *previous;

	plot = GP_currwin->plot_list;
	previous = NULL;
	while (plot != NULL) {
		if (plot->marked_for_death) {
			if (plot == GP_currwin->plot_list)
				GP_currwin->plot_list = plot->next;

			free((char *)plot->expression);
			free((char *)plot->ypts);
			if (previous != NULL)
				previous->next = plot->next;
			free((char *)plot);
			if (previous != NULL)
				plot = previous->next;
			else plot = GP_currwin->plot_list;
		}
		else {
			previous = plot;
			plot = plot->next;
		}
	}
}

static gp_plot_t *
which_name(y, plots, namepos)
	double y;
	gp_plot_t *plots;
	double *namepos;
{
	int pos;

	pos = 1.0;
	while ((plots != NULL) && (y > pos)) {
		pos++;
		plots = plots->next;
	}

	*namepos = pos;
	return(plots);
}

static
update_slope_line(popup, x1, y1, oldx2, oldy2, precision)
	double x1, y1, oldx2, oldy2;
	int popup, precision;
{
	double diffx, diffy, x_intercept, y_intercept;
	char format[128], string[128];

	graph_line(x1, y1, oldx2, y1);
	graph_line(oldx2, y1, oldx2, oldy2);
	graph_set_drawing(popup);
	sprintf( format, "%%%d.%de %%%d.%de\0", 
		8+Sig_decimals, Sig_decimals, 8+Sig_decimals, Sig_decimals );
	sprintf( string, format, oldx2, oldy2 );
	graph_text(30.0, 60.0, string );
	diffx = (oldx2 - x1);
	diffy = (oldy2 - y1);
	sprintf( string, format, diffx, diffy );
	graph_text(30.0, 40.0, string );
	sprintf( format, "%%%d.%de\0", 8+Sig_decimals, Sig_decimals );
	y_intercept = y1 - ( diffy/diffx ) * x1;
	x_intercept = - y_intercept / ( diffy/diffx );
	sprintf( string, format, y_intercept );
	graph_text(73.0, 60.0, string );
	sprintf( string, format, x_intercept );
	graph_text(73.0, 80.0, string );

	if (diffx == 0)
		graph_text(40.0, 20.0, "undefined");
	else {
		sprintf( string, format, diffy/diffx );
		graph_text(30.0, 20.0, string );
	}
	if (diffy == 0)
		graph_text(40.0, 20.0, "undefined");
	else {
		sprintf( string, format, diffx/diffy );
		graph_text(73.0, 20.0, string );
	}
}

slope()
{
	int popup, decreasing;
	double x, y, x1, y1, x2, y2, oldx2, oldy2;
	int set_size, set_number, win_id, val;
	gp_plot_t *plot, *which_plot();
	double axis_left, axis_right;
	int precision;
	char string[128], format[128];

	if (GP_currwin->plot_list == NULL)
		return;

	popup = graph_create_fixed_win(0.0, 0.0, 600.0, 50.0, 
		0.0, 0.0, 100.0, 100.0);
	if (popup == -1) 
		return;

	if (! set_gp_win_scale(GP_currwin_id, GP_currwin->xscale,
			GP_currwin->yscale))
		return;

	axis_left = GP_currwin->xscale.axis_left;
	axis_right = GP_currwin->xscale.axis_right;
	decreasing = ! GP_currwin->xscale.increasing;

	graph_set_drawing(popup);
	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 100.0, 100.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_text(20.0, 50.0, "Place the first slope point");
	
	/* First, accept the first point from user.  Determine which plot he
	   means, change point to be actually on the plot, start rubberbanding. */

	graph_set_input(GP_currwin_id, 1, "b");
	graph_await_event(&win_id, &val, &x, &y);
	if (decreasing) {
		if (x > axis_left)
			x = axis_left;
		else if (x < axis_right)
			x = axis_right;
	}
	else {
		if (x < axis_left)
			x = axis_left;
		else if(x > axis_right)
			x = axis_right;
	}
	
	plot = which_plot(x, y, &x1, &y1, &set_number, decreasing);
	if (plot == NULL) {
		graph_end_win(popup);
		graph_set_drawing(GP_currwin_id);
		graph_window_set(GP_currwin_id, 0.0, 0.0, 600.0, 500.0);
		graph_set_input(GP_currwin_id, 0, "bk");
		return;
	}

	if ((plot->xpts != NULL) && (plot->xpts->run->run_type == CHAR_CURVES))
		set_size = plot->xpts->run->num_X / plot->xpts->run->num_sets;
	else set_size = plot->numpts;

	precision = GP_currwin->precision;

	graph_set_layer(0);
	graph_rect(0.0, 0.0, 100.0, 100.0);
	graph_set_layer(10);
	graph_text(5.0, 60.0, plot->expression);
	graph_text(20.0, 80.0, "x1, y1:");
	sprintf( format, "%%%d.%de %%%d.%de\0", 
		8+Sig_decimals, Sig_decimals, 8+Sig_decimals, Sig_decimals );
	sprintf( string, format, x1, y1 );
	graph_text(30.0, 80.0, string );

	graph_text(20.0, 60.0, "x2, y2:");
	graph_text(20.0, 40.0, "dx, dy:");
	graph_text(20.0, 20.0, "dy/dx:");
	graph_text(60.0, 80.0, "X intercept:" );
	graph_text(60.0, 60.0, "Y intercept:" );
	graph_text(60.0, 20.0, "dx/dy:" );

	graph_set_input(GP_currwin_id, 1, "bm");
	oldx2 = x1;
	oldy2 = y1;
	graph_draw_mode(0);
	graph_set_drawing(GP_currwin_id);
	update_slope_line(popup, x1, y1, oldx2, oldy2, precision);
	while (graph_await_event(&win_id, &val, &x, &y) == 'm') {

		if (! decreasing) {
			if (x < axis_left)
				x = axis_left;
			else if (x > axis_right)
				x = axis_right;
		}
		else {
			if (x > axis_left)
				x = axis_left;
			else if (x < axis_right)
				x = axis_right;
		}

		graph_draw_mode(1);
		graph_set_drawing(GP_currwin_id);
		update_slope_line(popup, x1, y1, oldx2, oldy2, precision);

		which_point_on_plot(plot,x,&x2,&y2,set_size,set_number,decreasing);

		graph_draw_mode(0);
		graph_set_drawing(GP_currwin_id);
		update_slope_line(popup, x1, y1, x2, y2, precision);

		oldx2 = x2;
		oldy2 = y2;
	}
	graph_set_drawing(GP_currwin_id);
	graph_draw_mode(1);
	graph_line(x1, y1, oldx2, y1);
	graph_line(oldx2, y1, oldx2, oldy2);

	graph_end_win(popup);
	graph_window_set(GP_currwin_id, 0.0, 0.0, 600.0, 500.0);
	graph_set_input(GP_currwin_id, 0, "bk");
}

/* returns the y value corresponding to "x_in".  May also adjust x_in
   so that it is not off the end of the data found in the array X (will
   return x in "x_out").  It is assumed that array X is monotonically 
   increasing or decreasing. */
double
gp_interp_y(x_in, x_out, near_index, X, Y, numx, decreasing)
	double x_in, *x_out;
	int near_index, numx, decreasing;
	double *X, *Y;
{
	double timebelow, timeabove;
	double percentdist, voltincr;
	int next_index;

	/* adjust x val */
	*x_out = x_in;
	if (decreasing) {
		if (x_in > X[0])
			*x_out = X[0];
		else if (x_in < X[numx - 1])
			*x_out = X[numx - 1];
	}
	else {
		if (x_in < X[0])
			*x_out = X[0];
		else if (x_in > X[numx - 1])
			*x_out = X[numx - 1];
	}

	if (numx == 1) /* if a constant plot, there's only one val it could be. */
		return(Y[0]);

	if (near_index == (numx - 1))
		return(Y[numx - 1]);

	/* else we have to interpolate. We know that "near_index" contains the
	   index of the closest xval < "xval" as passed in here. */
	next_index = near_index + 1;

	timeabove = X[next_index];
	timebelow = X[near_index];

	percentdist = (*x_out - timebelow) / (timeabove - timebelow);
	voltincr = (Y[next_index] - Y[near_index]) * percentdist;
	return(Y[near_index] + voltincr);
}

gp_plot_t *
which_plot(x, y, x1, y1, set_number, decreasing)
	double x, y, *x1, *y1;
	int *set_number;
	int decreasing;
{
	int where;
	gp_plot_t *plot, *closest_yet;
	gp_xpt_t *scale;	
	double *xpts, tempy, closest_y_yet;
	double screen_dist();
	int i, set_size;
	double x_out, closest_x_yet;
	double xspan, yspan;

	closest_yet = NULL;
	scale = GP_currwin->xscale.scales;
	where = 0;
	while (scale != NULL) {
		xpts = scale->xpts;
		set_size = scale->run->num_X / scale->run->num_sets;
		where = hunt(xpts, set_size, x, where);
		xspan = ABS(GP_currwin->xscale.axis_left-GP_currwin->xscale.axis_right);
		yspan = ABS(GP_currwin->yscale.currmin - GP_currwin->yscale.currmax);

		/* Now let's look through all the plots which have this xpts as
		   their xpts.  Adjust "closest_yet" */
		plot = GP_currwin->plot_list;
		while (plot != NULL) {
			if (plot->xpts == scale) {
				if (closest_yet == NULL) {
					closest_yet = plot;
					point_on_plot(&closest_x_yet, &closest_y_yet, x, y, 
							where, xpts, plot->ypts, set_size, decreasing,
							xspan, yspan);
					*set_number = 0;
				}
				else {
					point_on_plot(&x_out, &tempy, x, y, where, xpts,
							plot->ypts, set_size, decreasing,
							xspan, yspan);
					if (screen_dist(x_out, tempy, x, y, xspan, yspan) < 
						screen_dist(closest_x_yet,closest_y_yet, x, y, 
									xspan, yspan)) {
						closest_yet = plot;
						closest_y_yet = tempy;
						closest_x_yet = x_out;
						*set_number = 0;
					}
				}
				if (scale->run->run_type == CHAR_CURVES) {
					for (i = 1; i < scale->run->num_sets; i++) {
						point_on_plot(&x_out, &tempy, x, y, where,
							&(xpts[set_size * i]), 
							&(plot->ypts[set_size * i]), set_size, decreasing,
							xspan, yspan);
						if (screen_dist(x_out, tempy, x, y, xspan, yspan) < 
							screen_dist(closest_x_yet, closest_y_yet, x, y, 
								xspan, yspan)) {
							closest_yet = plot;
							closest_y_yet = tempy;
							closest_x_yet = x_out;
							*set_number = i;
						}
					}
				}
			}
			plot = plot->next;
		}
		scale = scale->next;
	}
 	*y1 = closest_y_yet;
	*x1 = closest_x_yet;
	return(closest_yet);
}
				
which_point_on_plot(plot, x, x2, y2, set_size, set_number, decreasing)
	gp_plot_t *plot;
	double x, *x2, *y2;
	int set_number, set_size, decreasing;
{
	int where;

	where = hunt(plot->xpts->xpts, set_size, x, 0);

	if (set_number != 0)
		*y2 = gp_interp_y(x, x2, where, 
				&(plot->xpts->xpts[set_size * set_number]),
				&(plot->ypts[set_size * set_number]), set_size, decreasing);
	else *y2 = gp_interp_y(x, x2, where, plot->xpts->xpts, plot->ypts, 
				plot->numpts, decreasing);
}

/* we send in a point orig_x, orig_y and attempt to find the nearest
   point on the plot as represented by the lists X and Y.  First we
   find the point on the plot which has orig_x as it's x coordinate.
   Then we find the point which has orig_y as it's y coordinate.  We
   pick the one which is closest, sending back it's coords in "x" and "y".
*/
#define BETWEEN(_a, _b, _c) (((((_a) <= (_c)) && ((_b) >= (_c))) || (((_a) >= (_c)) && ((_b) <= (_c)))) ? 1 : 0)

point_on_plot(x, y, orig_x, orig_y, near_index, X, Y, num_elts, decreasing,
		xspan, yspan)
	double *x, *y; /* point on the plot nearest orig_x, orig_y */
	double orig_x, orig_y;
	int near_index; /* contains the index of the closest xval < "orig_x" in
						the X array. */
	double *X, *Y;
	int decreasing, num_elts;
	double xspan, yspan;
{
	double x1, y1, x2, y2;
	int f_index, b_index;
	double percentdist, fx, bx;
	double screen_dist();

	y1 = gp_interp_y(orig_x, &x1, near_index, X, Y, num_elts, decreasing);
	x1 = orig_x;

	/* we need to find what x would be so that orig_y would be on the plot.
	   Search forward from near_index to find first place where plot crosses
	   orig_y value (or we run off end of data).  Ditto for searching
	   backward.  Pick the crossing with x value closest to orig_x */

	f_index = near_index;
	while ((f_index != num_elts - 1) && 
			(! BETWEEN(Y[f_index], Y[f_index+1], orig_y)))
		f_index++;

	b_index = near_index;
	while ((b_index > 0) &&
			(! BETWEEN(Y[b_index], Y[b_index-1], orig_y)))
		b_index--;

	if (f_index != (num_elts-1)) { /*found a crossing when searching forward*/
		percentdist = (orig_y - Y[f_index]) / (Y[f_index+1] - Y[f_index]);
		fx = X[f_index] + ((X[f_index+1] - X[f_index]) * percentdist);
	}
	if (b_index != 0) { /* found a crossing when searching backward */
		percentdist = (orig_y - Y[b_index-1]) / (Y[b_index] - Y[b_index-1]);
		bx = X[b_index-1] + ((X[b_index] - X[b_index-1]) * percentdist);
	}

	if ((f_index != (num_elts-1)) && (b_index != 0)) { 
		/* so pick the crossing which is closest to orig_x */
		if ((ABS(orig_x - bx)) > (ABS(orig_x - fx))) {
			x2 = fx;
			y2 = orig_y;
		}
		else {
			x2 = bx;
			y2 = orig_y;
		}
	}
	else if (f_index != (num_elts-1)) {
		x2 = fx;
		y2 = orig_y;
	}
	else if (b_index != 0){
		x2 = bx;
		y2 = orig_y;
	}	
	else {
		*x = x1;
		*y = y1;
		return;
	}

	/* Now, finally, we can pick either x2,y2 or x1,y1 and send it back--
	   whichever point is closest to orig_x, orig_y */
	if (screen_dist(orig_x, orig_y, x1, y1, xspan, yspan) < 
		screen_dist(orig_x, orig_y, x2, y2, xspan, yspan)) {
		*x = x1;
		*y = y1;
	}
	else {
		*x = x2;
		*y = y2;
	}
}
