/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "gp.h"

int GP_currwin_id; /* window from which we are currently processing a command */
gp_win_t *GP_currwin; /* pointer to current window's info */

/* called from interact.c when screenscrewed event received so that we might
   resize labels if font size has changed. */
gp_resize_labels(win_id)
	int win_id;
{
	gp_win_t *win, *win_struct();
	double new_ht, wid, ht, x, y;
	gp_label_t *label;
	double graph_str_width(), graph_str_height();

	win = win_struct(win_id);
	if (win->label_list == NULL)
		return;

	label = win->label_list;
	new_ht = graph_str_height();
	
	if (new_ht == (label->y2 - label->y1))
		return;

	while (label != NULL) {
		x = label->cx;
		y = label->cy;
		wid = graph_str_width(label->text)/2.0;
		ht = new_ht/2.0;
		label->x1 = x - wid;
		label->y1 = y - ht;
		label->x2 = x + wid;
		label->y2 = y + ht;
	
		label = label->next;
	}
}

gp_add_label(x, y, text)
	double x, y; /* center point of text */
	char *text;
{
	double wid, ht;
	double graph_str_width(), graph_str_height();
	gp_label_t *lastlabel, *newlabel;

	/* first, allocate space for new label and attach it */
	lastlabel = GP_currwin->label_list;
	while ((lastlabel != NULL) && (lastlabel->next != NULL))
        lastlabel = lastlabel->next;

	newlabel = (gp_label_t *) malloc (sizeof(gp_label_t));
	newlabel->prev = lastlabel;
	newlabel->next = NULL;
	if (lastlabel != NULL)
		lastlabel->next = newlabel;
	else GP_currwin->label_list = newlabel;

	newlabel->text = (char *) calloc (strlen(text) + 1, sizeof(char));
	strcpy(newlabel->text, text);
	wid = graph_str_width(text)/2.0;
	ht = graph_str_height()/2.0;
	newlabel->x1 = x - wid;
	newlabel->y1 = y - ht;
	newlabel->x2 = x + wid ;
	newlabel->y2 = y + ht;

	newlabel->cx = x;
	newlabel->cy = y;
}

gp_delete_label(deletelabel)
	gp_label_t *deletelabel;
{
	if (deletelabel->prev != NULL)
		deletelabel->prev->next = deletelabel->next;
	if (deletelabel->next != NULL)
		deletelabel->next->prev = deletelabel->prev;
	if (deletelabel == GP_currwin->label_list)
		GP_currwin->label_list = deletelabel->next;
	free((char *)deletelabel->text);
	free((char *)deletelabel);
}

gp_move_label(movelabel, x, y)
	gp_label_t *movelabel;
	double x, y;
{
	double wid, ht;

	wid = (movelabel->x2 - movelabel->x1)/2.0;
	ht =  (movelabel->y2 - movelabel->y1)/2.0;

	movelabel->x1 = x - wid;
	movelabel->y1 = y - ht;
	movelabel->x2 = x + wid;
	movelabel->y2 = y + ht;

	movelabel->cx = x;
	movelabel->cy = y;
}

/* given an x,y point, find which label in the current picture is underneath
   this point.  Return a pointer to it.  Return NULL if no label under
   x,y.  Also return the center point of the label as well as it's message. */
	  
gp_label_t *
gp_find_label(x, y, cenx, ceny, text)
	double x, y;
	double *cenx, *ceny;
	char **text;
{
	gp_label_t * label;

	label = GP_currwin->label_list;

	while ((label != NULL) &&
			(!	((x >= label->x1) && (x <= label->x2) &&
				 (y >= label->y1) && (y <= label->y2)))) {
		label = label->next;
	}
	if (label == NULL)
		return(NULL);

	*text = label->text;
	*cenx = label->cx;
	*ceny = label->cy;
	return(label);
}


/* Prompts for which command is desired: "add", "move", or "delete".  Then
   prompts for the necessary info for the operation.  Finally the screen is
   adjusted and the database is adjusted to reflect the change. */
gp_label()
{
	double x,y;
	int popup;
	char graph_await_event();
	int win_id, val;

	popup = graph_create_fixed_win(0.0, 0.0, 600.0, 44.0, 
		0.0, 0.0, 600.0, 40.0);
	if (popup == -1) 
		return;

	graph_set_drawing(popup);
	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 40.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(100.0, 20.0, "Add");
	graph_text(300.0, 20.0, "Delete");
	graph_text(500.0, 20.0, "Move");
	graph_line(0.0, 40.0, 600.0, 40.0);
	graph_line(200.0, 0.0, 200.0, 40.0);
	graph_line(400.0, 0.0, 400.0, 40.0);

	/* now wait for user to pick one of the three functions. A hit
	   outside the menu area will cancel the whole function. */
	graph_set_input(popup, 0, "b");
	graph_await_event(&win_id, &val, &x, &y);
	if (win_id != popup) {
		graph_end_win(popup);
		graph_set_drawing(GP_currwin_id);
		graph_set_input(GP_currwin_id, 0, "bk");
		return;
	}
	if (x < 200.0)  	/* then "Add" was selected */
		add_label(popup);
	else if (x < 400.0) 
		delete_label();
	else move_label(popup);

	graph_end_win(popup);
	graph_set_drawing(GP_currwin_id);
	graph_set_input(GP_currwin_id, 0, "bk");
}

add_label(popup)
	int popup;
{
	char text[BLOCKSIZE];
	double x, y;
	int win_id, val;

	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 40.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_line(0.0, 40.0, 600.0, 40.0);
	graph_char_just_set("ll");
	graph_text(1.0, 30.0, "Label to be added:");

	graph_set_input(popup, 0, "bk");
	graph_await_event(&win_id, &val, &x, &y);

	if (win_id != popup)
		return;

	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_line(0.0, 40.0, 600.0, 40.0);

	if (! read_and_echo(popup, (char)val, text))
		return;

	if (drag_label(popup, &x, &y, text)) {
		gp_add_label(x, y, text);
		graph_draw_mode(0);
		graph_set_layer(2); /*textoverbkgd*/
		graph_char_just_set("cc");
		graph_text(x, y, text);
	}
}
	
static
delete_label()
{
	int win_id, val;
	char graph_await_event();
	gp_label_t *label;
	double x, y, cenx, ceny;
	char *text;

	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 40.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_line(0.0, 40.0, 600.0, 40.0);
	graph_char_just_set("cc");
	graph_text(300.0, 30.0, "Hit mouse button over");
	graph_text(300.0, 10.0, "label to be deleted");

	graph_cursor("pointer");
	graph_set_input(GP_currwin_id, 0, "b");
	graph_set_drawing(GP_currwin_id);
	graph_await_event(&win_id, &val, &x, &y);

	/* if user makes a hit outside of GP_currwin or NOT over a label,
	   we don't do anything. */
	if ((win_id == GP_currwin_id) &&
		((label = gp_find_label(x, y, &cenx, &ceny, &text)) != NULL)) {
/*		graph_draw_mode(1);
		graph_set_layer(10); */
		graph_draw_mode(0);
		graph_set_layer(0);
		graph_text(cenx, ceny, text);
		gp_delete_label(label);
/*		graph_draw_mode(0); */
	}
}

move_label(popup)
	int popup;
{
	char *text;
	gp_label_t *label;
	double x, y, cenx, ceny;
	int eventwin_id, val;
	char graph_await_event();

	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 40.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_line(0.0, 40.0, 600.0, 40.0);
	graph_char_just_set("cc");
	graph_text(300.0, 30.0, "Hit mouse button over");
	graph_text(300.0, 10.0, "label to be moved");

	graph_set_drawing(GP_currwin_id);
	graph_set_input(GP_currwin_id, 0, "b");
	graph_cursor("pointer");
	graph_await_event(&eventwin_id, &val, &x, &y);

	if ((eventwin_id != GP_currwin_id) ||
		((label = gp_find_label(x, y, &cenx, &ceny, &text)) == NULL)) 
		return;

	graph_draw_mode(1); /* undraw label in its present position */
	graph_text(cenx, ceny, text);

	if (drag_label(popup, &x, &y, text)) {
		gp_move_label(label, x, y);
		graph_draw_mode(0);
		graph_set_layer(2);
		graph_text(x, y, text);
	}
}

static int
drag_label(popup, x, y, text)
	int popup;
	double *x, *y;
	char *text;
{
	int eventwin_id, val;
	char graph_await_event(), ev;
	double lastx, lasty;

	graph_set_drawing(popup);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 40.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_line(0.0, 40.0, 600.0, 40.0);
	graph_char_just_set("cc");
	graph_text(300.0, 20.0, "Place Label With Cursor");

	graph_set_drawing(GP_currwin_id);
	graph_set_input(GP_currwin_id, 1, "bm");
	graph_set_layer(10);
	graph_cursor("invisible");
	lastx = 0.0; lasty = 0.0;

	ev = graph_await_event(&eventwin_id, &val, x, y);
	if ((ev == 'm') && (eventwin_id == GP_currwin_id)){
		graph_draw_mode(0);
		graph_text(*x, *y, text);
		lastx = *x; lasty = *y;

		while (1) {
			ev = graph_await_event(&eventwin_id, &val, x, y);
			if ((ev == 'm') && (eventwin_id == GP_currwin_id)){
				graph_draw_mode(1);
				graph_text(lastx, lasty, text);
				graph_draw_mode(0);
				graph_text(*x, *y, text);
				lastx = *x; lasty = *y;
			}
			else if (ev == 'b') {
			/*	if (eventwin_id != GP_currwin_id) { */
					graph_draw_mode(1);
					graph_text(lastx, lasty, text); 
			/*	} */
				graph_set_input(GP_currwin_id, 0, "bk");
				graph_cursor("pointer");
				if (eventwin_id != GP_currwin_id) 
					return(0);
				else return(1);
			}
		}
	}
	return(0);
}
