/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "gp.h"
#include "ctype.h"

extern comm_file_t *Sig_files;
extern int Sig_interactive_mode;
extern gp_win_t *GP_currwin;
extern int GP_currwin_id;
extern int GP_mode;

extern void gp_xrange(), gp_yrange();

gp_parse_and_execute(cmd)
	char cmd[];
{
	char *cmdname, *next_token();

	if (!Sig_interactive_mode) 
		sig_msg("Command: ", cmd, "");

	cmdname = next_token(&cmd);
	if (cmdname[0] == '\0')
		return;

	if (strncmp(cmdname, "addplot", 2) == 0) 
		addplot_p(cmd, 1);
	else if (strncmp(cmdname, "ft", 3) == 0)
		ft_p(cmd);
	else if (strncmp(cmdname, "analyze", 3) == 0)
		gp_analyze();
	else if (strncmp(cmdname, "grid", 2) == 0)
		gp_grid();
	else if (strncmp(cmdname, "help", 1) == 0)
		help(cmd);
	else if (strncmp(cmdname, "load", 4) == 0)
		load_p(cmd);
	else if (strncmp(cmdname, "logx", 4) == 0)
		gp_logx();
	else if (strncmp(cmdname, "logy", 4) == 0)
		gp_logy();
	else if (strncmp(cmdname, "precision", 2) == 0)
		gp_precision();
	else if (strncmp(cmdname, "quit", 1) == 0)
		sigview_quit();
	else if (strncmp(cmdname, "redraw", 2) == 0)
		gp_draw_screen(GP_currwin_id);
	else if (strncmp(cmdname, "run", 2) == 0)
		run_p(cmd);
	else if (strncmp(cmdname, "transient", 2) == 0)
		open_other_mode();
	else if (strncmp(cmdname, "snapshot", 2) == 0)
		snapshot_p(cmd);
	else if (strncmp(cmdname, "show", 2) == 0) 
		gp_show_p(cmd);
	else if (strncmp(cmdname, "compare", 2) == 0) 
		gp_show_p(cmd);
	else if (strncmp(cmdname, "xaxis", 2) == 0) {
		gp_xaxis(cmd);
		if (Sig_interactive_mode)
			gp_draw_screen(GP_currwin_id);
	}
	else if (strncmp(cmdname, "xrange", 2) == 0) 
		range_p(cmd, gp_xrange);
	else if (strncmp(cmdname, "yrange", 2) == 0) 
		range_p(cmd, gp_yrange);
	else if (strncmp(cmdname, "format", 1) == 0) {
                format_p(cmd);
		if (Sig_interactive_mode)
                        gp_draw_screen(GP_currwin_id);
	}
	else if (strncmp(cmdname, "ytics", 3) == 0) 
		ytics(cmd);
	else if (strncmp(cmdname, "xtics", 3) == 0) 
		xtics(cmd);
	else sig_msg("Command ", cmdname, " not identified--at least not for this mode");
}

addplot_p(expression, draw)
	char *expression;
	int draw;
{
	double *ypts;
	int numpts, manufactured;
	char nospace_expr[256]; 
	run_t *run;

	if (GP_currwin == NULL)
		return;

	if (Sig_files == NULL) { 
		sig_msg("No files loaded from which to create plot","","");
		return;
	}

	if (! gp_parse_expression (NULL, &run, expression, &ypts, 
			&numpts, &manufactured, 1))
		return;

	remove_whitespace(expression, nospace_expr);  
	addplot(run, ypts, numpts, manufactured, nospace_expr);

	if (draw && Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

static
load_p(arglist)
	char *arglist;
{
	char *filenm, *next_token();

	filenm = next_token(&arglist);
	while (filenm[0] != '\0') {
		load_simfile(filenm);
		filenm = next_token(&arglist);
	}
	/* update the global xscale and yscale max and min values */

	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

static
run_p(arglist)
	char *arglist;
{
	char *cmdfilenm, *next_token();
	FILE *cmdfile, *open_fl();
	char cmd[BUFSIZE];

	cmdfilenm = next_token(&arglist);
	while (cmdfilenm[0] != '\0') {
		if ((cmdfile = open_fl(cmdfilenm, "r")) != NULL) {
			while (get_a_cmd(cmdfile, cmd)) {
				if (GP_mode)
					gp_parse_and_execute(cmd);
				else parse_and_execute(cmd);
			}
		}
		cmdfilenm = next_token(&arglist);
	}
}


static char YY_expression[256]; /* global necessary for yacc parser to find buffer */
static char *YY_ptr;
run_t *Desired_run;  /* global necessary for yacc parser to find run */
struct vvec_s Result; /* results of parsing an expression from yacc */
int Print_errors;

/* Returns 1 if expression can be parsed successfully using vectors from
   the indicated "run".  Will pass back pointers to all the relevant info
   from the satisfying vector. */
int
gp_parse_expression (desired_run, run, expr, values, numpts, manufactured,
	print_errors)
	run_t *desired_run; /* the run from which we may want to extract 
		data for this expr.  Could also be NULL if we want to rely on
		f2/v2 notation or on GP_currwin->curr_run to tell us where to
		get our data. */
	run_t **run; /* the run from which we extracted data for this expr */
	char *expr; /* the expression which STILL CONTAINS WHITESPACE (if typed)*/
	double **values; /* the resulting array */
	int *numpts; /* number of points in the x and y arrays */
	int *manufactured; /* did we allocate new space for this or are we
		simply pointing to an already existent set of data? */
	int print_errors; /* do we want to print error msgs? */
{
	if (*expr == NULL)
		return(0);

	strcpy(YY_expression, expr);
	YY_ptr = YY_expression;
	Desired_run = desired_run;
	Print_errors = print_errors;

	yyparse();
	if (Result.num_vals == 0)
		return(0);
	else {
		*run = Result.run;
		*values = Result.values;
		*numpts = Result.num_vals;
		*manufactured = Result.manufactured;
		return(1);
	}

/*	if (yyparse() == 0) {
		*run = Result.run;
		*values = Result.values;
		*numpts = Result.num_vals;
		*manufactured = Result.manufactured;
		return(1);
	}
	return(0);
*/
}

/* remove all whitespace */
remove_whitespace(expr, cleaned)
	char *expr, *cleaned;
{
	int i, j;

	i = 0; j = 0;
	while (expr[i] != '\0') {
		if (! isspace(expr[i])) {
			cleaned[j] = expr[i];
			j++;
		}
		i++;
	}
	cleaned[j] = '\0';
}
/* remove all excess whitespace, leaving one blank in place of each
   group of blanks. */
remove_excess_whitespace(expr, cleaned)
	char *expr, *cleaned;
{
	int i, j;
	int spacect;

	spacect = 0;
	i = 0; j = 0;
	while (expr[i] != '\0') {
		if (! isspace(expr[i])) {
			cleaned[j] = expr[i];
			j++;
			spacect = 0;
		}
		else if (spacect == 0) {
			cleaned[j] = ' ';
			j++;
			spacect = 1;
		}	
		i++;
	}
	cleaned[j] = '\0';
}

/* for the lex lexical analyzer, here is a routine it calls to get the
   next character. */
int
get_sig_char()
{
	return((int)*YY_ptr++);
}
