/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include <stdio.h>
#include "gp.h"

extern comm_file_t *Sig_files;
extern int Sig_main;

extern gp_win_t *GP_wins;
extern gp_win_t *GP_currwin;
extern int GP_currwin_id;
extern int GP_mode;

int GP_grid_on;

/* draw a window of the general mode. */

gp_draw_screen(win)
	int win;
{
	graph_set_layer(0);
	graph_draw_mode(0);
	if (graph_set_drawing(win)) {
		graph_rect(0.0, 0.0, 600.0, 500.0);
		gp_menu_area(win);
		gp_runs_area(win);
		gp_draw_pic(win);
	}
}

gp_runs_area(win)
	int win;
{
	comm_file_t *fileptr;
	run_t *runptr;
	char buffer[BUFSIZE];
	double pos, graph_str_width();
	gp_win_t *win_ptr, *win_struct();

	if (! graph_set_drawing(win))
		return;
	win_ptr = win_struct(win);

	graph_set_layer(0);
	graph_draw_mode(0);
	graph_rect(0.0, 48.0, 600.0, 60.0);
	graph_set_layer(2);
	graph_draw_mode(0);
	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_text(22.0, 50.0, "Runs: ");

	fileptr = Sig_files;
	pos = 22.0 + graph_str_width("Runs: ");
	while (fileptr != NULL) {
		if (! DIGITAL(fileptr->type)) {

			runptr = fileptr->runs;
			while (runptr != NULL) {
				buffer[0] = '\0';
				if (win_ptr->curr_run == runptr)
					strcat(buffer, "[");
				strcat(buffer, runptr->name);
				if (win_ptr->curr_run == runptr)
					strcat(buffer, "] ");
				else strcat(buffer, " ");
				graph_set_layer(runptr->layer);
				graph_text(pos, 50.0, buffer);
				pos = pos + graph_str_width(buffer);
				runptr = runptr->next;
			}
		}
		fileptr = fileptr->next;
	}
}

gp_menu_area(win)
	int win;
{
	if (! graph_set_drawing(win))
		return;
	graph_draw_mode(0);
	graph_set_layer(4);
	graph_rect(0.0, 0.0, 100.0, 22.0);
	graph_set_layer(7);
	graph_rect(100.0, 0.0, 200.0, 22.0);
	graph_set_layer(9);
	graph_rect(200.0, 0.0, 300.0, 22.0);
	graph_set_layer(6);
	graph_rect(300.0, 0.0, 400.0, 22.0);
	graph_set_layer(8);
	graph_rect(400.0, 0.0, 500.0, 22.0);
	graph_set_layer(4);
	graph_rect(500.0, 0.0, 600.0, 22.0);

	graph_set_layer(5);
	graph_rect(0.0, 22.0, 100.0, 44.0);
	graph_set_layer(9);
	graph_rect(100.0, 22.0, 200.0, 44.0);
	graph_set_layer(5);
	graph_rect(200.0, 22.0, 300.0, 44.0);
	graph_set_layer(7);
	graph_rect(300.0, 22.0, 400.0, 44.0);
	graph_set_layer(9);
	graph_rect(400.0, 22.0, 500.0, 44.0);
	graph_set_layer(8);
	graph_rect(500.0, 22.0, 600.0, 44.0);

	graph_char_just_set("cc");
	graph_set_layer(3);
	graph_text(50.0, 11.0, "Track");
	graph_text(150.0, 11.0, "Del From Plot");
	graph_text(250.0, 11.0, "Slope");
	graph_text(350.0, 11.0, "Zoom Reset");
	graph_text(450.0, 11.0, "Create Win");
	graph_text(550.0, 11.0, "Quit Sigview");

	graph_text(50.0, 33.0, "Text");
	graph_text(150.0, 33.0, "Show");
	graph_text(250.0, 33.0, "Snapshot");
	graph_text(350.0, 33.0, "Zoom In");
	graph_text(450.0, 33.0, "Redraw");
	graph_text(550.0, 33.0, "Quit Win");
}

/*	This is called from only one place:  at startup of Sigview where we've
	discovered that we've loaded only AC or TRANSFER runs and there's no
	command file to tell us what to do.  We're going to fill a plot window 
	structure so that when we've graph_dev_initted we'll automatically open 
	a plot window and fill it by drawing each of the columns of data 
	(vectors) into it. */

gp_make_default_axes() 
{
	comm_file_t *file_ptr;
	run_t *run_ptr;
	gp_xscale_t *xscale;
	gp_yscale_t *yscale;
	gp_plot_t *plist;
	double global_min, global_max, min, max;
	double *d;
	int i, j;
	gp_xpt_t *find_xscale();

	xscale = &(GP_currwin->xscale);
	yscale = &(GP_currwin->yscale);

	/* no first column in any run would make a valid x scale */
	if (xscale->expression == NULL)
		return;

	file_ptr = Sig_files;
	plist = NULL;
	while (file_ptr != NULL) {
		run_ptr = file_ptr->runs;
		while (run_ptr != NULL) {
		  if (run_ptr->run_type != DIGITL) {

			if (run_ptr->run_type == AC) 
				gp_logx();
			
			if (xscale->scales != NULL) {
			for (i = 1; i < run_ptr->num_Y; i++) {
				if (plist == NULL) {
					GP_currwin->plot_list=
						(gp_plot_t *)malloc(sizeof(gp_plot_t));
					plist = GP_currwin->plot_list;
					global_min = global_max = run_ptr->data.analog[i][0];
					GP_currwin->curr_run = run_ptr;
				}
				else {
					plist->next = (gp_plot_t *) malloc (sizeof(gp_plot_t));
					plist = plist->next;
				}
				plist->next = NULL;
				plist->numpts = run_ptr->num_X;
				plist->layer = assign_layer();
				plist->marked_for_death = 0;
				plist->closest_to_zero = -1;
				plist->expression=(char *)calloc(strlen(run_ptr->Ynames[i])+1, 
						sizeof(char));
				strcpy(plist->expression, run_ptr->Ynames[i]);
				plist->xpts = find_xscale(run_ptr);
	
				min = max = run_ptr->data.analog[i][0];	
				plist->ypts = (double *)calloc(plist->numpts, sizeof(double));

				d = run_ptr->data.analog[i];
				for (j = 0; j < plist->numpts; j++) {
					plist->ypts[j] = d[j];
					if (d[j] > max)
						max = d[j];
					if (d[j] < min)
						min = d[j];
				}
				plist->ymin = min; plist->ymax = max;
				if (global_min > min)
					global_min = min;
				if (global_max < max)
					global_max = max;
			}
			}
			}
			run_ptr = run_ptr->next;
		}
		file_ptr = file_ptr->next;
	}
	yscale->log = 0;
	yscale->min = yscale->currmin = global_min;
	yscale->max = yscale->currmax = global_max;
	xscale->axis_left = xscale->data_left;
	xscale->axis_right = xscale->data_right;
	GP_currwin->bar_plot = 0;
}

/* 
   Called from rdanalog.c or rdraw.c when a new run has been successfully 
   loaded.  Go through every GP window and fix the xscale structure of
   each.  This entails looking for the run name in each scales->run
   until we find a match.  If no match, we create a new scales struct
   for our new run.

   In addition, update GP_currwin->curr_run, if it is still null.
*/
gp_update_xscales(run_ptr)
	run_t *run_ptr;
{
	gp_win_t *win;
	double *new_xpts;
	int numpts, made_up;
	gp_xpt_t *add_a_scale();
	char expr[SIGNMSIZE];
	run_t *run;

	win = GP_wins;
	while (win != NULL) {
		if (win->xscale.expression == NULL) 
			sprintf(expr, "\'%s\'", run_ptr->Ynames[0]);
		else strcpy(expr, win->xscale.expression);
		if (gp_parse_expression(run_ptr, &run, expr, &new_xpts, 
				&numpts, &made_up, 0) &&
			valid_scale(run_ptr, new_xpts, numpts, 
				&(win->xscale), expr, made_up)) 

			add_a_scale(run, new_xpts, expr, numpts, made_up, &(win->xscale));

		/* initialize curr_run, if necessary */
		if (win->curr_run == NULL)
			win->curr_run = run;

		win = win->next;
	}
}


/* Check that there are at least two points in the proposed xscale, that
   the points are monotonic, and, if "match_existing" is set, that they 
   increase or decrease like all the other scales for this window.  */
int
valid_scale(run, xpts, numpts, xscale, expression, made_up)
	double *xpts;
	run_t *run;
	int numpts, made_up;
	gp_xscale_t *xscale;
	char *expression;
{
	int valid;

	valid = 1;

	if (numpts < 2) {
		sig_msg("Xscale ",expression," produces too few points");
		sig_msg("from this run: ", run->name, "");
		valid = 0;
	}

	/* we want to allow for the plotting of curve families.  e.g. where 
	   the first column in a run goes 0 1 2 3 4 5 0 1 2 3 4 5....  So we 
	   need to check for monotonicity in the first grouping and ASSUME
	   that the rest are ok.  Obviously an inferior approach. */

	else if ((run->run_type == CHAR_CURVES) && 
			 (! monotonic(xpts, numpts/run->num_sets))) {
		sig_msg("Non-monotonic xscale: ", expression, run->name);
		valid = 0;
	}	
	else if ((run->run_type != CHAR_CURVES) && (! monotonic(xpts, numpts))) {
		sig_msg("Non-monotonic xscale: ", expression, run->name);
		valid = 0;
	}
	else {
		if (xscale->increasing == -1) { /* xscale has not yet been set, so
											set it */
			if (xpts[0] < xpts[numpts - 1])
				xscale->increasing = 1;
			else xscale->increasing = 0;
		}
		if (xscale->increasing && (xpts[0] > xpts[numpts - 1])) {
			sig_msg("Existing xscale runs in opposite direction from: ",
				expression, run->name);
			valid = 0;
		}
		else if (! xscale->increasing && (xpts[0] < xpts[numpts - 1])) {
			sig_msg("Existing xscale runs in opposite direction from: ",
				expression, run->name);
			valid = 0;
		}
	}

	if (!valid) {
		if (made_up && (xpts != NULL))
			free((char *)xpts);
		return(0);
	}
	else return(1);
}

int
monotonic(xpts, numpts)
	double *xpts; /* the set of x points */
	int numpts;
{
	int i; 

	if (numpts <= 1)
		return(0);

	/* check to see that this xscale is monotonic */
	if (xpts[0] < xpts[1]) { /* hopefully, monotonically increasing */
		i = 1;
		while ((i < numpts-1) && (xpts[i] <= xpts[i+1]))
			i++;
	}
	else { /* hopefully, monotonically decreasing */
		i = 1;
		while ((i < numpts-1) && (xpts[i] >= xpts[i+1]))
			i++;
	}

	if (i < numpts-1)  /* not monotonic */
		return(0);
	else if (xpts[0] == xpts[numpts - 1]) 
		return(0);
	else return(1);
}

/* Called solely by "addplot" and "make default axes" when a new plot 
   being added to the current window.  Returns a pointer to the correct 
   xscale, or NULL if one cannot be found. */
gp_xpt_t *
find_xscale(run)
	run_t *run;
{
	gp_xpt_t *xscales, *change_xscale_expr();
	double *new_xpts;
	int numpts, made_up;
	gp_xpt_t *add_a_scale();
	char expr[SIGNMSIZE];
	run_t *run_ptr;

	if (run == NULL)
		run = GP_currwin->curr_run;
	xscales = GP_currwin->xscale.scales;

	/* If no scale yet defined for this window, then let's assign the
	   default scale--the first column of data in this run. */
	if (xscales == NULL) {
		sprintf(expr, "\'%s\'", run->Ynames[0]);
		xscales = change_xscale_expr(run, run->data.analog[0],
				expr, run->num_X, 0, GP_currwin);
		return(xscales);
	}

	/* Else search for an xscale matching this run */
	while ((xscales != NULL) && (run != xscales->run))
		xscales = xscales->next;

	if (xscales != NULL) 
		return(xscales);

	/* Okay so let's try to manufacture an xscale for this run using
	   the current expression for this window's xscale */
	if (GP_currwin->xscale.expression == NULL)
		sprintf(expr, "\'%s\'", run->Ynames[0]);
	else strcpy(expr, GP_currwin->xscale.expression);

	if (gp_parse_expression(run, &run_ptr, expr, &new_xpts, 
				&numpts, &made_up, 1) &&
		valid_scale(run, new_xpts, numpts, 
				&(GP_currwin->xscale), expr, made_up)) 

		xscales = add_a_scale(run_ptr, new_xpts, expr, numpts, 
				made_up, &(GP_currwin->xscale));

	return(xscales);
}

static gp_xpt_t *
add_a_scale(run_ptr, xpts, expression, numpts, made_up, xscale) 
	run_t *run_ptr; /* the run from which this set of x pts comes */
	double *xpts; /* the set of x points */
	char *expression;
	int numpts, made_up;
	gp_xscale_t *xscale;
{
	gp_xpt_t *scale, *previous;
	int i;

	scale = xscale->scales;
	previous = NULL;	
	while ((scale != NULL) && (scale->run != run_ptr)) {
		previous = scale;
		scale = scale->next;
	}

	if (scale == NULL) {
		/* We're entering an xscale for a run that never contributed a scale
			before. */
		if (xscale->scales == NULL) { /* and this window never had an xscale */
			xscale->scales = scale =(gp_xpt_t *)malloc(sizeof(gp_xpt_t));
			xscale->axis_left = xscale->data_left = xpts[0];
			xscale->axis_right = xscale->data_right = xpts[numpts - 1];
			xscale->expression = (char *)calloc(strlen(expression) + 1,
				sizeof(char));
			strcpy(xscale->expression, expression);
		}
		else { /* this window has scales for other runs, but not this one */
			scale = previous->next = (gp_xpt_t *) malloc (sizeof(gp_xpt_t));
		}
		scale->run = run_ptr;
		scale->next = NULL;
	}
	else { /* we're changing xscale's expression for this run */
		free ((char *)scale->xpts);
		scale->xpts = (double *)calloc(numpts, sizeof(double));
	}

	if (made_up)
		scale->xpts = xpts;
	else {
		scale->xpts = (double *)calloc(numpts, sizeof(double));
		for (i = 0; i < numpts; i++)
			scale->xpts[i] = xpts[i];
	}

	/* Now we need to update the xscale structure itself */
	if (xscale->increasing) {
		if (scale->xpts[0] < xscale->data_left)
			xscale->data_left = scale->xpts[0];
		if (scale->xpts[numpts - 1] > xscale->data_right)
			xscale->data_right = scale->xpts[numpts - 1];
	}
	else {
		if (scale->xpts[0] > xscale->data_left)
			xscale->data_left = scale->xpts[0];
		if (scale->xpts[numpts - 1] < xscale->data_right)
			xscale->data_right = scale->xpts[numpts - 1];
	}
	return(scale);
}


gp_xpt_t *
change_xscale_expr(run_ptr, xpts, expression, numpts, made_up, win)
	run_t *run_ptr; /* the run from which this set of x pts comes */
	double *xpts; /* the set of x points */
	char *expression;
	int numpts, made_up;
	gp_win_t *win;
{
	if (xpts == NULL)
		return(NULL);

	if (! valid_scale(run_ptr, xpts, numpts, 
				&(win->xscale), expression, made_up))
		return(NULL);

	return(add_a_scale(run_ptr, xpts, expression, numpts, made_up, 
		&(win->xscale)));
}

gp_draw_pic(win)
	int win;
{
	gp_win_t *winptr, *picwin;

	/* first search for the GP_win which has "win" for its window id. */
	winptr = GP_wins;
	picwin = NULL;
	while ((picwin == NULL) && (winptr != NULL)) {
		if (winptr->id == win)
			picwin = winptr;
		winptr = winptr->next;
	}

/*	
	 we want a square aspect ratio for our plot, so figure the 
	   corners of the plot now. 
	graph_shortest_dim(5.0, 50.0, 595.0, 495.0, &square_x, &square_y);
	xoffset = (595.0 - square_x)/2.0;
	yoffset = (495.0 - square_y)/2.0;
	x1 = 5.0 + xoffset; x2 = 595.0 - xoffset;
	y1 = 50.0 + yoffset; y2 = 495.0 - yoffset;
	draw_scales(picwin->xscale, picwin->yscale, x1, y1, x2, y2);
	draw_plots(picwin->plot_list, x1, y1, x2, y2);
*/
	if (picwin->plot_list != NULL) {
		draw_scales(picwin->xscale, picwin->yscale);
		draw_plots(picwin->plot_list, picwin->xscale, 
			picwin->yscale, win, picwin->bar_plot);
		draw_header();
	}
	draw_labels(picwin->label_list);
}


/* We assume that GP_currwin is the win we're drawing to, and that it
   has been set to 0,0,600,500.  */

draw_labels(labels)
	gp_label_t *labels;
{
	gp_label_t *label;

	if (labels == NULL)
		return;

	graph_set_layer(2);
	graph_font_set("medium");
	graph_char_just_set("ll");

	label = labels;
	while (label != NULL) {
		graph_text(label->x1, label->y1, label->text);
		label = label->next;
	}
}

draw_scales(xscale, yscale)
	gp_xscale_t xscale;
	gp_yscale_t yscale;
{
	double incr, scale_incr;
	double middle, graph_str_height();
	int precision, idum;
	double left, right, min, max, pow(), floor();
	int ix, ynumtics, xnumtics;
	int nt;
	double ytval, xtval, logincr;

	graph_set_layer(2);
	graph_font_set("small");

	precision = GP_currwin->precision;
	
	/* wrr - the y scale */

	min = yscale.currmin;
	max = yscale.currmax;
	ynumtics = yscale.numtics;
	if ( yscale.log ) ynumtics = max - min + 1 ;
	incr = (YTOP - YBOT)/(ynumtics-1);
	scale_incr = (max - min)/(ynumtics-1);

	graph_set_line_width(4);
	graph_line(XLEFT, YBOT, XLEFT, YTOP);
	graph_line(XRIGHT, YBOT, XRIGHT, YTOP);
	graph_char_just_set("rc");
        for ( nt=0; nt<ynumtics; nt++ ) {
		if( yscale.log ) {
		   ytval = pow(10.0,min+nt*scale_incr);
		} else {
		   ytval = min+nt*scale_incr;
		}
	   graph_text_double(XLEFT - 1.0, YBOT + nt*incr , ytval, precision);
	   if( GP_grid_on ) {
	      graph_set_layer(9);
	      if ( nt > 0 && nt < (ynumtics-1) )
	      graph_set_line_width(-1);
	      graph_line(XLEFT, YBOT + nt*incr, XRIGHT, YBOT + nt*incr);
	      if( yscale.log && nt<ynumtics-1 ) {
		 for ( idum=2; idum<=9 ; idum=idum+1 ) {
		   logincr = idum;
	           graph_set_line_width(-1);
		   graph_line(XLEFT, YBOT + nt*incr + incr*LOGTEN(logincr), XRIGHT, YBOT + nt*incr + incr*LOGTEN(logincr) );
		 }
	      }
	   }
	   graph_set_layer(2);
	}

	/* wrr - the x scale */

	left = xscale.axis_left;
	right = xscale.axis_right;
	xnumtics = xscale.numtics;
	if (xscale.log) xnumtics = right - left + 1;
	incr = (XRIGHT - XLEFT)/(xnumtics-1);
	scale_incr = (right - left)/(xnumtics-1);

	graph_set_line_width(4);
	graph_line(XLEFT, YBOT, XRIGHT, YBOT);
	graph_line(XLEFT, YTOP, XRIGHT, YTOP);
	graph_char_just_set("cc");
        for ( nt=0; nt<xnumtics; nt++ ) {
		if( xscale.log ) {
		   xtval = pow(10.0,left+nt*scale_incr);
		} else {
		   xtval = left+nt*scale_incr;
		}
	graph_text_double(XLEFT + nt*incr, YBOT-8.0 , xtval, precision);
	if( GP_grid_on ) {
	   graph_set_layer(9);
	   if ( nt > 0 && nt < (xnumtics-1) )
	   graph_set_line_width(-1);
	   graph_line(XLEFT + nt*incr, YBOT, XLEFT + nt*incr, YTOP);
	   if( xscale.log ) {
	      for ( idum=2; idum<=9 ; idum=idum+1 ) {
		logincr = idum;
		graph_set_line_width(-1);
		graph_line( XLEFT +nt*incr + incr*LOGTEN(logincr), YBOT, XLEFT + nt*incr + incr*LOGTEN(logincr), YTOP);
	      }
	   }
	   graph_set_layer(2);
	}
	}
	graph_text((XRIGHT + XLEFT)/2.0, YBOT - 15.0, xscale.expression);
	graph_set_line_width(1);

}
int
set_gp_win_scale(win, xscale, yscale)
	gp_xscale_t xscale;
	gp_yscale_t yscale;
	int win;
{
	double x1, y1, x2, y2;
	double xunit, yunit;

	if (xscale.increasing) {
		if ((xscale.data_left > xscale.axis_right) ||
			(xscale.data_right < xscale.axis_left))
			return(0);
	}
	else { /* xscale decreasing */
		if ((xscale.data_left < xscale.axis_right) ||
			(xscale.data_right > xscale.axis_left))
			return(0);
	}

	xunit = (xscale.axis_right - xscale.axis_left) / (XRIGHT - XLEFT);
	x1 = xscale.axis_left - (XLEFT * xunit);
	x2 = xscale.axis_right + ((600.0 - XRIGHT) * xunit);
	yunit = (yscale.currmax - yscale.currmin) / (YTOP - YBOT);
	y1 = yscale.currmin - (YBOT * yunit);
	y2 = yscale.currmax + ((500.0 - YTOP) * yunit);
	graph_window_set(win, x1, y1, x2, y2);
	return(1);
}

static double *Current_xpts;

draw_plots(plist, xscale, yscale, win, bar_plot)
	gp_plot_t *plist;
	gp_xscale_t xscale;
	gp_yscale_t yscale;
	int win, bar_plot;
{
	double namepos;
	gp_plot_t *plist_ptr;

	plist_ptr = plist;
	namepos = YTOP - 15.0;
	graph_char_just_set("ll");
	graph_font_set("medium");

	while (plist_ptr != NULL) {
		graph_set_layer(plist_ptr->layer);
		graph_text(485.0, namepos, plist_ptr->expression);
		graph_line(465.0, namepos+3.0, 484.0, namepos+3.0);
		namepos = namepos - 15.0;
		plist_ptr = plist_ptr->next;
	}

	if (! set_gp_win_scale(win, xscale, yscale))
		return;

	Current_xpts = NULL;
	plist_ptr = plist;
	while (plist_ptr != NULL) { /* for each plot in this drawing */
		draw_plot(plist_ptr, xscale, yscale, bar_plot);
		plist_ptr = plist_ptr->next;
	}
	Current_xpts = NULL;

	graph_window_set(win, 0.0, 0.0, 600.0, 500.0);
}

draw_plot(plot, xscale, yscale, bar_plot)
	gp_plot_t *plot;
	gp_xscale_t xscale;
	gp_yscale_t yscale;
	int bar_plot;
{
	int i, j, yinside;
	double ymin, ymax;
	double firstx, firsty, lastx, lasty;
	static int xstart, xend;
	int hold_start, hold_end;
	double interp();
	int off_scale, increasing;
	double *ypts;
	run_t *run;

	if (plot == NULL)
		return;

	ymin = yscale.currmin;
	ymax = yscale.currmax;
	increasing = xscale.increasing;

	ypts = plot->ypts;
	if (plot->numpts == 1) { /* we're plotting a scalar */
		graph_set_layer(plot->layer);
		graph_line(xscale.axis_left, ypts[0], xscale.axis_right, ypts[0]);
	}

	/* if this plot is not totally above or below the current yaxis 
	   and not totally beyond the x axis, then plot it */

	else if (! ((ymin > plot->ymax) || (ymax < plot->ymin))) { 
	
		run = plot->xpts->run;
		off_scale = 0;
		if (Current_xpts != plot->xpts->xpts) {
			/* find the start and end indices into the X pts array */
			Current_xpts = plot->xpts->xpts;
			if (! xscale.increasing) {
				xstart = hunt(Current_xpts, run->num_X / run->num_sets, 
							xscale.axis_left, 0);
				xend = hunt(Current_xpts, run->num_X /run->num_sets, 
							xscale.axis_right, 0);
				if ((xstart != xend) &&
					((Current_xpts[xend] > xscale.axis_left) ||
					(Current_xpts[xstart] < xscale.axis_right)))
					off_scale = 1;
			}
			else {
				xstart = hunt(Current_xpts, run->num_X / run->num_sets,
						xscale.axis_left, 0);
				xend = hunt(Current_xpts, run->num_X / run->num_sets,
						xscale.axis_right, 0);
				if ((xstart != xend) &&
					((Current_xpts[xend] < xscale.axis_left) ||
					 (Current_xpts[xstart] > xscale.axis_right)))
					off_scale = 1;
			}
		}

		if (! off_scale) {
			if ((plot->ymax <= ymax) &&
				(plot->ymin >= ymin)) 
				yinside = 1;
			else yinside = 0;

			/* If plot crosses left side of scale, cut it off */
			if ((xscale.increasing && 
				(Current_xpts[xstart] < xscale.axis_left)) ||
				(! xscale.increasing &&
				(Current_xpts[xstart] > xscale.axis_left))) {

				firsty = interp(ypts[xstart],Current_xpts[xstart],
					ypts[xstart+1], Current_xpts[xstart+1], 
					xscale.axis_left);
				firstx = xscale.axis_left;
			}
			else {
				firsty = ypts[xstart];
				firstx = Current_xpts[xstart];
			}

			/* If plot crosses right side of scale, cut it off */
			if ((xend + 1) < (run->num_X / run->num_sets)) {	
				lasty = interp(ypts[xend],Current_xpts[xend],
					ypts[xend+1], Current_xpts[xend+1], 
					xscale.axis_right);
				lastx = xscale.axis_right;
			}
			else {
				lasty = ypts[xend];
				lastx = Current_xpts[xend];
			}

			graph_set_layer(plot->layer);
			if (yinside) {
				if (run->run_type == CHAR_CURVES) {
					hold_start = xstart;
					hold_end = xend;
					for (j = 0; j < run->num_sets; j++) {
						graph_start_dots(run->num_X);
						if (firstx == xscale.axis_left)  /* cutoff at left */
							firsty = interp(ypts[xstart],Current_xpts[xstart],
								ypts[xstart+1], Current_xpts[xstart+1], 
								xscale.axis_left);
						else firsty = ypts[xstart];
						graph_store_dot(firstx, firsty);

						for (i = xstart + 1; i < xend; i++) 
							graph_store_dot(Current_xpts[i], ypts[i]);

						if (lastx == xscale.axis_right) /* cutoff at right */
							lasty = interp(ypts[xend-1], Current_xpts[xend-1],
								ypts[xend], Current_xpts[xend], 
								xscale.axis_right);
						else lasty = ypts[xend];
						
						graph_store_dot(lastx, lasty);
						graph_connect_dots();

						xstart = xstart + (run->num_X/run->num_sets);
						xend = xend + (run->num_X/run->num_sets);
					}
					xstart = hold_start;
					xend = hold_end;
				}
				else if (bar_plot) {
					for (i = xstart; i <= xend; i++)
						graph_fat_line(Current_xpts[i], ymin,
								Current_xpts[i], ypts[i]);
				}
				else {
					graph_start_dots(plot->numpts);
					graph_store_dot(firstx, firsty);
					for (i = xstart + 1; i <= xend; i++) 
						graph_store_dot(Current_xpts[i], ypts[i]);
					graph_store_dot(lastx, lasty);
					graph_connect_dots();
				}
			}
			else { 
				/* complicated plot which crosses top and bottom bounds */
				if (run->run_type == CHAR_CURVES) {
					hold_start = xstart;
					hold_end = xend;
					for (j = 0; j < run->num_sets; j++) {
						if (firstx == xscale.axis_left)  /* cutoff at left */
							firsty = interp(ypts[xstart],Current_xpts[xstart],
								ypts[xstart+1], Current_xpts[xstart+1], 
								xscale.axis_left);
						else firsty = ypts[xstart];

						if ((increasing && (firstx < Current_xpts[xstart+1])) ||
							(!increasing && (firstx > Current_xpts[xstart+1])))
							graph_line_yclip(firstx, firsty, 
								Current_xpts[xstart+1], ypts[xstart+1], 
								ymin, ymax);

						for (i = xstart+1; i < xend; i++) {
							if ((increasing && 
									(Current_xpts[i] < Current_xpts[i+1])) ||
								(!increasing &&
									(Current_xpts[i] > Current_xpts[i+1])))
								graph_line_yclip(Current_xpts[i], ypts[i], 
									Current_xpts[i+1], ypts[i+1], ymin, ymax);
						}

						if (lastx == xscale.axis_right) /* cutoff at right */
							lasty = interp(ypts[xend-1], Current_xpts[xend-1],
								ypts[xend], Current_xpts[xend], 
								xscale.axis_right);
						else lasty = ypts[xend];
						if ((increasing && (Current_xpts[i] < lastx)) || 
							(!increasing && (Current_xpts[i] > lastx)))
							graph_line_yclip(Current_xpts[xend], ypts[xend], 
									lastx, lasty, ymin, ymax);

						xstart = xstart + (run->num_X/run->num_sets);
						xend = xend + (run->num_X/run->num_sets);
					}
					xstart = hold_start;
					xend = hold_end;
				}
				else if (bar_plot) {
					for (i = xstart; i <= xend; i++)
						graph_fat_line(Current_xpts[i], ymin,
								Current_xpts[i], ypts[i]);
				}
				else {
					graph_line_yclip(firstx, firsty, Current_xpts[xstart+1],
						ypts[xstart+1], ymin, ymax);
					for (i = xstart+1; i < xend; i++) 
						graph_line_yclip(Current_xpts[i], ypts[i], 
							Current_xpts[i+1], ypts[i+1], ymin, ymax);
					graph_line_yclip(Current_xpts[xend], ypts[xend], 
							lastx, lasty, ymin, ymax);
				}
			}
		}
	}
}


gp_win_t *
win_struct(win_id)
	int win_id;
{
	gp_win_t *winptr;

	winptr = GP_wins;
	while ((winptr != NULL) && (winptr->id != win_id))
		winptr = winptr->next;
	return(winptr);
}


static int
assign_layer()
{
	static int next_layer = 4;
	
	if (next_layer == 9)
		next_layer = 4;
	else next_layer++;
	return(next_layer);
}

addplot(run, ypts, numpts, made_up, expression)
	run_t *run;
	double *ypts;
	char *expression;
	int numpts;
	int made_up;
{
	gp_plot_t *plot_ptr, *previous;
	double min, max, save_max, save_min;
	int i, ctz, first, log;
	gp_xpt_t *xscale;
	double *save_ypts;

	log = GP_currwin->yscale.log;

	xscale = find_xscale(run);
	if (numpts != 1) { /* if this is not a constant plot */
		if (xscale == NULL) {
			/* If we were to implement the reconciliation of xpts from two
			   different files being used in one expression, then we'd have
			   to delete xpts possibly also. */
			if (made_up)
				free ((char *)ypts);
			return;
		}
	}

	if (GP_currwin->plot_list == NULL) {
		GP_currwin->plot_list = (gp_plot_t *) malloc (sizeof(gp_plot_t));
		plot_ptr = GP_currwin->plot_list;
		first = 1;
	}
	else {
		plot_ptr = GP_currwin->plot_list;
		previous = NULL;
		while(plot_ptr != NULL) {
			previous = plot_ptr;
			plot_ptr = plot_ptr->next;
		}
		previous->next = (gp_plot_t *) malloc (sizeof(gp_plot_t));
		plot_ptr = previous->next;
		first = 0;
	}

	plot_ptr->next = NULL;
	if (made_up) {
		if (log) {
			plot_ptr->ypts = (double *)calloc(numpts, sizeof(double));
			for (i = 0; i < numpts; i++)
				plot_ptr->ypts[i] = LOGTEN(ypts[i]);
			save_ypts = plot_ptr->save_ypts = ypts;
			ypts = plot_ptr->ypts;
		}
		else plot_ptr->ypts = ypts;
	}
	else {
		plot_ptr->ypts = (double *)calloc(numpts, sizeof(double));
		if (log) 
			plot_ptr->save_ypts = (double *)calloc(numpts, sizeof(double));
		
		if (log) {
			for (i = 0; i < numpts; i++) {
				plot_ptr->save_ypts[i] = ypts[i];
				plot_ptr->ypts[i] = LOGTEN(ypts[i]);
			}
			ypts = plot_ptr->ypts;
			save_ypts = plot_ptr->save_ypts;
		}
		else {
			for (i = 0; i < numpts; i++) 
				plot_ptr->ypts[i] = ypts[i];
			ypts = plot_ptr->ypts;
		}
	}

	plot_ptr->numpts = numpts;
	plot_ptr->xpts = xscale;
	plot_ptr->marked_for_death = 0;
	plot_ptr->layer = assign_layer();
	
	plot_ptr->expression = (char *) calloc (strlen(expression) + 1,
			sizeof(char));
	strcpy(plot_ptr->expression, expression);

	min = max = ypts[0];
	if (log) {
		save_min = save_max = save_ypts[0];	
		ctz = 0;
		for (i = 0; i < numpts; i++) {
			if (ypts[i] > max)
				max = ypts[i];
			if (ypts[i] < min) {
				min = ypts[i];
				ctz = i;
			}
			if (save_ypts[i] > save_max)
				save_max = save_ypts[i];
			if (save_ypts[i] < save_min)
				save_min = save_ypts[i];
		}
	}
	else {
		for (i = 0; i < numpts; i++) {
			if (ypts[i] > max)
				max = ypts[i];
			if (ypts[i] < min) 
				min = ypts[i];
		}
	}
	plot_ptr->ymin = min;
	plot_ptr->ymax = max;
	if (log) {
		plot_ptr->save_ymin = save_min;
		plot_ptr->save_ymax = save_max;
	}
	if (log)
		plot_ptr->closest_to_zero = ctz;
	else plot_ptr->closest_to_zero = -1;

	/* For now, I'm gonna try always updating yscale whenever a new
	   plot is added.  This is so that the new plot will always be
	   visible and not offscreen beyond the old yscale range. */
	if (log)
		adjust_yscale(first, min, max, save_min, save_max);
	else adjust_yscale(first, min, max, 0.0, 0.0);

}

adjust_yscale(first, min, max, save_min, save_max)
	int first;
	double min, max, save_min, save_max;
{
	gp_yscale_t *yscale;

	yscale = &(GP_currwin->yscale);
	if (first) {
		if (min == max)
			max = 1.1 * min;
		yscale->min = yscale->currmin = min;
		yscale->max = yscale->currmax = max;

		if (yscale->log) {
			if (save_min == save_max)
				save_max = 1.1 * save_min;
			yscale->save_min = yscale->save_currmin = save_min;
			yscale->save_max = yscale->save_currmax = save_max;
		}	
	}
	else {
		if (yscale->min > min)
			yscale->min = yscale->currmin = min;
		if (yscale->max < max)
			yscale->max = yscale->currmax = max;

		if (yscale->log) {
			if (yscale->save_min > save_min)
				yscale->save_min = yscale->save_currmin = save_min;
			if (yscale->save_max < save_max)
				yscale->save_max = yscale->save_currmax = save_max;
		}
	}
}

open_gp_win(newwin)
	gp_win_t *newwin;
{
	newwin->id = graph_create_free_win(0.0, 0.0, 600.0, 500.0, 
				0.65, 0.65, 'c', 0, 0, "sigview");
	gp_draw_screen(newwin->id);
	graph_set_input(newwin->id, 0, "bk");
}

/* Create a new general window structure and initialize it.  Set the
   GP_currwin pointer to this structure. */

gp_win_t *
define_gp_win()
{
	gp_xscale_t *xscale;
	gp_win_t *newwin, *winptr;
	int inherit;

	inherit = 0;

	if (GP_wins == NULL)  {
		newwin = (gp_win_t *) malloc (sizeof(gp_win_t));
		GP_wins = newwin;
	}
	else {
		if (GP_wins->id == -1)
			newwin = GP_wins;
		else {
			newwin = (gp_win_t *) malloc (sizeof(gp_win_t));
			winptr = GP_wins;
			while (winptr->next != NULL)
				winptr = winptr->next;
			winptr->next = newwin;

			inherit = 1;
			newwin->curr_run = winptr->curr_run;
		}
	}
	GP_currwin = newwin;

	xscale = &(newwin->xscale);
	xscale->expression = NULL;
	xscale->scales = NULL;
	xscale->increasing = -1;
	newwin->xscale.log = 0;
	xscale->log = 0;
	newwin->xscale.numtics=4;

	if (! inherit) {
		if (Sig_files != NULL) {
			newwin->curr_run = Sig_files->runs;
			if (newwin->curr_run->run_type == AC)
				newwin->xscale.log = 1;
		}
		else newwin->curr_run = NULL;
	}

	newwin->yscale.log = 0;
	newwin->yscale.min = newwin->yscale.currmin = 0.0;
	newwin->yscale.max = newwin->yscale.currmax = 0.0;
	newwin->id = -1;
	newwin->bar_plot = 0;
	newwin->plot_list = NULL;
	newwin->label_list = NULL;
	newwin->precision = 1;
	newwin->next = NULL;
	newwin->yscale.numtics=6;

	GP_grid_on = 1;

	return(newwin);
}

def_and_open_gp_win()
{
	gp_win_t *newwin, *define_gp_win();

	newwin = define_gp_win();
	open_gp_win(newwin);
}

/* This removes the current window (as denoted by GP_currwin) from the
   screen and also removes its structure from the win list.  If it was
   the only window on the screen, then we must quit Sigview too. */

destroy_gp_win() 
{
	gp_win_t *winptr, *previous;
	gp_plot_t *pptr;
	gp_xpt_t *scale;
	gp_label_t *label;
	gp_win_t *define_gp_win();

	winptr = GP_wins;
	previous = NULL;
	while ((winptr != NULL) && (winptr != GP_currwin)) {
		previous = winptr;
		winptr = winptr->next;
	}
	if (previous == NULL)	
		GP_wins = winptr->next;
	else previous->next = winptr->next;

	if (winptr->xscale.expression != NULL)
		free ((char *)winptr->xscale.expression);
	while (winptr->plot_list != NULL) {
		pptr = winptr->plot_list;
		free ((char *)pptr->expression);
		free ((char *)pptr->ypts);
		winptr->plot_list = pptr->next;
		free ((char *)pptr);
	}
	while (winptr->xscale.scales != NULL) {
		scale = winptr->xscale.scales;
		free ((char *)scale->xpts);
		winptr->xscale.scales = scale->next;
		free ((char *)scale);
	}

	while (winptr->label_list != NULL) {
		label = winptr->label_list;
		free ((char *)label->text);
		winptr->label_list = label->next;
		free ((char *)label);
	}

	GP_currwin = NULL;
	graph_end_win(winptr->id);
	free ((char *)winptr);

	if (GP_wins == NULL) { 
		if (Sig_main == -1) { /* now there are NO windows left! */
			sigview_quit();
			return;
		}
		GP_currwin_id = -1;
		define_gp_win(); /* we always want one default win struct around 
						in case we do some commands from a command file.*/
		GP_mode = 0;
	}
}

draw_header()
{
	char date[40], *sigvers, c;
	FILE *datefile;
	int i=0;

	graph_set_layer(2);
	graph_font_set("small");
	graph_char_just_set("ll");

	sigvers = "Sigview Version 3.1";
	system("date > /usr/tmp/sigdate");
	datefile = fopen( "/usr/tmp/sigdate", "r");
	while( ( c = fgetc( datefile ) ) != EOF ) date[i++] = c;
	date[i-1] = '\0';
	system("rm /usr/tmp/sigdate" );
	graph_text(XLEFT, YTOP + 5.0, sigvers);
	graph_char_just_set("lr");
	graph_text(XRIGHT, YTOP + 5.0, date);

	graph_font_set("medium");
}

