/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "gp.h"
#include <ctype.h>

int GP_currwin_id; /* window from which we are currently processing a command */
gp_win_t *GP_currwin; /* pointer to current window's info */
gp_win_t *GP_wins;  /* list of pointers to our children windows */

extern comm_file_t *Sig_files;
extern int Sig_interactive_mode;

gp_do_show()
{
	char vecnames[BUFSIZE], runnames[BUFSIZE];

	if (select_signames(vecnames)) /* found in "interact.c" */
		if (select_runnames(runnames))
			gp_show(vecnames, runnames);
}

static int
select_runnames(runnames)
	char *runnames;
{
	struct holder_t {
		int selected;
		double startx, endx;
		run_t *runptr;
	} *holder;
	comm_file_t *fileptr;
	run_t *runptr;
	int win_id;
	double pos, graph_str_width();
	char buffer[BUFSIZE];
	int found, selected, i, val, done, all, counter;
	double x, y;
	char ev, graph_await_event();

	if (Sig_files == NULL) {
		sig_msg("No runs have yet been loaded","","");
		return(0);
	}

	graph_set_drawing(GP_currwin_id);

	counter = 0;
	fileptr = Sig_files;
	while (fileptr != NULL) {
		if (! DIGITAL(fileptr->type)) {
			runptr = fileptr->runs;
			while (runptr != NULL) {
				counter++;
				runptr = runptr->next;
			}
		}
		fileptr = fileptr->next;
	}

	holder = (struct holder_t *) calloc (counter, sizeof(struct holder_t));

	i= 0; 
	pos = 22.0 + graph_str_width("Runs: ");
	fileptr = Sig_files;
	while (fileptr != NULL) {
		if (! DIGITAL(fileptr->type)) {

			runptr = fileptr->runs;
			while (runptr != NULL) {
				holder[i].selected = 0;
				holder[i].runptr = runptr;
				holder[i].startx = pos;

				buffer[0] = '\0';
				if (GP_currwin->curr_run == runptr) {
					strcat(buffer, "[");
					strcat(buffer, runptr->name);
					strcat(buffer, "] ");
				}
				else {
					strcat(buffer, runptr->name);
					strcat(buffer, " ");
				}
				pos = pos + graph_str_width(buffer);
				holder[i].endx = pos;
				runptr = runptr->next;
				i++;
			}
		}
		fileptr = fileptr->next;
	}

	/* draw over menu here to show mouse buttons */
	graph_draw_mode(0);
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(150.0, 33.0, "Please select run names from Runs list above.");
	graph_text(150.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(150.0, 11.0, "unselect   *   commit   *   select");
	graph_set_layer(5); /* green */
	graph_rect(310.0, 15.0, 340.0, 25.0);
	graph_set_layer(3);
	graph_text(325.0, 20.0, "ALL");

	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_set_input(GP_currwin_id, 1, "b");
	all = done = 0;
	while (! done) {
		ev = graph_await_event(&win_id, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if (ev == 's')
			;	
		else if ((y >= 15.0) && (y <= 25.0)) {
			if ((x >= 310.0) && (x <= 340.0)) 
				all = done = 1;
		}
		else if (val == MIDBUTTON)
			done = 1;

		else if ((y >= 48.0) && (y <= 58.0)) { /*perhaps over a name */
			i = 0;
			found = 0;
			while ((i < counter) && (!found)) {
				if ((x >= holder[i].startx) && (x <= holder[i].endx)) {
					found = 1;
					if ((val == RTBUTTON) && (! holder[i].selected)) {
						holder[i].selected = 1;
						graph_draw_mode(0);
						graph_set_layer(holder[i].runptr->layer);
						graph_line(holder[i].startx, 48.0,
							holder[i].endx, 48.0);
						/*
						if (GP_currwin->curr_run == holder[i].runptr) {
							buffer[0] = '\0';
							strcat(buffer, "[");
							strcat(buffer, holder[i].runptr->name);
							strcat(buffer, "]");
							graph_text(holder[i].startx, 50.0, buffer);
						}
						else graph_text(holder[i].startx, 50.0, 
									holder[i].runptr->name);
						*/
					}
					else if ((val == LEFTBUTTON) && holder[i].selected) {
						holder[i].selected = 0;
						graph_draw_mode(1);
						graph_set_layer(holder[i].runptr->layer);
						graph_line(holder[i].startx, 48.0,
							holder[i].endx, 48.0);
						/*
						if (GP_currwin->curr_run == holder[i].runptr) {
							buffer[0] = '\0';
							strcat(buffer, "[");
							strcat(buffer, holder[i].runptr->name);
							strcat(buffer, "]");
							graph_text(holder[i].startx, 50.0, buffer);
						}
						else graph_text(holder[i].startx, 50.0, 
									holder[i].runptr->name);
						*/
					}
				}
				i++;
			}
		}
	}
	graph_set_input(GP_currwin_id, 0, "bk");
	graph_draw_mode(0);
	/* redraw the run names, which may possibly be underlined */
	for (i= 0; i < counter; i++) {
		graph_set_layer(holder[i].runptr->layer);
		if (GP_currwin->curr_run == holder[i].runptr) {
			buffer[0] = '\0';
			strcat(buffer, "[");
			strcat(buffer, holder[i].runptr->name);
			strcat(buffer, "]");
			graph_text(holder[i].startx, 50.0, buffer);
		}
		else graph_text(holder[i].startx, 50.0, holder[i].runptr->name);
	}

	runnames[0] = '\0';
	if (all) {
		free ((char *)holder);
		return(1);
	}
	selected = 0;
	for (i = 0; i < counter; i++) {
		if (holder[i].selected) {
			selected++;
			strcat(runnames, holder[i].runptr->name);
			strcat(runnames, " ");
		}
	}
	free ((char *)holder);
	if (selected == 0) 
		return(0);
	return(1);
}

gp_show_p(arglist)
	char *arglist;
{
	char *vecnms, *runnms, *namelist();
	char holder[512];
	char *stupid;
	int len;

	if (arglist[0] == '\0') {
		gp_show("","");
		return;
	}

	remove_excess_whitespace(arglist, holder);
	stupid = &(holder[0]);
	if (*stupid == ' ')
		stupid++;
	len = strlen(stupid);
	if (stupid[len - 1] != ' ') {
		stupid[len] = ' ';
		stupid[len + 1] = '\0';
	}

	vecnms = namelist(&stupid, 's');
	runnms = namelist(&stupid, 'f');
	gp_show(vecnms, runnms);
}

gp_show(vecnames, runnames)
	char *vecnames, *runnames;
{
	comm_file_t *file_ptr;
	run_t *run, *which_run();
	char *runname, *index(), *blank;

	if (*runnames == '\0') { /* then use all runs loaded */
		file_ptr = Sig_files;
		while (file_ptr != NULL) {
			run = file_ptr->runs;
			while (run != NULL) {
				if (run->run_type != DIGITL) 
					generate_from_vecnames(run, vecnames);
				run = run->next;
			}
			file_ptr = file_ptr->next;
		}
	}
	else { /* use only those runs listed in runnames */
		runname = runnames;

		while (*runname != '\0') {
			blank = index(runname, ' ');
			*blank = '\0';
			run = which_run(runname);
			if (run != NULL) {
				if (run->run_type != DIGITL) 
					generate_from_vecnames(run, vecnames);
			}
			blank++;
			runname = blank;
		}
	}

	if (Sig_interactive_mode)
		gp_draw_screen(GP_currwin_id);
}

static
generate_from_vecnames(run, vecnames)
	run_t *run;
	char *vecnames;
{
	int i;
	char *vecname;

	if (*vecnames == '\0') { /* then create plot from all vectors 
								in this run except for the first one */
		for (i = 1; i < run->num_Y; i++) 
			addplot(run, run->data.analog[i], run->num_X, 0, run->Ynames[i]);
	}
	else { /* use only those listed in vecnames */
		vecname = vecnames;	
		while ((vecname != (char *)1) && (*vecname != '\0')) {
			i = find_signm(vecname, run);
			if (i != -1)
				addplot(run, run->data.analog[i],run->num_X,0,run->Ynames[i]);
			vecname = index(vecname, ' ');
			vecname++;
		}
	}
}
