/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "hardcopy.h"
#include <ctype.h>

#define BUFSIZE	512

H_cmap_t H_cmap[] = { 
{0.0,	0.0,	0.0}, 	/* 0 bkgd, white  */
{1.0,	1.0,	1.0}, 	/* 1 black */
{0.6,	0.6	,	0.6},	/* 2 grey */
{0.0,	1.0,	1.0}, 	/* 3 lt blue */
{1.0,	0.0	,	0.0}, 	/* 4 red */
{0.0,	1.0	,	0.0}, 	/* 5 green */
{1.0,	0.0,	0.8}, 	/* 6 purple */
{0.8,	0.8,	0.0}	/* 7 yellow */
};

int  H_lmap[] = {
/* 0 bkgd */  0,	
/* 1 dkblue */ 1, 
/* 2 textoverbkgd */ 1,
/* 3 textovercolors */ 1,
/* 6 red */	4,
/* 7 green */ 5,
/* 5 yellow */  3,
/* 8 purple */ 6,
/* 9 ltblue */  7,
/* 4 grey */ 2, 
/* 10 leftvert */	1,
/* 11 rightvert */	1,
};

func_t Hard_funcs;
int Hardcopy;
char Hardcopy_dev[8], Hardcopy_unixcmd[128];

int
hardcopy_on(devnm)
	char *devnm;
{
	FILE *devfile, *open_fl();
	char line[BUFSIZE];
	char name[BUFSIZE];
	int done;
	char *lineptr, *hh;

	/* open the "devices" file to find the other necessary info.  Note that
	   SIG_DEV_FILE is defined by the -D option when compiling Sigview. */
	strcpy(name, SIG_DEV_FILE);    
/*	strcpy(name, "/usr/garbage");  */

	if ((devfile = open_fl(name, "r")) == NULL) {
		sig_msg("Cannot find ", name, ".");
		return(0);
	}

	/* locate a matching devnm, if possible */
	done = 0;
	while (! done) {
		if (fgets(line, BUFSIZE, devfile) == NULL) 
			done = 1;
		else if ((sscanf(line, "%s", name) == 1) &&
				 (strcmp(name, devnm) == 0))
			done = 1;
	}
	if (strcmp(name, devnm) != 0) {
		sig_msg("Cannot find ", devnm, " in devices file.");
		return(0);
	}

	/* gobble and pass the devnm */
	lineptr = line;
	while (isspace(*lineptr)) lineptr++;
	while (! isspace(*lineptr)) lineptr++;
	while (isspace(*lineptr)) lineptr++;

	/* the next field should be the device type */
	*Hardcopy_dev = '\0';
	hh = Hardcopy_dev;
	while (! isspace(*lineptr)) { 
		*hh = *lineptr;
		hh++; lineptr++;
	}
	if (*Hardcopy_dev == '\0') {
		sig_msg("Missing second field in ",name," line");
		return(0);
	}
	*hh = '\0'; /* terminate Hardcopy_dev */

	while (isspace(*lineptr)) lineptr++;
	strcpy(Hardcopy_unixcmd, lineptr); 
	/* there will be a return at the end of the line. remove it. */
	hh = Hardcopy_unixcmd;
	while ((*hh != '\n') && (*hh != '\0'))
		hh++;
	if (*hh == '\n')
		*hh = ' ';

	if (*Hardcopy_unixcmd == '\0') {
		sig_msg("Missing third field in ",name," line");
		return(0);
	}

	if ((strcmp(Hardcopy_dev, "tek") == 0) ||
		(strcmp(Hardcopy_dev, "tekfile") == 0))
		connect_to_tek_funcs();
	else if ((strcmp(Hardcopy_dev, "ps") == 0) ||
			 (strcmp(Hardcopy_dev, "psfile") == 0))
		connect_to_ps_funcs();
	else if ((strcmp(Hardcopy_dev, "hp") == 0) ||
			 (strcmp(Hardcopy_dev, "hpfile") == 0)) {
		connect_to_hp_funcs();
	}
	else return(0);

	if ((Hard_funcs.init)(Hardcopy_dev, Hardcopy_unixcmd) == 0)
		return(0);

	Hard_funcs.color_map();
	Hard_funcs.window_set(0.0, 0.0, 600.0, 500.0);
	Hardcopy = 1;
	return(1);
}

hardcopy_off()	
{
	Hardcopy = 0;
	Hard_funcs.term();
}

/* remove the garbage at the end of a filename */
strip_crap(filename)
	char *filename;
{
	char *ptr;

	ptr = filename;
	while ((*ptr != '\0') && (! isspace(*ptr)) && (*ptr != '\n'))
		ptr++;

	*ptr = '\0';
}
