/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "hardcopy.h"

extern func_t Hard_funcs;
extern int H_lmap[];
extern H_cmap_t H_cmap[];

FILE	*Hp_spool;
char Hp_fonttype[12];
int Hp_color;
double	Hp_charwidth;
int Hp_devtype;
char Hp_cmd[256];
char Hp_filenm[256];

#define HP_XRES	5000
#define HP_YRES	5000

/* all through here we assume that the pen is UP.  If it gets put down to
   draw something, then it is the responsibility of that function (e.g
   graph_text or graph_line) to return it to it's UP position. */
int
hp_init(devtype, astring)
	char *devtype, *astring;
{
	FILE *open_fl();
	int baudrate;
	char slashdev[256];

	/* determine whether we send to a real file or to a temp file to be
	   sent to an actual Tektronix. */
	Hp_devtype = TFILE;

	if (strcmp(devtype, "hp") == 0) {
		unique_tempnm(Hp_filenm, "#Sig_hpXXXXXX");
		if ((Hp_spool = open_fl(Hp_filenm, "w")) == NULL) {
			sig_msg("Cannot open temp file ",Hp_filenm," for HP plot");
			return(0);
		}
		/* there is supposed to be a baud rate and a /dev in "astring" */
		if (sscanf(astring, "%d %s", &baudrate, slashdev) != 2) {
			sig_msg("Baud and/or /dev incorrect in HP line in ",
				"\"devices\" file.","");
			return(0);
		}
		sprintf(Hp_cmd, "(stty %d; cat %s) > %s", baudrate, Hp_filenm,
			slashdev);
		Hp_devtype = TDIRECT;
	}
	else {
		strip_crap(astring);
		if ((Hp_spool = open_fl(astring, "w")) == NULL) {
			sig_msg("Cannot open ",astring, "for HP dump");
			return(0);
		}
		else strcpy(Hp_filenm, astring);
	}

	/* I don't really know what this sequence does.  I stole it from the
	   Vivid Z code, and mysteriously, it makes this driver work. Here is
	   the comment which came with it:
 			* This string inits either type of plotter.  It turns the 
			* plotter on (7220 only), sets up for Xon Xoff handshaking,
 			* and specifies LF as the terminator character.
	   */
	fprintf(Hp_spool, "\033.(IN;\033.I80;;17:\033.N;19:\033.M;;;10;:");

	fprintf(Hp_spool, "DF;");
	fprintf(Hp_spool, "IP;"); 
	fprintf(Hp_spool, "SC0,%d,0,%d;", HP_XRES, HP_YRES);
	fprintf(Hp_spool, "DI0,-1;"); /* rotate all text */

	strcpy(Hp_fonttype, "");
	hp_font_set("medium");

	Hp_color = 0;
	return(1);
};

hp_color_map()
{
}

static double Gx1, Gy1;		/* current lower left coords of the "world" of
							   the graphics program using graph.c */
static double Gx2, Gy2;
static double GtoX_scalex, GtoX_scaley;
static double XtoG_scalex, XtoG_scaley;

int
hp_window_set(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	if (x1 == x2) {
		fprintf(stderr,
			"hp_window_set: not resetting scale -- window has no width\n");
		return(0);
	}
	if (y1 == y2) {
		fprintf(stderr,
			"hp_window_set: not resetting scale -- window has no height\n");
		return(0);
	}
/*
	Gx1 = SMIN(x1,x2);
	Gy1 = SMIN(y1,y2);
	Gx2 = Gx1 + (ABS((x2 - x1)));   
	Gy2 = Gy1 + (ABS((y2 - y1)));
*/
	Gx1 = x1;
	Gx2 = x2;
	Gy1 = y1;
	Gy2 = y2;

	GtoX_scalex = HP_XRES/(x2 - x1);   
	GtoX_scaley = HP_YRES/(y2 - y1);
	XtoG_scalex = 1.0 / GtoX_scalex;
	XtoG_scaley = 1.0 / GtoX_scaley;

	return(1);
}

/* user coordinate to HP coordinate */
GtoHp(x, y, tx, ty)
	double x, y;
	int *tx, *ty;
{
/*
	if (x > Gx2) x = Gx2;
	if (y > Gy2) y = Gy2;
*/
	*tx = (int)((y - Gy1) * GtoX_scaley);
	*ty = (int)(HP_XRES - ((x - Gx1) * GtoX_scalex));
}

#define HP_BIGFONT	.2	
#define HP_LILFONT	.14
#define CMS_IN_8	16.5  /* approx. number of centimetres in ht of plot */

hp_font_set(type)
	char *type;
{
	if ((strcmp(Hp_fonttype, type)) == 0)
		return;
	strcpy(Hp_fonttype, type);

	if (strcmp(type, "small")==0)  {
		Hp_charwidth = (HP_LILFONT * HP_XRES) / CMS_IN_8;
		fprintf(Hp_spool, "SI%.2f,%.2f;", HP_LILFONT, HP_LILFONT);
	}
	else {
		Hp_charwidth = (HP_BIGFONT * HP_XRES) / CMS_IN_8;
		fprintf(Hp_spool, "SI%.2f,%.2f;", HP_BIGFONT, HP_BIGFONT);
	}
}

hp_char_just_set(type)
	char *type;
{
	if (strcmp(type,"cc")== 0)
		fprintf(Hp_spool, "LO5;");
	else if (strcmp(type,"rc")== 0)
		fprintf(Hp_spool, "LO8;");
	else if (strcmp(type,"ll")== 0)
		fprintf(Hp_spool, "LO1;");
	else if (strcmp(type,"ul")== 0)
		fprintf(Hp_spool, "LO3;");
	else if (strcmp(type,"bc")== 0)
		fprintf(Hp_spool, "LO4;");
}

hp_text(x, y, string)
	double x, y;
	char *string;
{
	int xx, yy;

	GtoHp(x, y, &xx, &yy);
	fprintf(Hp_spool, "PA%d,%d;", xx, yy);
	sconv(string);
	fprintf(Hp_spool, "LB%s", string);
}

static
sconv(p)
register char *p;
{
	for ( ; *p; p++) {
		*p &= 0177;
		if (*p < ' ')
			*p = ' ';
	}
}

double
hp_str_width(str)
	char *str;
{
	double strwidth;
	double x;

	strwidth = Hp_charwidth * strlen(str);
	x = (strwidth * XtoG_scalex) + Gx1;
	return(x);
}

hp_set_layer(which)
	int which;
{
	if (Hp_color == H_lmap[which])
		return;

	Hp_color = H_lmap[which];
	/* make a pen assignment. Note we assume that Hp_color is from 0 to 7 */
	fprintf(Hp_spool, "SP%d;", Hp_color+1);
}

hp_term()
{
	fprintf(Hp_spool, "SP;IN;");
	fclose(Hp_spool);
	if (Hp_devtype == TDIRECT) {
		sig_msg("Executing:  ",Hp_cmd,"");
		if (system(Hp_cmd))
			sig_msg("Could not execute: ",Hp_cmd,"");
		delete_file(Hp_filenm);	
	}
	else sig_msg("Have written HPGL to ", Hp_filenm, "");
}

hp_line(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	int tx1, ty1, tx2, ty2;

	GtoHp(x1, y1, &tx1, &ty1);
	GtoHp(x2, y2, &tx2, &ty2);
	fprintf(Hp_spool, "PA%d,%d;PD;PA%d,%d;PU;", tx1, ty1, tx2, ty2);
}

hp_connect_dots(Dotlist, Dotpointer)
	XPoint *Dotlist;
	int Dotpointer;
{
	int i;

	fprintf(Hp_spool, "PA%d,%d;PD;", Dotlist[0].x, Dotlist[0].y);
	for (i = 1; i < Dotpointer-1; i++)
		fprintf(Hp_spool, "PA%d,%d;", Dotlist[i].x, Dotlist[i].y);

	fprintf(Hp_spool, "PA%d,%d;PU;", Dotlist[Dotpointer-1].x, 
		Dotlist[Dotpointer-1].y);
}

connect_to_hp_funcs()
{
	Hard_funcs.init = hp_init;
	Hard_funcs.color_map = hp_color_map;
	Hard_funcs.window_set = hp_window_set;

	Hard_funcs.font_set = hp_font_set;
	Hard_funcs.char_just_set = hp_char_just_set;
	Hard_funcs.text = hp_text;
	Hard_funcs.str_width = hp_str_width;

	Hard_funcs.set_layer = hp_set_layer;

	Hard_funcs.GtoWhatever = GtoHp;
	Hard_funcs.connect_dots = hp_connect_dots;

	Hard_funcs.line = hp_line;

	Hard_funcs.term = hp_term;
}
