/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "gp.h"

extern int Sig_notype;
extern vert_t Sig_leftvert, Sig_rightvert;
extern char **Sig_signms;
extern int Sig_signm_ct;
extern comm_file_t *Sig_files;
extern axis_t *Sig_axes;
extern int Sig_axis_ct;
extern axis_t *Sig_top_axis, *Sig_bottom_axis;
extern double Sig_timeend, Sig_timestart, Sig_interval, Sig_multiplier;

extern int Global_quit, GP_mode, GP_currwin_id;
extern int Sig_main;
extern gp_win_t *GP_currwin;
extern char **GP_vecnames;
extern int GP_vecname_ct;

int	Sig_keybd_mode, Sig_mouse_mode;
int Sig_interactive_mode;

#define	COMPARE	1
#define COPY	2
#define CREATE	3
#define SHOW	4
#define DELETE	5
#define COMBINE	6
#define SHUFFLE	7
#define SNAPSHOT	8
#define RESET	9
#define ZOOM	10
#define QUIT	11
#define REDRAW	12


interact()
{
	char code, graph_await_event();
	int val;
	double x, y;
	int menuitem, which, winid;
	char cmdbuf[512];
	gp_win_t *win_struct();

	Sig_keybd_mode = 0;
	Sig_mouse_mode = 0;
	Sig_interactive_mode = 1;
	if (Sig_main != -1)
		graph_set_input(Sig_main, 0, "bk");
	if (GP_currwin_id != -1)
		graph_set_input(GP_currwin_id, 0, "bk");

	while (! Global_quit) {
		code = graph_await_event(&winid, &val, &x, &y);
		graph_set_drawing(winid);
		if (winid == Sig_main) {
		switch (code) {
			case 'k':
				Sig_keybd_mode = 1;
				graph_set_layer(0);
				graph_draw_mode(0);
				graph_rect(0.0,0.0,600.0,44.0);

				read_and_echo(winid, (char)val, cmdbuf);
				parse_and_execute(cmdbuf);
				/* signal error message handler to reset the msg ptr */
				end_command(); 

				Sig_keybd_mode = 0;
				menu_area();
				break;
			case 'b':
				Sig_mouse_mode = 1;
				graph_set_layer(0);
				graph_draw_mode(0);
				graph_rect(0.0,0.0,600.0,44.0);

				if (over_menu_item(x, y, &menuitem)) 
					do_menuitem(menuitem);
				else if (over_vertical(x, y, &which)) 
					do_verticals(which);
				else if (over_horiz_scroll(x, y)) 
					horiz_scroll();
				else if (over_vert_scroll(x, y)) 
					vert_scroll();

				Sig_mouse_mode = 0;
				/* signal error message handler to reset the msg ptr */
				end_command(); 
				menu_area();
				break;
			case 's':
			/*	graph_window_set(0, 0.0, 0.0, 600.0, 500.0); */
				draw_screen();
				break;
		}
	}
	else {
		GP_currwin_id = winid;
		GP_currwin = win_struct(winid);
		switch (code) {
			case 'b':
				gp_window_menu(x, y);
				end_command(); 
				gp_menu_area(GP_currwin_id);
				break;

			case 'k':
				graph_set_layer(0);
				graph_draw_mode(0);
			/*	graph_set_drawing(GP_currwin_id); */
				graph_rect(0.0, 0.0, 600.0, 44.0);

				read_and_echo(GP_currwin_id, (char)val, cmdbuf);
				gp_parse_and_execute(cmdbuf);
				/* signal error message handler to reset the msg ptr */
				end_command();  
				
				gp_menu_area(GP_currwin_id);
				break;

			case 's':
				gp_resize_labels(GP_currwin_id);
				gp_draw_screen(GP_currwin_id);
				break;
			}
		}
	}
}

/* read input from keyboard until carriage return or buttonhit.
   Assumes that the user coords in the area of the window where this
   echoing is taking place are 0,0,600,40. Returns 1 if valid string held 
   in "response", else returns 0.
*/

#define	Y		30.0
#define XINC	10.0

int
read_and_echo(win, firstchar, response)
	char *response;
	char firstchar;
	int win;
{
	char event, graph_await_event();
	int val, win_id;
	double x, y;
	int done;
	double screenptr; /* curr x position in window where text is being writ */
	int responseptr;	/* current position within response string */
	char oneletter[2];

	graph_char_just_set("ll"),
	graph_set_layer(2);
	graph_set_input(win, 1, "bk");
	
	val = (int)firstchar;
	responseptr = 0;
	done = 0;
	screenptr = XINC;
	response[0] = '\0';
	oneletter[1] = '\0';
	event = 'k';

	while (! done) {
		if (event == 'b')
			done = 1;

		else if (event == 'k') {
			if (val == 10)	/* return hit */
				done = 1;

			else if (val == 8 || val == 127) { /* backspace hit */ 
				if (responseptr != 0)  {	/* not at beginning of line */
					graph_draw_mode(1); /* undraw */
					graph_text(screenptr, Y, "_");
					screenptr = screenptr - XINC;

					responseptr--;
					oneletter[0] = response[responseptr];
					graph_text(screenptr, Y, oneletter);
					response[responseptr] = '\0';

					graph_draw_mode(0);	/* now draw underscore back one spot*/
					graph_text(screenptr, Y, "_");
				}
			}

			else if ((val >= 32) && (val <= 126)) { /* add to response */
				graph_draw_mode(1);	/* undraw underscore */
				graph_text(screenptr, Y, "_");

				graph_draw_mode(0);	/* draw new letter */
				oneletter[0] = val;
				graph_text(screenptr, Y, oneletter);
				response[responseptr++] = val;

				screenptr = screenptr + XINC;	/* now draw new underscore*/
				graph_text(screenptr, Y, "_");
			}
		}
		if (!done)
			event = graph_await_event(&win_id, &val, &x, &y);
	}
	graph_set_input(win, 0, "bk"); 

	if (response[0] == '\0')
		return(0);
	else {
		response[responseptr] = '\0';
		return(1);
	}
}

over_menu_item(x, y, menuitem)
	double x, y;
	int *menuitem;
{
	if (y <= 22.0) {
		if (x <= 100.0)
			*menuitem = COMPARE;
		else if (x <= 200.0)
			*menuitem = CREATE;
		else if (x <= 300.0)
			*menuitem = DELETE;
		else if (x <= 400.0)
			*menuitem = SHUFFLE;
		else if (x <= 500.0)
			*menuitem = RESET;
		else if (x <= 600.0)
			*menuitem = QUIT;
		return(1);
	}
	else if (y <= 44.0) {
		if (x <= 100.0)
			*menuitem = COPY;
		else if (x <= 200.0)
			*menuitem = SHOW;
		else if (x <= 300.0)
			*menuitem = COMBINE;
		else if (x <= 400.0)
			*menuitem = SNAPSHOT;
		else if (x <= 500.0)
			*menuitem = ZOOM;
		else if (x <= 600.0)
			*menuitem = REDRAW;
		return(1);
	}
	return(0);
}

do_menuitem(menuitem)
	int menuitem;
{
	char signames[BUFSIZE], runnames[BUFSIZE], axnames[BUFSIZE];
	char nm[SIGNMSIZE], *next_token();

	switch(menuitem) {
		case COMPARE:
			if (select_signames(signames))
				if (select_runnames(runnames))
					compare(signames, runnames);
			break;
		case COPY:
			if (select_axes(axnames))
				copy(axnames);
			break;
		case CREATE:
			if (Sig_signm_ct == 0) {
				sig_msg("No signals have yet been loaded","","");
				return;
			}
			if (prompt_for_name(0, "Name of axis you wish to create: ",nm, 1))
				if (select_signames(signames))
					if (select_runnames(runnames))
						create(nm, signames, runnames);
			break;
		case SHOW:
			if (select_signames(signames))
				if (select_runnames(runnames))
					show(signames, runnames);
			break;
		case DELETE:
			if (select_axes(axnames)) 
				delete_axes(axnames);
			break;
		case COMBINE:
			if (Sig_signm_ct == 0) {
				sig_msg("No signals have yet been loaded","","");
				return;
			}
			if (prompt_for_name(0, "Name of axis you wish to create: ",nm, 1))
				if (select_axes(axnames))
					combine(nm, axnames);
			break;
		case SHUFFLE:
			interactive_shuffle();
			break;
		case SNAPSHOT:
			do_snapshot();
			break;
		case RESET:
			reset_screen();
			break;
		case ZOOM:
			zoom();
			break;
		case QUIT:
			sigview_quit();
			break;
		case REDRAW:
			draw_screen();
			break;
	}
}


do_snapshot()
{
	char nm[SIGNMSIZE];

	if (prompt_for_name(GP_currwin_id, 
		"Name of device: (hit return to get default)", nm, 1)) 
		snapshot(nm);
	else snapshot("");
}

#define PLAY	2.0

over_vertical(x, y, which)
	double x, y;
	int *which;
{
	if ((x <= (Sig_leftvert.x + PLAY)) && (x >= (Sig_leftvert.x - PLAY))) {
		if (y >= 100.0) {
			*which = LEFT;
			return(1);
		}
		else return(0);
	}
	if ((x <= (Sig_rightvert.x + PLAY)) && (x >= (Sig_rightvert.x - PLAY))) {
		if (y >= 100.0) {
			*which = RIGHT;
			return(1);
		}
		else return(0);
	}
	return(0);
}

do_verticals(which)
	int which;
{
	v_info_t *vinfo, *collect_vertical_info();
	int win_id, val, done;
	char code, graph_await_event();
	double x, y;
	vert_t *vert;
	run_t *run, *which_runname();
	double time, othertime, draw_vert();
	int precise, time_precision();

	if (which == RIGHT) {
		vert = &Sig_rightvert;
		othertime = ((((Sig_leftvert.x - AXISLEFT) / (AXISRIGHT - AXISLEFT)) 
				* Sig_interval) + Sig_timestart) * Sig_multiplier;
	}
	else  {
		vert = &Sig_leftvert;
		othertime = ((((Sig_rightvert.x - AXISLEFT) / (AXISRIGHT - AXISLEFT)) 
				* Sig_interval) + Sig_timestart) * Sig_multiplier;
	}
	vinfo = collect_vertical_info(which);
	precise = time_precision(Sig_timestart + Sig_interval);

	/* loop on mouse movement, redrawing vertical in new spot, until
	   middle button hit (reassign vertical to new file), or the other
	   two are hit (end the process). */

	graph_set_input(Sig_main, 1, "bm");
	done = 0;
	while (!done) {
		code = graph_await_event(&win_id, &val, &x, &y);
		if (code == 'm') {
			/* undraw old vertical */
			graph_draw_mode(1);
			time = draw_vert(which, vinfo, 1, precise);
			draw_dt(time, othertime, precise);

			if (x < AXISLEFT)
				vert->x = AXISLEFT;
			else if (x > AXISRIGHT)
				vert->x = AXISRIGHT;
			else vert->x = x;
			graph_draw_mode(0);
			time = draw_vert(which, vinfo, 1, precise);
			draw_dt(time, othertime, precise);
		}
		else if (code == 'b') {
			if (val == MIDBUTTON) { /* we're to assign vert to a run */
				run = which_runname(x, y);
				if (run != NULL) {
					/* undraw old vertical */
					graph_draw_mode(1);
					time = draw_vert(which, vinfo, 1, precise);
					draw_dt(time, othertime, precise);
					
					vert->run = run;
					vinfo = collect_vertical_info(which);
					graph_draw_mode(0);
					time = draw_vert(which, vinfo, 1, precise);
					draw_dt(time, othertime, precise);
				}
			}
			else done = 1;  /* put down the vertical and end the process */
		}
	}
	if (vinfo)
		free((char *) vinfo);
	graph_set_input(Sig_main, 0, "bk");
}

run_t * 
which_runname(x, y)
	double x, y;
{
	comm_file_t *fileptr;
	run_t * runptr;
	double pos, next_pos, graph_str_width();
	char buffer[BUFSIZE];

	if ((y >= 58.0) && (y <= 68.0)) { /* perhaps over a name */
		pos = 45.0;
		fileptr = Sig_files;
		while (fileptr != NULL) {
			runptr = fileptr->runs;
			while (runptr != NULL) {
				strcpy(buffer,runptr->name);
				strcat(buffer, " ");
				next_pos = pos + graph_str_width(buffer);
				if ((x >= pos) && (x <= next_pos))
					return(runptr);
				pos = next_pos;
				runptr = runptr->next;
			}
			fileptr = fileptr->next;
		}
	}
	return(NULL);
}

over_horiz_scroll(x, y)
	double x, y;
{
	if ((y >= 83.0) && (y <= 98.0) && (x >= 15.0))
		return(1);
	return(0);
}

horiz_scroll()
{
	int val, done, win_id;
	double x, y, percentage;

	/* draw over menu here to show mouse buttons */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(300.0, 33.0, "Scrolling through Time.");
	graph_text(300.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(300.0, 11.0, "back   *   quit   *   forward");

	graph_set_input(Sig_main, 1, "b");
	done = 0;
	while (! done) {
		graph_await_event(&win_id, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if (val == MIDBUTTON)
			done = 1;

		if (x > AXISLEFT) {
			if (val == LEFTBUTTON) { /* scroll back */
				percentage = (x - AXISLEFT) / (600.0 - AXISLEFT);
				start_time((Sig_timestart - (Sig_interval * percentage)) *
					Sig_multiplier, "");
			}
			else if (val == RTBUTTON) {
				percentage = (x - AXISLEFT) / (600.0 - AXISLEFT);
				start_time((Sig_timestart + (Sig_interval * percentage)) *
					Sig_multiplier, "");
			}
		}
		else {
			sig_msg("Please hit while cursor over axes.", "","");
			graph_set_layer(0);	/*bkgd*/
			graph_rect(0.0, 0.0, 600.0, 44.0);
			graph_set_layer(2); /*textoverbkgd*/
			graph_char_just_set("cc");
			graph_text(300.0, 33.0, "Scrolling through Time.");
			graph_text(300.0,22.0,
				"Mouse button meanings (from left to right):");
			graph_text(300.0, 11.0, "back   *   quit   *   forward");
		}
	}
	graph_set_input(Sig_main, 0, "bk");
}

over_vert_scroll(x, y)
	double x, y;
{
	if ((x <= 15.0) && (y >= 98.0))
		return(1);
	return(0);
}

vert_scroll()
{
	int val, win_id, done, i;
	double x, y;
	axis_t *temp, *axptr, *which_axis();

	if (Sig_axes == NULL)
		return;

	/* draw over menu here to show mouse buttons */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(300.0, 33.0, "Scrolling Axes Up/Down.");
	graph_text(300.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(300.0, 11.0, "down   *   quit   *   up");

	graph_set_input(Sig_main, 1, "b");
	done = 0;
	while (! done) {
		graph_await_event(&win_id, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if (val == MIDBUTTON)
			done = 1;
		else {
			axptr = which_axis(y);
			if (axptr == NULL)
				sig_msg("Please hit button over an axis.", "","");
			else {
				if (val == LEFTBUTTON) { /* scroll down */
					temp = axptr;
					i = 0;
					while ((i < Sig_axis_ct-1) && (temp->next != NULL)) {
						temp = temp->next;
						i++;
					}
					if (i < Sig_axis_ct - 1){ /* don't have enough to keep
						Sig_axis_ct axes on screen, so go down as far as can */
						Sig_bottom_axis = temp;
						i = 0;
						while (i < Sig_axis_ct-1) {
							temp = temp->previous;
							i++;
						}
						Sig_top_axis = temp;
					}
					else {
						Sig_top_axis = axptr;
						Sig_bottom_axis = temp;
					}
				}
				else { /* scroll up */
					temp = axptr;
					i = 0;
					while ((i < Sig_axis_ct-1) && (temp->previous != NULL)) {
						temp = temp->previous;
						i++;
					}
					if (i < Sig_axis_ct - 1){ /* don't have enough left to fill
												screen */
						Sig_top_axis = temp;
						i = 0;
						while (i < Sig_axis_ct-1) {
							temp = temp->next;
							i++;
						}
						Sig_bottom_axis = temp;
					}
					else {
						Sig_bottom_axis = axptr;
						Sig_top_axis = temp;
					}
				}
				axis_area();
			}
		}
	}
	graph_set_input(Sig_main, 0, "bk");
}

int
select_signames(signames)
	char *signames;
{
	int counter, i, j, start, end;
	int val, done, all;
	double x, y;
	int *selected_nm_indices;
	int popup_id, event_win;

	int name_ct;
	char **names;

	if (GP_mode) {
		name_ct = GP_vecname_ct;
		names = GP_vecnames;
	}
	else {
		name_ct = Sig_signm_ct;
		names = Sig_signms;
	}

	if (name_ct == 0) {
		sig_msg("No signals have yet been loaded","","");
		return(0);
	}

	selected_nm_indices = (int *) calloc (name_ct, sizeof(int));
	for (i= 0; i < name_ct; i++)
		selected_nm_indices[i] = -1;

	/* DRAW OVER MENU HERE TO SHOW MOUSE BUTTONS */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(475.0, 33.0, "Please select signal names from popup.");
	graph_text(475.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(475.0, 11.0, "unselect   *   commit   *   select");

	popup_id = graph_create_fixed_win(375.0, 70.0, 575.0, 475.0, 
		0.0, 0.0, 100.0, 200.0);
	if (popup_id == -1) 
		return(0);

	graph_draw_mode(0);
	graph_set_drawing(popup_id);

	graph_set_layer(8); /* ltblue */
	graph_rect(0.0, 0.0, 100.0, 10.0);
	graph_rect(0.0, 190.0, 100.0, 200.0);
	graph_set_layer(5); /* green */
	graph_rect(0.0, 10.0, 100.0, 20.0);
	graph_set_layer(3); /* textovercolors */
	graph_char_just_set("cc");
	graph_text(25.0, 5.0, "BACK");
	graph_text(75.0, 5.0, "FORWARD");
	graph_text(50.0, 15.0, "ALL");
	graph_text(50.0, 195.0, "SIGNALS");

	graph_char_just_set("ll");
	graph_set_layer(2); /* textoverbkgd */

	start = 0;
	end = SMIN((start + 16), name_ct-1);
	all = 0;
	counter = 0;
	draw_signames_only(start, end, selected_nm_indices);

	signames[0] = '\0';
	graph_set_input(popup_id, 1, "b");
	done = 0;
	while (! done) {
		graph_await_event(&event_win, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Ignore for now. */

		if (y <= 10.0) {
			if (x <= 50.0)  { /* scroll names back */ 
				start = SMAX(0, start - 17);
				end = SMIN((start + 16), name_ct-1);
			}
			else {  /* scroll names forward only if there are more names to
					   be seen */
				if (! (end == name_ct -1)) {
					start = end + 1;
					end = SMIN((start + 16), name_ct-1);
				}
			}
			draw_signames_only(start, end, selected_nm_indices);
		}
		else if (y <= 20.0) {  /* "ALL" selected, regardless of button used */
			signames[0] = '\0';
			done = 1;
			all = 1;
		}
		else if (val == MIDBUTTON) 	/* commit hit */
			done = 1;

		else if (y <= 190.0) { /* name possibly hit */
			i = start + (18 - (((int) y) / 10)); /* index into names */
			if (i < name_ct) { 
				if (val == RTBUTTON) { /* maybe add this signame to the list */
					if (selected_nm_indices[i] == -1) {
						selected_nm_indices[i] = counter;
						/* draw the number next to the name */
						graph_set_layer(3); 
						graph_text_int(1.0, ((double)((int)y / 10) * 10.0)+3.0, 
							counter);
						counter++;
					}
				}
				else { /* delete this name from list */
					/* undraw the number next to the name */
					graph_set_layer(8); 
					graph_text_int(1.0, ((double)((int)y / 10) * 10.0)+3.0, 
						selected_nm_indices[i]);
					selected_nm_indices[i] = -1;
				}
			}
		}
	}
	graph_end_win(popup_id);
	if (GP_mode) {
		graph_set_drawing(GP_currwin_id);
		graph_set_input(GP_currwin_id, 0, "bk");
	}
	else {
		graph_set_drawing(Sig_main);
		graph_set_input(Sig_main, 0, "bk");
	}
	if (all) {
		free ((char *)selected_nm_indices);
		return(1);
	}
	if (counter == 0) {
		free ((char *)selected_nm_indices);
		return(0);
	}
	for (i = 0; i < counter; i++) {
		j = 0;
		while ((j < name_ct) && (selected_nm_indices[j] != i))
			j++;
		if (j != name_ct) {
			strcat(signames, names[j]);
			strcat(signames, " ");
		}
	}
	free ((char *)selected_nm_indices);
	return(1);
}

static
draw_signames_only(start, end, selected_nm_indices)
	int start, end, selected_nm_indices[];
{
	double ypos;
	int i, name_ct;
	char **names;

	if (GP_mode) {
		name_ct = GP_vecname_ct;
		names = GP_vecnames;
	}
	else {
		name_ct = Sig_signm_ct;
		names = Sig_signms;
	}

	if (start == name_ct)
		return;

	graph_draw_mode(0);
	graph_set_layer(0);
	graph_rect(0.0, 20.0, 100.0, 190.0);
	graph_set_layer(8); /* ltblue */
	graph_rect(0.0, 20.0, 7.0, 190.0);
	graph_rect(93.0, 20.0, 100.0, 190.0);

	ypos = 183.0;
	for (i = start; i <= end; i++) {
		graph_set_layer(2); /* textoverbkgd */
		graph_text(10.0, ypos, names[i]);
		if (selected_nm_indices[i] != -1) {
			graph_set_layer(3); /* textovercolors */
			graph_text_int(1.0, ypos, selected_nm_indices[i]);
		}
		ypos = ypos - 10.0;
	}
}

int
select_runnames(runnames)
	char *runnames;
{
	struct holder_t {
		int selected;
		double startx, endx;
		run_t *runptr;
	} *holder;
	comm_file_t *fileptr;
	run_t *runptr;
	int win_id;
	double pos, graph_str_width();
	char buffer[BUFSIZE];
	int found, selected, i, val, done, all, counter;
	double x, y;
	char ev, graph_await_event();

	if (Sig_files == NULL) {
		sig_msg("No runs have yet been loaded","","");
		return(0);
	}

	graph_set_drawing(Sig_main);

	counter = 0;
	fileptr = Sig_files;
	while (fileptr != NULL) {
		runptr = fileptr->runs;
		while (runptr != NULL) {
			counter++;
			runptr = runptr->next;
		}
		fileptr = fileptr->next;
	}
	holder = (struct holder_t *) calloc (counter, sizeof(struct holder_t));
	i= 0; 
	pos = 22.0 + graph_str_width("Runs: ");
	fileptr = Sig_files;
	while (fileptr != NULL) {
		runptr = fileptr->runs;
		while (runptr != NULL) {
			holder[i].selected = 0;
			holder[i].runptr = runptr;
			holder[i].startx = pos;
			buffer[0] = '\0';
			strcat(buffer, runptr->name);
			strcat(buffer, " ");
			pos = pos + graph_str_width(buffer);
			holder[i].endx = pos;
			runptr = runptr->next;
			i++;
		}
		fileptr = fileptr->next;
	}

	/* draw over menu here to show mouse buttons */
	graph_draw_mode(0);
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(150.0, 33.0, "Please select run names from Runs list above.");
	graph_text(150.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(150.0, 11.0, "unselect   *   commit   *   select");
	graph_set_layer(5); /* green */
	graph_rect(280.0, 15.0, 310.0, 25.0);
	graph_set_layer(3);
	graph_text(295.0, 20.0, "ALL");

	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_set_input(Sig_main, 1, "b");
	all = done = 0;
	while (! done) {
		ev = graph_await_event(&win_id, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if (ev == 's')
			;	
		else if ((y >= 15.0) && (y <= 25.0)) {
			if ((x >= 280.0) && (x <= 310.0)) 
				all = done = 1;
		}
		else if (val == MIDBUTTON)
			done = 1;

		else if ((y >= 58.0) && (y <= 68.0)) { /*perhaps over a name */
			i = 0;
			found = 0;
			while ((i < counter) && (!found)) {
				if ((x >= holder[i].startx) && (x <= holder[i].endx)) {
					found = 1;
					if ((val == RTBUTTON) && (! holder[i].selected)) {
						holder[i].selected = 1;
						graph_draw_mode(0);
						graph_set_layer(holder[i].runptr->layer);
						graph_line(holder[i].startx, 58.0, holder[i].endx,
							58.0);
				/*		graph_text(holder[i].startx, 60.0, 
							holder[i].runptr->name);
*/
					}
					else if ((val == LEFTBUTTON) && holder[i].selected) {
						holder[i].selected = 0;
						graph_draw_mode(1);
						graph_set_layer(holder[i].runptr->layer);
						graph_line(holder[i].startx, 58.0, holder[i].endx,
							58.0);
				/*		graph_text(holder[i].startx, 60.0, 
							holder[i].runptr->name);
*/
					}
				}
				i++;
			}
		}
	}
	graph_set_input(Sig_main, 0, "bk");
	graph_draw_mode(0);
	runnames[0] = '\0';
	if (all) {
		free ((char *)holder);
		return(1);
	}
	selected = 0;
	for (i = 0; i < counter; i++) {
		if (holder[i].selected) {
			selected++;
			strcat(runnames, holder[i].runptr->name);
			strcat(runnames, " ");
		}
	}
	free ((char *)holder);
	if (selected == 0) 
		return(0);
	return(1);
}

static int
select_axes(axes)
	char *axes;
{
	struct holder_t {
		int selected;
		double top, bot;
		axis_t *axptr;
	} *holder;
	axis_t *axis_ptr;
	int i, win_id, found, val, done, all, selected;
	double x, y, axspace, axtop;

	if (Sig_axis_ct == 0) {
		sig_msg("No axes have yet been created","","");
		return(0);
	}

	holder = (struct holder_t *) calloc (Sig_axis_ct, sizeof(struct holder_t));
	axspace = 395.0 / Sig_axis_ct;
	axtop = 495.0;
	axis_ptr = Sig_top_axis;
	for (i= 0; i < Sig_axis_ct; i++) {
		holder[i].selected = 0;
		holder[i].axptr = axis_ptr;
		holder[i].top = axtop;
		axtop = axtop - axspace;
		holder[i].bot = axtop;
		axis_ptr = axis_ptr->next;
	}
	holder[0].top = 500.0;
	holder[Sig_axis_ct-1].bot = 98.0;

	/* draw over menu here to show mouse buttons */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(150.0, 33.0, "Please select axes by clicking over them.");
	graph_text(150.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(150.0, 11.0, "unselect   *   commit   *   select");
	graph_set_layer(5); /* green */
	graph_rect(280.0, 15.0, 310.0, 25.0);
	graph_set_layer(3);
	graph_text(295.0, 20.0, "ALL");

	graph_set_input(Sig_main, 1, "b");
	graph_set_layer(2); /*textoverbkgd*/
	all = done = 0;
	while (! done) {
		graph_await_event(&win_id, &val, &x, &y);
		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if ((y >= 15.0) && (y <= 25.0)) {
			if ((x >= 280.0) && (x <= 310.0)) 
				all = done = 1;
		}
		else if (val == MIDBUTTON)
			done = 1;

		else if (y >= 98.0) { /*over an axis */
			found = 0;
			i = 0;
			while ((i < Sig_axis_ct) && (!found)) {
				if ((y >= holder[i].bot) && (y <= holder[i].top)) {
					found = 1;
					if ((val == RTBUTTON) && (! holder[i].selected)) {
						holder[i].selected = 1;
						graph_draw_mode(0);
						graph_text(17.0, holder[i].bot + (axspace * 0.6), "*");
					}
					else if ((val == LEFTBUTTON) && holder[i].selected) {
						holder[i].selected = 0;
						graph_draw_mode(1);
						graph_text(17.0, holder[i].bot + (axspace * 0.6), "*");
					}
				}
				i++;
			}
		}
	}
	graph_set_input(Sig_main, 0, "bk");
	graph_draw_mode(0);
	axes[0] = '\0';
	if (all) {
		free ((char *)holder);
		return(1);
	}
	selected = 0;
	for (i = 0; i < Sig_axis_ct; i++) {
		if (holder[i].selected) {
			selected++;
			strcat(axes, holder[i].axptr->name);
			strcat(axes, " ");
		}
	}
	free ((char *)holder);
	if (selected == 0) 
		return(0);
	return(1);

}

int
prompt_for_name(win, prompt, name, first_word)
	char *prompt, *name;
	int win, first_word;
{
	int val, win_id;
	double x, y;
	char *next_token();

	if (Sig_notype) {
		name[0] = '\0';
		return(1);
	}
	graph_set_drawing(win);
	graph_set_layer(0);
	graph_draw_mode(0);
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_char_just_set("ll");
	graph_set_layer(2);
	graph_text(10.0, 22.0, prompt);

	graph_await_event(&win_id, &val, &x, &y);
	graph_set_layer(0);
	graph_rect(0.0, 0.0, 600.0, 44.0);

	read_and_echo(win, (char)val, name);
	if (first_word) 
		name = next_token(&name);
	if (name[0] == '\0')
		return(0);
	return(1);
}

interactive_shuffle()
{
	axis_t *picked_up, *insert_above, *which_axis();
	int val, done, win_id;
	double x, y;

	if (Sig_axis_ct == 0) {
		sig_msg("No axes have yet been created","","");
		return;
	}

	/* draw over menu here to show mouse buttons */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(150.0, 33.0, "Please select axes by clicking over them.");
	graph_text(150.0, 22.0, "Mouse button meanings (from left to right):");
	graph_text(150.0, 11.0, "put down   *   stop   *   pick up");

	picked_up = insert_above = NULL;
	graph_set_input(Sig_main, 1, "b");
	graph_set_layer(2); /*textoverbkgd*/
	done = 0;
	while (! done) {
		graph_await_event(&win_id, &val, &x, &y);
		graph_set_layer(0);	/*bkgd*/ /* clear error area, just in case */
		graph_rect(250.0, 0.0, 600.0, 44.0);

		/* If we get back a screenscrewed event, we're in big trouble. 
		   Just ignore for now. */
		if (val == MIDBUTTON)
			done = 1;

		else if (val == LEFTBUTTON) { /* put down the axis */
			if (y >= 98.0) { /*over an axis */
				if (picked_up == NULL)  {
					graph_set_layer(2); /*textoverbkgd*/
					graph_text(400.0, 30.0,
						"Nothing to put down--pick up one first.");
				}
				else {
					insert_above = which_axis(y);
					swap_axes(picked_up, insert_above);
					axis_area();
					picked_up = NULL;
				}
			}
		}
		else if (val == RTBUTTON) {
			if (y >= 98.0) { /*over an axis */
				if (picked_up != NULL)  {
					graph_set_layer(2); /*textoverbkgd*/
					graph_text(400.0, 30.0,
						"Already picked one up--put it down first.");
				}
				else {
					picked_up = which_axis(y);
					graph_set_layer(2); /*textoverbkgd*/
					graph_text(17.0, y, "*");
				}
			}
		}
	}
	graph_set_input(Sig_main, 0, "bk");
}

axis_t *
which_axis(y)
	double y;
{
	axis_t *picked_up;
	int i, index;
	double axspace;

	axspace = 395.0 / Sig_axis_ct;

	if (y >= 495.0)
		picked_up = Sig_top_axis;
	else if (y <= 100.0)
		picked_up = Sig_bottom_axis;
	else {
		index = (int) ((495.0 - y) / axspace);
		picked_up = Sig_top_axis;
		for (i = 0; i < index; i++)	
			picked_up = picked_up->next;
	}
	return(picked_up);
}

reset_screen()
{
	axis_t *axptr;
	double newinterval, maxtime_of_all_runs();

	Sig_top_axis = Sig_axes;
	Sig_axis_ct = 0;
	axptr = Sig_bottom_axis = Sig_axes;
	while (axptr != NULL) {
		Sig_bottom_axis = axptr;
		Sig_axis_ct++;
		axptr = axptr->next;
	}

	Sig_timestart = 0.0;
	newinterval = maxtime_of_all_runs();
	if (newinterval > 0.0) {
		Sig_interval = newinterval;
		Sig_timeend = Sig_timestart + Sig_interval;
	}

	/* set time interval to max for all axes.  start_time will also
	   redraw the axis area. */
	start_time(0.0, "");
}

zoom()
{
	axis_t *axptr, *which_axis();
	double x1, y1, x2, y2;
	double temp, lastx2, lasty1;
	int win_id, val;
	double newstartime, timeperunit;

	if (Sig_axis_ct == 0) {
		sig_msg("No axes have yet been created","","");
		return;
	}

	/* draw over menu here to show mouse buttons */
	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_char_just_set("cc");
	graph_text(300.0, 33.0, "Zoom.");
	graph_text(300.0, 22.0, "Use rubber-band box to set focus.");
	graph_text(300.0, 11.0, "Place upper left corner now.");

	/* first give upper left cursor so that user can set new upper left */
	graph_cursor("upperleft");
	graph_set_input(Sig_main, 1, "b");
	/*wait for buttonhit that places corner*/
	graph_await_event(&win_id, &val, &x1, &y2);

	graph_set_layer(0);	/*bkgd*/
	graph_rect(0.0, 0.0, 600.0, 44.0);
	graph_set_layer(2); /*textoverbkgd*/
	graph_text(300.0, 33.0, "Zoom.");
	graph_text(300.0, 22.0, "Now place lower right corner.");

	graph_set_layer(10); /* leftvert */
	graph_cursor("lowerright");
	graph_set_input(Sig_main, 1, "bm");
	lastx2 = x1; lasty1 = y2;

	while (graph_await_event(&win_id, &val, &x2, &y1) == 'm') {
		graph_draw_mode(1);
		graph_line_rect(x1, lasty1, lastx2, y2);
		graph_draw_mode(0);
		graph_line_rect(x1, y1, x2, y2);
		lastx2 = x2; lasty1 = y1;
	}
	graph_draw_mode(1);
	graph_line_rect(x1, lasty1, lastx2, y2);

	/* if the bounding box doesn't bound anything, then get the
	   heck out of here. */
	if ((y1 == y2) || (x1 == x2)) {
		graph_cursor("pointer");
		graph_set_input(Sig_main, 0, "bk");
		return;
	}

	/* now find the axes trapped under the bounding box */
	if (y1 > y2) {
		temp = y2; y2 = y1; y1 = temp; }
	axptr = which_axis(y2);
	Sig_bottom_axis = which_axis(y1);
	Sig_top_axis = axptr;
	Sig_axis_ct = 1;
	axptr = Sig_top_axis;
	while (axptr != Sig_bottom_axis) {
		Sig_axis_ct++;
		axptr = axptr->next;
	}

	/* next recalculate the new time interval */
	if (x1 > x2) {
		temp = x2; x2 = x1; x1 = temp; }
	timeperunit = Sig_interval / (AXISRIGHT - AXISLEFT);
	newstartime = Sig_timestart + ((x1 - AXISLEFT) * timeperunit);
	Sig_interval = (x2 - x1) * timeperunit;

	/* Note that the call to start_time will redraw the axis_area */
	start_time(newstartime * Sig_multiplier, "");

	graph_cursor("pointer");
	graph_set_input(Sig_main, 0, "bk");
}
