/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <ctype.h>
#include <sys/param.h>
#include "gp.h"

extern int Sig_interactive_mode;
extern int Sig_decimals;
extern comm_file_t *Sig_files;
extern int GP_mode;

void yrange();

/* First try to read a .sigview in the home directory.  Then try to read
   a .sigview in the current directory.  Return 0 if no cmdfiles read
   successfully, else return 1.   Side effects include the filling of
   all structures in sigview.h. */
int
read_cmdfile()
{
	FILE *cmdfile, *open_fl();
	int read_one;
	char cmd[BUFSIZE];
	/* MAXPA. defined in sys/param.h */
	char workdir[MAXPATHLEN], homedir[MAXPATHLEN]; 
	extern char *getenv();

	read_one = 0;

	if ((cmdfile = open_fl("~/.sigview", "r")) != NULL) {
		while (get_a_cmd(cmdfile, cmd)) {
			if (GP_mode)
				gp_parse_and_execute(cmd);
			else parse_and_execute(cmd);
		}
		read_one = 1;
	}

	/* if home directory not equal to current directory, then try to 
		execute a .sigview  here in the current directory. */
/*	getwd(workdir); 	wrr 1/7/92 - no getwd on HPUX */
	strcpy(workdir, getenv("PWD"));
	strcpy(homedir, getenv("HOME"));
	if (strcmp(homedir, workdir) != 0) {
		if ((cmdfile = open_fl(".sigview", "r")) != NULL) {
			while (get_a_cmd(cmdfile, cmd)) {
				if (GP_mode)
					gp_parse_and_execute(cmd);
				else parse_and_execute(cmd);
			}
			read_one = 1;
		}
	}
	return(read_one);
}

/* Return 0 if a command cannot be gotten, else return 1.  Cmd comes back
   with each token separated by a single space and the end of the command
   is indicated by finding a \0. */
int
get_a_cmd(cmdfile, cmd)
	FILE *cmdfile;
	char cmd[];
{
	char buf[BUFSIZE];
	int bufptr, cmdptr;
	int semicolon_gotten, space_gotten;

	if (fgets(buf, BUFSIZE, cmdfile) == NULL) 
		return(0);
	bufptr = cmdptr = 0;
	semicolon_gotten = space_gotten = 0;

	while (!semicolon_gotten) {
		for (bufptr = 0; bufptr < strlen(buf); bufptr++) {
			if (buf[bufptr] == ';') {
				semicolon_gotten = 1;
				cmd[cmdptr] = '\0';
				return(1);
			}
			else if (isspace(buf[bufptr])) {
				if (! space_gotten) {
					cmd[cmdptr++] = ' ';
					space_gotten = 1;
				}
			}
			else {
				cmd[cmdptr++] = buf[bufptr];
				space_gotten = 0;
			}
		}

		if (! semicolon_gotten) {
			if (fgets(buf, BUFSIZE, cmdfile) == NULL) 
				return(0);
			bufptr = 0;
		}
	}
	return(1);
}

parse_and_execute(cmd)
	char cmd[];
{
	char *cmdname, *next_token();

	if (!Sig_interactive_mode) 
		sig_msg("Command: ", cmd, "");

	cmdname = next_token(&cmd);
	if (cmdname[0] == '\0')
		return;

	if (strncmp(cmdname, "create", 2) == 0)
		create_p(cmd);
	else if (strncmp(cmdname, "mode", 1) == 0)
		mode_p(cmd);
	else if (strncmp(cmdname, "load", 3) == 0)
		load_p(cmd);
	else if (strncmp(cmdname, "addsigs", 4) == 0)
		add_p(cmd);
	else if (strncmp(cmdname, "combine", 4) == 0)
		combine_p(cmd);
	else if (strncmp(cmdname, "compare", 4) == 0)
		compare_p(cmd);
	else if (strncmp(cmdname, "convert", 3) == 0)
		convert_p(cmd);
	else if (strncmp(cmdname, "copy", 3) == 0)
		copy_p(cmd);
	else if (strncmp(cmdname, "delaxes", 4) == 0)
		delaxes_p(cmd);
	else if (strncmp(cmdname, "delsigs", 4) == 0)
		delsigs_p(cmd);
	else if (strncmp(cmdname, "ft", 3) == 0)
		ft_p(cmd);
	else if (strncmp(cmdname, "help", 1) == 0)
		help(cmd);
	else if (strncmp(cmdname, "interval", 1) == 0)
		interval_p(cmd);
	else if (strncmp(cmdname, "logy", 3) == 0)
		logy_p(cmd);
	else if (strcmp(cmdname, "n") == 0)
		search_next();
	else if (strncmp(cmdname, "notype", 2) == 0)
		notype();
	else if (strncmp(cmdname, "nums", 2) == 0)
		nums();
	else if (strcmp(cmdname, "p") == 0)
		search_previous();
	else if (strncmp(cmdname, "precision", 2) == 0)
		precision();
	else if (strncmp(cmdname, "quit", 1) == 0)
		sigview_quit();
	else if (strncmp(cmdname, "run", 2) == 0)
		run_p(cmd);
	else if (strncmp(cmdname, "redraw", 2) == 0)
		draw_screen();
	else if (strncmp(cmdname, "general", 2) == 0)
		open_other_mode();
	else if (strncmp(cmdname, "search", 2) == 0)
		search_p(cmd);
	else if (strncmp(cmdname, "show", 2) == 0)
		show_p(cmd);
	else if (strncmp(cmdname, "snapshot", 2) == 0)
		snapshot_p(cmd);
	else if (strncmp(cmdname, "time", 1) == 0)
		time_p(cmd);
	else if (strncmp(cmdname, "units", 1) == 0)
		units_p(cmd);
	else if (strncmp(cmdname, "xscale", 2) == 0)
		xscale_p(cmd);
	else if (strncmp(cmdname, "yaxes", 2) == 0)
		yaxes();
	else if (strncmp(cmdname, "yrange", 2) == 0) 
		range_p(cmd, yrange);
	else if (strncmp(cmdname, "format", 1) == 0) 
		format_p(cmd);
	else sig_msg("Command ", cmdname, " not identified");
}

static
logy_p(arglist)
	char *arglist;
{
	logy(arglist);
}

range_p(arglist, rangeproc)
	char *arglist;
	void (*rangeproc)();
{
	double ymin, ymax;
	int yminspecified, ymaxspecified;
	char *next_token(), *token;
	int done;

	done = yminspecified = ymaxspecified = 0;
	token = next_token(&arglist);

	while (!done) {
		switch (*token) {
			case '>':
				if (token[1] == '\0')
					token = next_token(&arglist);
				else token = &(token[1]);
				if (sscanf(token, "%lf", &ymin) != 1)  {
					sig_msg("Range:  ", token, " is not a numeric value");
					return;
				}
				else {
					yminspecified = 1;
					token = next_token(&arglist);
				}
				break;
			case '<':
				if (token[1] == '\0')
					token = next_token(&arglist);
				else token = &(token[1]);
				if (sscanf(token, "%lf", &ymax) != 1) { 
					sig_msg("Range:  ", token, " is not a numeric value");
					return;
				}
				else {
					ymaxspecified = 1;
					token = next_token(&arglist);
				}
				break;
			default:
				done = 1;
				break;
		}
	}
	(*rangeproc)(arglist, yminspecified, ymaxspecified, ymin, ymax);
}

static
xscale_p(arglist)
	char *arglist;
{
	double scale;
	char *next_token(), *token;

	token = next_token(&arglist);
	if (*token == '\0')
		return;

	if (sscanf(token, "%lf", &scale) == 1) 
		xscale(scale);
	else sig_msg("Xscale: scale value ",token," is not a valid number");
}

static
units_p(arglist)
	char *arglist;
{
	double dunits;
	char *next_token(), *token;

	token = next_token(&arglist);
	if (*token == '\0')
		return;

	if (sscanf(token, "%lf", &dunits) == 1) 
		units(dunits);
	else sig_msg("Units:  ",token," is not a valid number");
}

static
time_p(arglist)
	char *arglist;
{
	char *time, *next_token();
	double timebegin;
	char *axnms, *namelist();

	time = next_token(&arglist);
	if (*time == '\0') {
		sig_msg("Time:  missing time","","");
		return;
	}
	if (sscanf(time, "%lf", &timebegin) != 1)  {
		sig_msg("Time:  ",time," is not a valid number\n");
		return;
	}
	axnms = namelist(&arglist, 'a');
	start_time(timebegin, axnms);
}

/* this is called from both the Sigview and the General Plot parsers */
snapshot_p(arglist)
	char *arglist;
{
	char *token, *next_token();

	token = next_token(&arglist);
	if (*token == '\0') {
		snapshot("");
	}
	else snapshot(token);
}

show_p(arglist)
	char *arglist;
{
	char *signms, *runnms;
	char *namelist();

	signms = namelist(&arglist, 's');
	runnms = namelist(&arglist, 'f');

	show(signms, runnms);
}

static
search_p(arglist)
	char *arglist;
{
	char *next_token(), *axnm, *token, *pattern, *temp;
	int individual;
	double btime, etime;
	int done, bflag, eflag;
	int i,j, numpats;

	token = next_token(&arglist);
	if (*token == '\0') {
		sig_msg("Search: axisname missing","","");
		return;
	}
	else axnm = token;

	individual = bflag = eflag = done = 0;
	token = next_token(&arglist);
	while (!done) {
		if (strcmp(token, "-i") == 0) {
			individual = 1;
			token = next_token(&arglist);
		}
		else if (strcmp(token, "-b") == 0) {
			token = next_token(&arglist);
			if (token[0] == '\0') {
				sig_msg("Search: need to specify a begin time","","");
				return;
			}
			if (sscanf(token, "%lf", &btime) != 1) {
				sig_msg("Search: begin time ",token," is not a number");
				return;
			}
			bflag = 1;
			token = next_token(&arglist);
		}
		else if (strcmp(token, "-e") == 0) {
			token = next_token(&arglist);
			if (token[0] == '\0') {
				sig_msg("Search: Need to specify an end time","","");
				return;
			}
			if (sscanf(token, "%lf", &etime) != 1) {
				sig_msg("Search:  end time ",token," is not a number");
				return;
			}
			eflag = 1;
			token = next_token(&arglist);
		}
		else if (*token == '-') {
			sig_msg("Search:  invalid option ",token," encountered");
			return;
		}
		else { /* we must be into the pattern section */
			done = 1;
			if (*token == '\0') {
				sig_msg("Search:  need to specify a pattern","","");
				return;
			}	
			pattern = token;
			numpats = 0;
			while (*token != '\0') {
				numpats++;
				token = next_token(&arglist);
			}	
		}
	}
	/* since next_token inserts a null after each token, remove it from
	   the pattern list and replace with a blank. */
	temp = pattern;
	for(i = 0; i < numpats-1; i++) {
		j = strlen(temp);
		temp[j] = ' ';
		temp = temp + j + 1;
	}
	/* make sure all elements in pattern are lowercase so that
	   they may match with the lowercase data stored for each run. */
	temp = pattern;
	j = 0;
	while(temp[j] != '\0') {
		temp[j] = LOWERCASE(temp[j]);
		j++;
	}	
	do_search(axnm, individual, btime, bflag, etime, eflag, pattern);
}
			
static
interval_p(arglist)
	char *arglist;
{
	char *token, *next_token();
	double interv;

	token = next_token(&arglist);
	if (sscanf(token, "%lf", &interv) == 1)
		do_interval(interv, 0);
	else if (strcmp(token, "max") == 0)
		do_interval(0.0, 1);
	else sig_msg("Interval:   missing argument","","");
}

static
delsigs_p(arglist)
	char *arglist;
{
	char *axnms, *signms, *runnms;
	char *namelist();

	axnms = namelist(&arglist, 'a');
	signms = namelist(&arglist, 's');
	runnms = namelist(&arglist, 'f');

	delsigs(axnms, signms, runnms);
}

static
delaxes_p(arglist)
	char *arglist;
{
	delete_axes(arglist);
}

extern run_t *Desired_run;

ft_p(arglist)
	char *arglist;
{
	char *vecname, *token, *next_token();
	int inverse, quadratic;
	int nfreq, create_file;
	double fund_freq;
	double *Y, *X;
	int numpts;
	char vecnmout[SIGNMSIZE];
	struct vvec_s vecrec, is_vectorname();

	if (Sig_files == NULL) {
		sig_msg("Ft:  no data loaded on which to run transform.","","");
		return;
	}

	vecname = next_token(&arglist);
	if (*vecname == '\0') {
		sig_msg("Ft:  vectorname missing","","");
		return;
	}

	Desired_run = NULL;
	vecrec = is_vectorname(vecname);
	if (vecrec.num_vals == 0)
		return;
	Y = vecrec.values;
	X = vecrec.run->data.analog[0];
	numpts = vecrec.num_vals;

	inverse = 0;
	quadratic = 0;
	nfreq = 16;
	fund_freq = 0.0;
	create_file = 0;

	token = next_token(&arglist);
	while (token[0] != '\0') {
		if (strncmp(token, "-i", 2) == 0)
			inverse = 1;
		else if (strncmp(token, "-q", 2) == 0)
			quadratic = 1;
		else if (strncmp(token, "-nfreq", 5) == 0) {
			token = next_token(&arglist);
			if (sscanf(token, "%d", &nfreq) != 1) {
				sig_msg("Ft:  nfreq not given as an integer","","");
				return;
			}
			else if (nfreq < 1) {
				sig_msg("Ft:  nfreq < 1","","");
				return;
			}
		}
		else if (strncmp(token, "-ffreq", 5) == 0) {
			token = next_token(&arglist);
			if (sscanf(token, "%lf", &fund_freq) != 1) {
				sig_msg("Ft:  ffreq not given as a number","","");
				return;
			}
			else if (fund_freq < 0.0) {
				sig_msg("Ft:  fund_freq < 0","","");
				return;
			}
		}
		else if (strncmp(token, "-f", 2) == 0)
			create_file = 1;
		else sig_msg("Ft:  ",token," is invalid option");

		token = next_token(&arglist);
	}

	/* clean up name by removing any single quotes and preceding run name
	   with slashes. */
	just_vecname(vecname, vecnmout);
	do_ft(vecnmout, X, Y, numpts, nfreq, inverse, fund_freq, 
			quadratic, create_file);
}

static
just_vecname(dirty, clean)
	char *dirty, *clean;
{
	char *vecnm;

	strcpy(clean, dirty);

	/* If there are single quotes, remove single quotes */
	if (clean[0] == '\'') {
		strcpy(clean, &(clean[1]));
		clean[strlen(clean)-1] = '\0';
	}
	if (vecnm = rindex(clean, '/'))  /* then we have a runname * */
		/* save only the part after that last slash */
		strcpy(clean, ++vecnm); 
}

static
copy_p(arglist)
	char *arglist;
{
	copy(arglist);
}

static
convert_p(arglist)
	char *arglist;
{
	char axnm[30], conversion[30];

	if (sscanf(arglist, "%s %s", axnm, conversion) == 2) 
		convert_axis(axnm, conversion);
}

static
compare_p(arglist)
	char *arglist;
{
	char *signms, *runnms;
	char *namelist();

	signms = namelist(&arglist, 's');
	runnms = namelist(&arglist, 'f');
	
	compare(signms, runnms);
}

static
combine_p(arglist)
	char *arglist;
{
	char *next_token(), *axnm;

	axnm = next_token(&arglist);
	if (*axnm == '\0') {
		sig_msg("Combine:  axisname missing","","");
		return;
	}
	combine(axnm, arglist);
}

static
add_p(arglist)
	char *arglist;
{
	char *axnms, *signms, *runnms;
	char *namelist();

	axnms = namelist(&arglist, 'a');
	signms = namelist(&arglist, 's');
	runnms = namelist(&arglist, 'f');

	addsigs(axnms, signms, runnms);
}

static
create_p(arglist)
	char *arglist;
{
	char *axnm, *signms, *runnms;
	char *next_token(), *namelist();

	axnm = next_token(&arglist);
	if (axnm[0] == '\0') {
		sig_msg("Create:  axisname is required.","","");
		return;
	}	
	signms = namelist(&arglist, 's');
	runnms = namelist(&arglist, 'f');

	create(axnm, signms, runnms);
}

static
mode_p(arglist)
	char *arglist;
{
	char axnm[30], mode[30], option[30], conversion[30];

	if (sscanf(arglist, "%s %s %s %s", axnm, mode, option, conversion) == 4) {
		if ((option[0] == '-') && (option[1] = 'c'))
			change_mode(axnm, mode, conversion);
		else sig_msg("Mode:  ",option," is invalid option");
	}
	else if (sscanf(arglist, "%s %s", axnm, mode) == 2)
		change_mode(axnm, mode, "hex");
}

static
load_p(arglist)
	char *arglist;
{
	char *filenm, *next_token();

	filenm = next_token(&arglist);
	while (filenm[0] != '\0') {
		load_simfile(filenm);
		filenm = next_token(&arglist);
	}
	if (Sig_interactive_mode)
		draw_screen();
}

static
run_p(arglist)
	char *arglist;
{
	char *cmdfilenm, *next_token();
	FILE *cmdfile, *open_fl();
	char cmd[BUFSIZE];

	cmdfilenm = next_token(&arglist);
	while (cmdfilenm[0] != '\0') {
		if ((cmdfile = open_fl(cmdfilenm, "r")) != NULL) {
			while (get_a_cmd(cmdfile, cmd)) {
				if (GP_mode)
					gp_parse_and_execute(cmd);
				else parse_and_execute(cmd);
			}
		}
		cmdfilenm = next_token(&arglist);
	}
}

/* Returns a pointer to the beginning of the namelist part of the arglist.
   The list is terminated by a '\0'.  Each name is separated by a single
   blank.  No preceeding blank exists before the first name . */

char *
namelist(arglist, option)
	char **arglist;
	char option;
{
	char *nms;
	int i;

	if (strlen(*arglist) < 4)
		return("");

	if (((*arglist)[0] == '-') && ((*arglist)[1] == option) &&
			((*arglist)[2] == ' ')) {

		nms = &((*arglist)[3]);
		i=0;
		while ((nms[i] != '-') && (nms[i] != '\0'))
			i++;

		if (nms[i] == '\0')
			*arglist = &(nms[i]);
		else {	/* nms[i] == '-' */
			nms[i-1] = '\0';
			*arglist = &(nms[i]);
		}
		return(nms);
	}
	
	else return("");
}

/* Adjusts arglist pointer such that it points to the spot after the
   next token.  Token is returned pointing to the token. */
char *
next_token(arglist)
	char **arglist;
{
	int i;
	char *token;

	if ((*arglist)[0] == ' ')
		token = &((*arglist)[1]);
	else token = &((*arglist)[0]);

	i=0;
	while ((token[i] != ' ') && (token[i] != '\0'))
		i++;
	if (token[i] != '\0') {
		token[i] = '\0';
		i++;
	}
	*arglist = &(token[i]);

	return(token);
}

format_p(arglist)
	char *arglist;
{
	int dec_places;
	char *formats;
	char *next_token(), *namelist();

	formats = next_token(&arglist);
	if (formats[0] == '\0') {
		sig_msg("Format:  Number of decimal places is required.","","");
		return;
	}	

	sscanf(formats,"%d",&dec_places);
	Sig_decimals = dec_places;
		if (Sig_interactive_mode)
				draw_screen();

}
