/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */

#include "hardcopy.h"

#define PSWD	792 	/* 11 inches in postscript units */
#define PSHT	612		/* 8.5 inches in postscript units */
#define PSSTACKLIM	475	/* make sure we don't overload the Postscript stack */

extern func_t Hard_funcs;
extern int H_lmap[];
extern H_cmap_t H_cmap[];

FILE	*Ps_spool;
char Ps_fonttype[12];
int Ps_fontsize;
int Ps_color;
int Ps_devtype;
char Ps_filenm[256];
char Ps_cmd[256];

int
ps_init(devtype, astring)
	char *devtype, *astring;
{
	FILE *open_fl();

	/* determine whether we send to a real file or to a temp file to be
	   sent to an actual printer. */
	Ps_devtype = TFILE;

	if (strcmp(devtype, "ps") == 0) {
		unique_tempnm(Ps_filenm, "#Sig_psXXXXXX");
		if ((Ps_spool = open_fl(Ps_filenm, "w")) == NULL) {
			sig_msg("Cannot open temp file ",Ps_filenm," for Postscript plot");
			return(0);
		}
		sprintf(Ps_cmd, "%s %s ; rm %s", astring, Ps_filenm, Ps_filenm);
		Ps_devtype = TDIRECT;
	}
	else  {
		strip_crap(astring);
		if ((Ps_spool = open_fl(astring, "w")) == NULL) {
			sig_msg("Cannot open ",astring, "for Postscript dump");
			return(0);
		}
		else strcpy(Ps_filenm, astring);
	}

	fprintf(Ps_spool, "%%!\n");
	fprintf(Ps_spool, "%%%%Creator: Sigview\n");
	fprintf(Ps_spool, "%%%%Pages: (atend)\n");

	strcpy(Ps_fonttype, "");
	ps_font_set("medium");

	Ps_color = 0;
	return(1);
};

ps_color_map()
{
	fprintf(Ps_spool, "/P0 {[4 1 1 1 5 1 1 1] 0 setdash} def\n");
	fprintf(Ps_spool, "/P1 {[] 0 setdash} def\n");
/*	fprintf(Ps_spool, "/P2 {[8 1 1 1 1 1 1 1] 0 setdash} def\n"); */
	fprintf(Ps_spool, "/P2 {[] 0 setdash} def\n");
	fprintf(Ps_spool, "/P3 {[3 1] 0 setdash} def\n");
	fprintf(Ps_spool, "/P4 {[1 1] 0 setdash} def\n");
	fprintf(Ps_spool, "/P5 {[] 0 setdash} def\n");
/*	fprintf(Ps_spool, "/P5 {[4 4] 0 setdash} def\n"); */
	fprintf(Ps_spool, "/P6 {[2 2] 0 setdash} def\n");
	fprintf(Ps_spool, "/P7 {[1 3] 0 setdash} def\n");
}

static double Gx1, Gy1;		/* current lower left coords of the "world" of
							   the graphics program using graph.c */
static double Gx2, Gy2;
static double GtoX_scalex, GtoX_scaley;
static double XtoG_scalex, XtoG_scaley;

/* I would love the use the "rotate" "scale" etc commands in Postscript
   except that it screws up linewidth and god knows what else.  This makes
   them nearly worthless.  So I need to do my own transforms so as to send
   coords sensible to Postscripts 0,0,792,612 system. */

/* This routine and GtoPs both assume that x1,y1 will be the lower left
   corner of the picture and that y increases from bottom to top */

int
ps_window_set(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	if (x1 == x2) {
		fprintf(stderr,
			"ps_window_set: not resetting scale -- window has no width\n");
		return(0);
	}
	if (y1 == y2) {
		fprintf(stderr,
			"ps_window_set: not resetting scale -- window has no height\n");
		return(0);
	}
/*
	Gx1 = SMIN(x1,x2);
	Gy1 = SMIN(y1,y2);
	Gx2 = Gx1 + (ABS((x2 - x1)));   
	Gy2 = Gy1 + (ABS((y2 - y1)));
*/
	Gx1 = x1;
	Gx2 = x2;
	Gy1 = y1;
	Gy2 = y2;

	GtoX_scalex = PSWD/(x2 - x1);   
	GtoX_scaley = PSHT/(y2 - y1);
	XtoG_scalex = 1.0 / GtoX_scalex;
	XtoG_scaley = 1.0 / GtoX_scaley;

	return(1);
}

/* user coordinate (tx,ty) to ps coordinate (x,y) */
GtoPs(x, y, tx, ty)
	double x, y;
	int *tx, *ty;
{
/*
	if (x > Gx2) x = Gx2;
	if (y > Gy2) y = Gy2;
*/
	/* Here we shift the picture down and to the left (by sub'ing 20 from
	   x and adding 10 to y).  Why?  Our Postscript printer doesn't actually
	   print right up to the edge of the paper....Obviously, your mileage
	   may vary. */
	*tx = (int)(((y - Gy1) * GtoX_scaley) - 20.0);
	*ty = (int)((PSWD - ((x - Gx1) * GtoX_scalex)) + 10.0);
}

ps_font_set(type)
	char *type;
{
	if ((strcmp(Ps_fonttype, type)) == 0)
		return;

	strcpy(Ps_fonttype, type);
	if (strcmp(Ps_fonttype, "medium") == 0) {
		Ps_fontsize = 6;
		fprintf(Ps_spool, "/Times-Roman findfont 12 scalefont setfont\n");
	}
	else {
		Ps_fontsize = 4;
		fprintf(Ps_spool, "/Times-Roman findfont 10 scalefont setfont\n");
	}
}

#define CENTERCENTER		0
#define LOWERLEFT		2
#define UPPERLEFT		3
#define BOTTOMCENTER		5
#define RIGHTCENTER		6
#define LOWERRIGHT		7

static int Ps_charjust;

ps_char_just_set(type)
	char *type;
{
	/* tell graph_text where to put text in relation to points.  */
	if (strcmp(type,"cc")== 0)
		Ps_charjust = CENTERCENTER;
	else if (strcmp(type,"rc")== 0)
		Ps_charjust = RIGHTCENTER;
	else if (strcmp(type,"ll")== 0)
		Ps_charjust = LOWERLEFT;
	else if (strcmp(type,"ul")== 0)
		Ps_charjust = UPPERLEFT;
	else if (strcmp(type,"bc")== 0)
		Ps_charjust = BOTTOMCENTER;
	else if (strcmp(type,"lr")== 0)
		Ps_charjust = LOWERRIGHT;
}

ps_text(x, y, string)
	double x, y;
	char *string;
{
	int tx, ty;
	/* Ps_fontsize is the height and width in units of the current font,
	   I think.  I'm a little hazy on this */

	GtoPs(x, y, &tx, &ty);
	fprintf(Ps_spool, "%d %d moveto\n", tx, ty);

	switch (Ps_charjust) {
		case CENTERCENTER:	
		case BOTTOMCENTER:
			fprintf(Ps_spool, 
				"(%s) stringwidth pop\n",string);
			fprintf(Ps_spool, "2 div %d add\n", ty);
			fprintf(Ps_spool, "%d exch moveto\n", tx);
			break;
		case RIGHTCENTER:
			fprintf(Ps_spool, 
				"(%s) stringwidth pop\n",string);
			fprintf(Ps_spool, "%d add\n", ty);
			fprintf(Ps_spool, "%d exch moveto\n", tx);
			break;
		case LOWERLEFT:	
			break;
		case UPPERLEFT: 
			break;
		case LOWERRIGHT: 
			fprintf(Ps_spool, 
				"(%s) stringwidth pop\n",string);
			fprintf(Ps_spool, "%d add\n", ty);
			fprintf(Ps_spool, "%d exch moveto\n", tx);
			break;
	}
	/* because of the nasty business regarding not being able to let
	   Postscript do our rotating and translating, we have to momentarily
	   rotate the world while printing chars, else they would be
	   sideways when printed in landscape format. */

	fprintf(Ps_spool, "gsave\n");
	fprintf(Ps_spool, "currentpoint translate\n");
	fprintf(Ps_spool, "-90 rotate\n");
	fprintf(Ps_spool, "(%s) show\n", string); 
	fprintf(Ps_spool, "grestore\n");
}

double
ps_str_width(str)
	char *str;
{
	int len, strwidth;
	double x;

	len = strlen(str);
	strwidth = len * Ps_fontsize;
	x = (double) (strwidth * XtoG_scalex);
	return(x);
}

ps_set_layer(which)
	int which;
{
	if (Ps_color == H_lmap[which])
		return;

	Ps_color = H_lmap[which];
	fprintf(Ps_spool, "P%d\n", Ps_color);

	/* IF YOU HAD A COLOR POSTSCRIPT PRINTER, you might find this line
	   useful.  However, we don't, so I can't test it. 
	fprintf(Ps_spool, "%f %f %f setrgbcolor\n", H_cmap[Ps_color].red, 
		H_cmap[Ps_color].green, H_cmap[Ps_color].blue);
	*/
}

ps_term()
{
	fprintf(Ps_spool, "showpage\n");
	fprintf(Ps_spool, "%%%%Pages: 1\n");
	fclose(Ps_spool);

	if (Ps_devtype == TDIRECT) {
/*		sig_msg("Executing: ", Ps_cmd, ""); */
		if (system(Ps_cmd))
			sig_msg("Could not execute: ",Ps_cmd,"");
	}
	else sig_msg("Have written Postscript instructions to ", Ps_filenm, "");
}

ps_line(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	int tx, ty;

	fprintf(Ps_spool, "newpath\n");
	GtoPs(x1,y1, &tx, &ty);
	fprintf(Ps_spool, "%d %d moveto\n", tx, ty);
	GtoPs(x2, y2, &tx, &ty);
	fprintf(Ps_spool, "%d %d lineto\n", tx, ty);
/*	fprintf(Ps_spool, "%g setlinewidth\n"); */
	fprintf(Ps_spool, "stroke\n");
}

ps_connect_dots(Dotlist, Dotpointer)
	XPoint *Dotlist;
	int Dotpointer;
{
	int i, j;

	i = 0;
	while (i < Dotpointer) {
		fprintf(Ps_spool, "newpath\n");
		fprintf(Ps_spool, "%d %d moveto\n", Dotlist[i].x, Dotlist[i].y);
		j = 0;
		i++;
		while ((i < Dotpointer) && (j < PSSTACKLIM)) {
			fprintf(Ps_spool, "%d %d lineto\n", Dotlist[i].x, Dotlist[i].y);
			i++; j++;
		}
		if (i < Dotpointer)
			i--;
		fprintf(Ps_spool, "stroke\n");
	}
}

ps_set_line_width( line_width )
int line_width;
{
	float width;

	if( line_width < 0 ) {
	   width = 0.25 + 0.1 * line_width;
	} else {
	   width = 0.25 * line_width;
	}

		fprintf(Ps_spool, "%g setlinewidth\n", width);
}

connect_to_ps_funcs()
{
	Hard_funcs.init = ps_init;
	Hard_funcs.color_map = ps_color_map;
	Hard_funcs.window_set = ps_window_set;

	Hard_funcs.font_set = ps_font_set;
	Hard_funcs.char_just_set = ps_char_just_set;
	Hard_funcs.text = ps_text;
	Hard_funcs.str_width = ps_str_width;

	Hard_funcs.set_layer = ps_set_layer;

	Hard_funcs.GtoWhatever = GtoPs;
	Hard_funcs.connect_dots = ps_connect_dots;

	Hard_funcs.line = ps_line;

	Hard_funcs.term = ps_term;
	Hard_funcs.set_line_width = ps_set_line_width;
}

