/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "sigview.h"

extern axis_t *Sig_axes;
extern int Sig_axis_ct;
extern axis_t *Sig_top_axis, *Sig_bottom_axis;
extern double Sig_timestart, Sig_timeend, Sig_interval, Sig_units;
extern comm_file_t *Sig_files;
extern double Sig_multiplier;
extern int Sig_main;

int Sig_precision;
int Sig_decimals;
vert_t Sig_leftvert, Sig_rightvert;
extern int Sig_yaxes, Sig_nums;
extern int Hardcopy;

#define MAINSCALE 1

draw_screen()
{
	graph_set_layer(0);
	graph_draw_mode(0);
	if (! graph_set_drawing(Sig_main)) 
		return;

	graph_rect(0.0,0.0,600.0,500.0);
	axis_area();
	runnames();
	menu_area();
}

axis_area()
{
	graph_set_layer(0);
	graph_draw_mode(0);
	if (! graph_set_drawing(Sig_main))
		return;

	graph_rect(0.0,66.0,600.0,500.0);
	timescale();
	vertical_scroll();
	axes();
	verts();
}

axes()
{
	double axspace, axbot, axtop;
	sigT *sigptr;
	double cheater;
	axis_t *axptr, *endaxes;
	int yaxis_drawn;

	if (Sig_axis_ct == 0)
		return;
	axspace = 395.0 / Sig_axis_ct;
	axtop = 495.0; 
	cheater = axspace * 0.2;

	axptr = Sig_top_axis;
	endaxes = Sig_bottom_axis->next;
	while (axptr != endaxes) {
		axbot = axtop - axspace;
		graph_set_layer(2);
		graph_char_just_set("ll");
		graph_font_set("medium");
		graph_text(22.0, (axbot + (axspace * 0.6)), axptr->name);

		if (axptr->timebegin != Sig_timestart)
			axis_timescale(axbot + (axspace * 0.05), axptr->timebegin, 
				axptr->timebegin + Sig_interval, !MAINSCALE);

		if (axptr->axistype != NORMAL) 
			plot_bus(axbot + cheater, axtop, axptr);

		else {
			sigptr =  axptr->signals;
			yaxis_drawn = 0;
			while (sigptr != NULL) { 
				if (sigptr->dig_Y)
					plot_digital(axbot + cheater, axtop, 
						axptr->timebegin, sigptr->run, sigptr->dig_Y);
				else {
					if (! yaxis_drawn) {
						draw_yaxis(axbot + cheater, axtop, axptr->yaxis);
						yaxis_drawn = 1;
					}
					plot_analog(axbot + cheater, axtop, axptr->timebegin, 
						sigptr->run, sigptr->anal_Y, axptr->yaxis);
				}
				sigptr = sigptr->next;
			}
		}
		axtop = axbot;
		axptr = axptr->next;
	}
}

static 
draw_yaxis(axbot, axtop, yaxis)
	double axbot, axtop;
	yaxis_t yaxis;
{
	double third, thirdv;

	graph_set_layer(2);
	graph_line(AXISLEFT, axbot, AXISLEFT, axtop);
	graph_line(AXISLEFT, axbot, AXISRIGHT, axbot);

	graph_char_just_set("rc");
	graph_font_set("small");
	graph_line(AXISLEFT-2.0, axbot, AXISLEFT, axbot);
	graph_line(AXISLEFT-2.0, axtop, AXISLEFT, axtop);
	if (yaxis.type == LINEAR) {
		graph_text_double(AXISLEFT-4.0, axbot, yaxis.minv, Sig_precision);
		graph_text_double(AXISLEFT-4.0, axtop, yaxis.maxv, Sig_precision);
	}
	else { /* type is LOG */
		graph_text_double(AXISLEFT-4.0,axbot,LOGTEN(yaxis.minv),Sig_precision);
		graph_text_double(AXISLEFT-4.0,axtop,LOGTEN(yaxis.maxv),Sig_precision);
	}

	if (Sig_yaxes) {
		third = (axtop - axbot)/3.0;
		graph_line(AXISLEFT-2.0, axbot+third, AXISLEFT, axbot+third);
		graph_line(AXISLEFT-2.0, axtop-third, AXISLEFT, axtop-third);
		if (yaxis.type == LINEAR) {
			thirdv = (yaxis.maxv - yaxis.minv)/3.0;
			graph_text_double(AXISLEFT-4.0, axbot+third, 
				yaxis.minv+thirdv,Sig_precision);
			graph_text_double(AXISLEFT-4.0, axtop-third, 
				yaxis.maxv-thirdv,Sig_precision);
		}
	}	
}

verts()
{
	v_info_t *vinfo1, *vinfo2, *collect_vertical_info();
	double leftime, rightime, draw_vert();
	int precise, time_precision();

	if ((Sig_nums) && (Sig_axis_ct > 0)) {
		vinfo1 = collect_vertical_info(LEFT);
		vinfo2 = collect_vertical_info(RIGHT);
	}
	else vinfo1 = vinfo2 = NULL;

	precise = time_precision(Sig_timestart + Sig_interval);

	graph_font_set("medium");
	leftime = draw_vert(LEFT, vinfo1, 1, precise);
	rightime = draw_vert(RIGHT, vinfo2, 0, precise);
	draw_dt(rightime, leftime, precise);

	if (vinfo1 != NULL)
		free((char *) vinfo1);
	if (vinfo2 != NULL)
		free((char *) vinfo2);
}

double
draw_vert(which, vinfo, draw_dv, precise)
	int which, draw_dv;
	v_info_t *vinfo;
	int precise;
{
	double screentime, axistime, y;
	vert_t *vert;
	v_info_t *vinfoptr;
	int screen_i, axis_i;
	double axis_voltage;
	int layer;
	run_t *run;
	double interp_voltage();
	double screentimereally;

	screentime = screentimereally = 0.0;
	graph_char_just_set("cc");
	if (which == LEFT) {
		y = 78.0;
		graph_set_layer(10);
		vert = &Sig_leftvert;
	}
	else {
		y = 70.0;	
		graph_set_layer(11);
		vert = &Sig_rightvert;
	}

	/* first we draw the screentime below the vertical */
	graph_line(vert->x, y+2.0, vert->x, 500.0);

	if (vert->run != NULL) {
		run = vert->run;
		graph_set_layer(run->layer);
		screentimereally = (((vert->x - AXISLEFT) / (AXISRIGHT - AXISLEFT)) 
			* Sig_interval) + Sig_timestart;
		screentime = screentimereally * Sig_multiplier;
		graph_text_double(vert->x, y, screentime, precise);

		if (run->run_type == DIGITL)
			return(screentime);
		else screen_i =hunt(run->data.analog[0], run->num_X,
			screentimereally, 0);
	}
	else {
		graph_set_layer(2);
		screentime = ((((vert->x - AXISLEFT) / (AXISRIGHT - AXISLEFT)) 
			* Sig_interval) + Sig_timestart) * Sig_multiplier;
		graph_text_double(vert->x, y, screentime, precise);
		return(screentime);
	}

	/* draw the voltages along each vertical change-in-time line.  Also
       draw the difference in these voltages under each axis name.  Note
   	   that it is assumed that graph_draw_mode has been set appropriately
   	   right before calling this routine. */

	if (! Sig_nums)
		return(screentime);

	if (which == LEFT) 
		layer = 10;
	else layer = 11;
	graph_set_layer(layer);

	vinfoptr = vinfo;
	axis_i = 0;
	while (vinfoptr != NULL) {
		/* print the voltage at the vertical */
		if ((Sig_timestart == vinfoptr->timestart) &&
			(run == vinfoptr->signal->run)) {
			axistime = screentimereally;
			axis_i = screen_i;
		}	
		else {
			axistime = ((((vert->x - AXISLEFT) / (AXISRIGHT - AXISLEFT)) 
					* Sig_interval) + vinfoptr->timestart);
			axis_i = hunt(vinfoptr->signal->run->data.analog[0], 
					vinfoptr->signal->run->num_X, axistime, axis_i);
		}

		axis_voltage = interp_voltage(axistime, axis_i, 
			vinfoptr->signal->run->data.analog[0], 
			vinfoptr->signal->anal_Y,
			vinfoptr->signal->run->num_X, vinfoptr->type);

		graph_text_double(vert->x, vinfoptr->ypos, axis_voltage, Sig_precision);
		
		/* print the dv between this voltage and the other vert's voltage */
		if (draw_dv) {
			graph_set_layer(2);
			graph_text_double(30.0, vinfoptr->dvypos, 
				ABS((axis_voltage - vinfoptr->othervolt)), Sig_precision);
			graph_set_layer(layer);
		}
		vinfoptr = vinfoptr->next;
	}
	return(screentime);
}

draw_dt(rightime, leftime, precise)
	double rightime, leftime;
	int precise;
{
	double graph_str_width();

	graph_set_layer(2);
	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_text(22.0, 74.0, "dt= ");
	graph_text_double(22.0 + graph_str_width("dt= "), 
		74.0, ABS((rightime - leftime)), precise); 
}

runnames()
{
	comm_file_t *fileptr;
	run_t *runptr;
	char buffer[BUFSIZE];
	double pos, graph_str_width();

	graph_set_layer(2);
	graph_draw_mode(0);
	graph_char_just_set("ll");
	graph_font_set("medium");
	graph_text(22.0, 60.0, "Runs: ");

	fileptr = Sig_files;
	pos = 22.0 + graph_str_width("Runs: ");
	while (fileptr != NULL) {
		runptr = fileptr->runs;
		while (runptr != NULL) {
			buffer[0] = '\0';
			strcat(buffer, runptr->name);
			strcat(buffer, " ");
			graph_set_layer(runptr->layer);
			graph_text(pos, 60.0, buffer);
			pos = pos + graph_str_width(buffer);
			runptr = runptr->next;
		}
		fileptr = fileptr->next;
	}
}

timescale()
{
	graph_set_layer(5);
	graph_rect(15.0, 83.0, 600.0, 98.0);
	axis_timescale(89.0, Sig_timestart, Sig_timeend, MAINSCALE);

	graph_set_layer(3); /* textovercolors */
	graph_font_set("small");
	graph_char_just_set("bc");
	/* print the current multiplier for the times to be printed here. */
	graph_text_double(35.0, 89.0, 1.0 / Sig_multiplier, 1);

}

static 
axis_timescale(y, begintime, endtime, mainscale)
	double y, begintime, endtime;
	int mainscale;
{
	double blip_increment, time_increment;
	double x, time;
	int i,j, precise, time_precision();

	precise = time_precision(endtime);

	if (mainscale == MAINSCALE)
		graph_set_layer(3); /* text over colors */
	else graph_set_layer(2); /* text over bkgd */
	graph_font_set("small");
	graph_char_just_set("bc");
	time_increment = (endtime - begintime)/4.0;
	blip_increment = (AXISRIGHT - AXISLEFT)/20.0;
	x = AXISLEFT;
	time = begintime;

	for (i = 0; i < 4; i++) {
		graph_text_double(x, y, time*Sig_multiplier, precise);
		time = time + time_increment;
		for (j = 0; j < 4; j++) {
			x = x + blip_increment;
			graph_line(x, y, x, y+2.0);
		}
		x = x + blip_increment;
	}
	graph_text_double(x, y, time*Sig_multiplier, precise);
}

plot_bus(axbot, axtop, axptr)
	double axbot, axtop;
	axis_t *axptr;
{
	double timebegin;
	sigT *sigptr;
	yaxis_t yaxis;
	int axistype;
	double int_begin, interval;
	double timestart, timeend;
	double possible_end;
	double curr_time;
	sigT *signal;
	int count, i, j, yaxis_drawn;
	struct bus_sig {
		char *Y;
		double *X;
		int index;
		int num_X;
	} *bus_signals;
	char *holder, *a_string;
	int up, simple;
	double uppos, downpos, cheat;
	double *X;
	int new_index, new_x;

	timebegin = axptr->timebegin;
	sigptr = axptr->signals;
	yaxis = axptr->yaxis;
	axistype = axptr->axistype;

	/* count the number of digital signals in the sigptr list */
	signal = sigptr;
	count = 0;
	curr_time = 0;
	while (signal != NULL ) {
		if (signal->dig_Y != NULL) {
			count++;
			curr_time = SMAX(curr_time, signal->run->data.digital.X[0]);
		}
		signal = signal->next;
	}

	/* now fill each element in bus signals, plotting any analog signals we
	   may find immediately. */
	if (count > 0) {
		a_string = (char *)calloc(count+1, sizeof(char));
		bus_signals =(struct bus_sig *) calloc(count, sizeof(struct bus_sig));
		holder = (char *)calloc(count+1, sizeof(char));
	}
	signal = sigptr; i = 0;
	yaxis_drawn = 0;
	/* timestart and timeend must be in digital "units"--that is, they are
	   timesteps, not nanoseconds or anything silly like that. If they
	   are (RNL output, for instance) then let the user set Sig_multiplier
	   accordingly. */
	timestart = (int)(timebegin/Sig_units);
	timeend = (int)((timebegin+Sig_interval)/Sig_units);

	while (signal != NULL ) {
		if (signal->dig_Y != NULL) {
			bus_signals[i].Y = signal->dig_Y;
			bus_signals[i].X = signal->run->data.digital.X;
			bus_signals[i].index = hunt(bus_signals[i].X, 
				signal->run->num_X, curr_time, 0);
			bus_signals[i].num_X = signal->run->num_X;
			i++;
		}
		else  {
			if (! yaxis_drawn) {
				draw_yaxis(axbot, axtop, yaxis);
				yaxis_drawn = 1;
			}
			plot_analog(axbot, axtop, timebegin, signal->run, 
				signal->anal_Y, yaxis);
		}
		signal = signal->next;
	}

	/* if this "bus" axis contains entirely analog signals, then get the
	   heck out of here */
	if (count == 0) 
		return;

	possible_end = bus_signals[0].X[bus_signals[0].num_X-1];
	simple = 1; X = bus_signals[0].X;
	for (i = 1; i < count; i++) {
		if (X != bus_signals[i].X)
			simple = 0;
		possible_end=SMIN(possible_end,bus_signals[i].X[bus_signals[i].num_X-1]);
	}
	if (simple) {
		free((char *)a_string);
		free((char *)holder);
		free((char *)bus_signals);
		plot_simple_bus(count, axbot, axtop, timebegin, sigptr, axistype);
		return;
	}
		
	interval = (AXISRIGHT - AXISLEFT) / (timeend - timestart); 
	graph_set_layer(sigptr->run->layer);
	graph_char_just_set("ll");
	cheat = (axtop - axbot) * 0.2;
	uppos = axtop - cheat;
	downpos = axbot + cheat;
	up = 1;
	curr_time = SMAX(timestart, curr_time);
	timeend = SMIN(timeend, possible_end);

	while (curr_time <= timeend) {
		i = 0;
		for (j = count - 1; j >= 0; j--)  
			holder[i++] = bus_signals[j].Y[bus_signals[j].index];
		holder[count] = '\0';
		convert(holder, count, a_string, axistype);

		int_begin = AXISLEFT+((curr_time - timestart) * interval);
		graph_line(int_begin, axbot, int_begin, axtop);
		if (up)
			graph_text(int_begin, uppos, a_string);
		else graph_text(int_begin, downpos, a_string);
		up = 1 - up;

		/* update curr_time to the next lowest time mentioned in any signal.
		   Update the indices in bus_signals to point to the X which is less
		   than or equal to curr_time */
		if (curr_time < timeend) {
			curr_time = bus_signals[0].X[bus_signals[0].index + 1];
			for (j = 1; j < count; j++) 
			curr_time=SMIN(curr_time,bus_signals[j].X[bus_signals[j].index+1]);

			for (j = 0; j < count; j++) {
				new_index = bus_signals[j].index + 1;	
				new_x = bus_signals[j].X[new_index];
				if (new_x <= curr_time)
					bus_signals[j].index = new_index;
			}
		}
		else curr_time = timeend + 1;
	}
	free((char *)a_string);
	free((char *)holder);
	free((char *)bus_signals);
}

plot_simple_bus(count, axbot, axtop, timebegin, sigptr, axistype)
	double axbot, axtop;
	double timebegin;
	sigT *sigptr;
	int axistype;
	int count;
{
	run_t *the_run;
	int i, j;
	int start_index, end_index;
	double int_begin, interval;
	double *X;
	double timestart, timeend;
	sigT *signal;
	char *holder, *a_string;
	int up;
	double uppos, downpos, cheat;

	holder = (char *)calloc(count+1, sizeof(char));
	a_string = (char *)calloc(count+1, sizeof(char));

	the_run = NULL;
	signal = sigptr;
	while ((signal != NULL ) && (the_run == NULL))  {
		if (signal->dig_Y != NULL)
			the_run = signal->run;
		else signal = signal->next;
	}

	/* Locate the indices of timestart and timeend in the run's X array */
	X = the_run->data.digital.X;
	timestart = (timebegin/Sig_units);
	timeend =  ((timebegin+Sig_interval)/Sig_units);
	
	/* if data is totally off_screen, then get out of here */
	if ((X[the_run->num_X-1] < timestart) || (X[0] > timeend)) {
		free((char *)a_string);
		free((char *)holder);
		return;
	}
	start_index = hunt(X, the_run->num_X, timestart, 0); 
	end_index = hunt(X, the_run->num_X, timeend, the_run->num_X-1);
	if ((X[end_index] < timeend) && ((end_index + 1) < the_run->num_X)) 
		end_index++;

	interval = (AXISRIGHT - AXISLEFT) / (timeend - timestart); 
	if (X[start_index] > timestart)
		int_begin =AXISLEFT+((X[start_index]-timestart) * interval);
	else int_begin = AXISLEFT;
	graph_set_layer(the_run->layer);
	graph_char_just_set("ll");

	cheat = (axtop - axbot) * 0.2;
	uppos = axtop - cheat;
	downpos = axbot + cheat;
	up = 1;
	for (i = start_index; i <= end_index; i++) {
		signal = sigptr;
		for (j = count-1; j >= 0; j--) {
			holder[j] = signal->dig_Y[i];
			signal = signal->next;
		}
		holder[count] = '\0';
		convert(holder, count, a_string, axistype);
		graph_line(int_begin, axbot, int_begin, axtop);
		if (up)
			graph_text(int_begin, uppos, a_string);
		else graph_text(int_begin, downpos, a_string);
		up = 1 - up;
		if (i < end_index)
			int_begin = AXISLEFT+((X[i+1] - timestart) * interval);
	}
	free((char *)a_string);
	free((char *)holder);
}

plot_digital(axbot, axtop, timebegin, run, signal)
	double axbot, axtop;
	run_t *run;
	char signal[];
	double timebegin;
{
	double int_begin, int_end, interval;
	int i;
	char val1, val2;
	double *X;
	double startv, endv; /* vectors corresponding to timebegin, timeend */
	int start_index, end_index; /* indices into X where values are as close
		to startv, endv as possible */
	double axmid, tiny;
	double lastvec;

	/* Locate the indices of startv and endv in the run's X array */
	X = run->data.digital.X;
	startv = timebegin/Sig_units;
	endv = (timebegin+Sig_interval)/Sig_units;
	axmid = (axtop + axbot) / 2.0;

	/* if data is totally off_screen, then get out of here */
	if ((X[run->num_X-1] < startv) || (X[0] > endv))
		return;

	start_index = hunt(X, run->num_X, startv, 0); 
	end_index = hunt(X, run->num_X, endv, run->num_X-1);
	if ((X[end_index] < endv) && ((end_index + 1) < run->num_X)) 
		end_index++;

	interval = (AXISRIGHT - AXISLEFT) / (endv - startv); 
	if (X[start_index] > startv) {
		int_begin = AXISLEFT + (((double)X[start_index]-startv) * interval);
		startv = X[start_index];
	}
	else int_begin = AXISLEFT;
	val1 = signal[start_index];
	lastvec = startv;
	graph_set_layer(run->layer);

	for (i = start_index+1; i <= end_index; i++) { 
		val2 = signal[i];
		int_end = int_begin + ((X[i] - lastvec) * interval);
		tiny = (int_end - int_begin)/4.0;
		switch (val1) {
			case '1':
				graph_line(int_begin, axtop, int_end, axtop);
				if (val2 == '0') 
					graph_line(int_end, axtop, int_end, axbot);
				break;
			case '0':
				graph_line(int_begin, axbot, int_end, axbot);
				if (val2 == '1') 
					graph_line(int_end, axtop, int_end, axbot);
				break;
			case 'x':
				graph_line(int_begin, axbot, int_end, axtop);
				graph_line(int_begin, axtop, int_end, axbot);
				break;
			case 'z':
				graph_line(int_begin, axmid, int_begin + tiny, axtop);
				graph_line(int_begin+tiny,axtop,int_begin+(2.0*tiny),axmid);
				graph_line(int_begin+(2.0*tiny),axmid,
					int_begin+(3.0*tiny),axtop);
				graph_line(int_begin+(3.0*tiny),axtop, int_end,axmid);
				break;
		}
		val1 = val2;
		int_begin = int_end;
		lastvec = X[i];
	}
}

plot_analog(axbot, axtop, timebegin, run, signal, yaxis)
	double axbot, axtop, timebegin;
	run_t *run;
	double signal[];
	yaxis_t yaxis;
{
	int start_index, end_index;
	int i;
	double *X;
	double d, x1, x2, y1, y2;
	double timestart, timeend;
	double interp(), minv, maxv;
	double firstx, firsty, lasty, lastx;

	/* Locate the indices of timestart and timeend in the run's X array */
	X = run->data.analog[0];
	timestart = timebegin;
	timeend = (timebegin + Sig_interval);

	if ((X[run->num_X-1] < timestart) || (X[0] > timeend))
		return;

	start_index = hunt(X, run->num_X, timestart, 0); 
	end_index = hunt(X, run->num_X, timeend, run->num_X-1);
	graph_set_layer(run->layer);

	/* reset the screen coordinates so that we can just plop down
	   the lines from point to point. */
	d = timeend - timestart;
	x1 = timestart - (d/4.9);
	x2 = timeend + (d/49.0);

	if (yaxis.type == LOG) {
		graph_window_set(Sig_main, x1, 0.0, x2, 500.0);
		plot_analog_log(axbot, axtop, end_index, start_index, X, signal,yaxis);
		graph_window_set(Sig_main, 0.0, 0.0, 600.0, 500.0);
		return;
	}

	d = yaxis.maxv - yaxis.minv;
	y1 = yaxis.minv - (d / ((axtop - axbot)/axbot));
	y2 = yaxis.maxv + (d / ((axtop - axbot)/(500.0 - axtop)));
	
	graph_window_set(Sig_main, x1, y1, x2, y2);

	/* If plot crosses left side of scale, cut it off */
	if (X[start_index] < timestart) {
		firsty = interp(signal[start_index], X[start_index], 
					signal[start_index+1], X[start_index+1], timestart);
		firstx = timestart;
	}
	else {
		firsty = signal[start_index];
		firstx = X[start_index];
	}

	/* If plot crosses right side of scale, cut it off */
	if ((end_index + 1) < run->num_X) {	
		lasty = interp(signal[end_index], X[end_index],
					signal[end_index+1], X[end_index+1], timeend);
		lastx = timeend;
	}
	else {
		lasty = signal[end_index];
		lastx = X[end_index];
	}

	if (yaxis.defawlt) {
		graph_start_dots(end_index - start_index + 2);
		graph_store_dot(firstx, firsty);
		for (i = start_index + 1; i <= end_index; i++) 
			graph_store_dot(X[i], signal[i]);
		graph_store_dot(lastx, lasty);
		graph_connect_dots();
	}
	else {
		minv = yaxis.minv;
		maxv = yaxis.maxv;

		graph_line_yclip(firstx, firsty, X[start_index+1],
						signal[start_index+1], minv, maxv);
		for (i = start_index+1; i < end_index; i++)
			graph_line_yclip(X[i], signal[i], X[i+1], signal[i+1], minv, maxv);
		graph_line_yclip(X[end_index], signal[end_index], 
				lastx, lasty, minv, maxv);
	}
	graph_window_set(Sig_main, 0.0, 0.0, 600.0, 500.0);
}


/* This is needlessly complicated because I didn't really understand
   the task when I wrote it.  Someday (ha) I will rewrite it the way
   it should have been done. */

plot_analog_log(axbot, axtop, end_index, start_index, X, signal, yaxis)
	double axbot, axtop;
	int end_index, start_index;
	double *X;
	double signal[];
	yaxis_t yaxis;
{
	double axspace, y, log10_of_yspan;
	int i, inside;
	double lastx, lasty, interpx, interpy, interp();
	double minv, maxv;
	double log10_of_minv;

	axspace = axtop - axbot;
	log10_of_yspan = LOGTEN(yaxis.maxv) - LOGTEN(yaxis.minv);
	log10_of_minv = LOGTEN(yaxis.minv);

	if (yaxis.defawlt) {
		graph_start_dots(end_index - start_index + 1);
		for (i = start_index; i <= end_index; i++) { 
			y = axbot + (((LOGTEN(signal[i]) - log10_of_minv) / log10_of_yspan) 
				* axspace);
			graph_store_dot(X[i], y);
		}
		graph_connect_dots();
	}
	else {
		/* find that first point between maxv and minv */
		i = start_index;
		minv = yaxis.minv; maxv = yaxis.maxv;
		if (signal[i] < minv) {  /* loop till we cross minv */
			while ((i <= end_index) && (signal[i] < minv))
				i++;
			lasty = axbot;
			lastx = interp(X[i-1], LOGTEN(signal[i-1]), X[i], 
				LOGTEN(signal[i]), minv);
		}
		else if (signal[i] > maxv) {
			while ((i <= end_index) && (signal[i] > maxv))
				i++;
			lasty = axtop;
			lastx = interp(X[i-1], signal[i-1], X[i], signal[i], maxv);
		}
		else {
			lastx = X[i];
			lasty = axbot + ((LOGTEN(signal[i])/log10_of_yspan) * axspace);
		}
		if (i==end_index+1)	/* well heck, the whole thing is above or below */
			return;		    /*		the axis */

		inside = 1;
		while (i <= end_index) {
			if (inside) {
				if ((signal[i] <= maxv) && (signal[i] >= minv)) {
					y = axbot + ((LOGTEN(signal[i])/log10_of_yspan) * axspace);
					graph_line(lastx, lasty, X[i], y);
					lastx = X[i];
					lasty = y;
				}
				else { /* about to go outside */
					if (signal[i] > maxv) 
						interpy = axtop;
					else interpy = axbot;
					interpx = interp(lastx, lasty, X[i], signal[i], interpy);
					graph_line(lastx, lasty, interpx, interpy);
					inside = 0;
				}
			}
			else { /* outside, maybe coming back in */
				if ((signal[i] <= maxv) && (signal[i] >= minv))  { /* coming in */
					if (signal[i-1] > maxv)
						lasty = maxv;
					else lasty = minv;
					lastx = interp(X[i-1], signal[i-1], X[i], signal[i], lasty);
					graph_line(lastx, lasty, X[i], signal[i]);
					lastx = X[i];
					lasty = signal[i];
					inside = 1;
				}
				else if (((signal[i-1] <= minv) && (signal[i] >= maxv)) ||
						 ((signal[i-1] >= maxv) && (signal[i] <= minv))) { 
						 /* passing over */
					lasty = minv;
					interpy = maxv;
					lastx = X[i-1] + 
						(((minv - signal[i-1])/(signal[i] - signal[i-1]))*
						(X[i] - X[i-1]));
					interpx = X[i-1] + 
						(((maxv - signal[i-1])/(signal[i] - signal[i-1]))*
						(X[i] - X[i-1]));
					graph_line(lastx, lasty, interpx, interpy);
				}
			}	
			i++;
		}
	}
}

/* given value we desire, and the points above and below it, 
   figure what the interpolated value must be. */
double
interp(last1, last2, this1, this2, value)
	double last1, last2, this1, this2, value;
{
	if (this2 == last2)
		return(this1);
	else return(last1 + (((value - last2)/(this2 - last2)) *
			(this1 - last1)));
}

menu_area()
{
	if (Hardcopy)
		return;
	if (! graph_set_drawing(Sig_main))
		return;

	graph_draw_mode(0);
	graph_set_layer(4);
	graph_rect(0.0, 0.0, 100.0, 22.0);
	graph_set_layer(5);
	graph_rect(0.0, 22.0, 100.0, 44.0);
	graph_set_layer(6);
	graph_rect(100.0, 0.0, 200.0, 22.0);
	graph_set_layer(7);
	graph_rect(100.0, 22.0, 200.0, 44.0);
	graph_set_layer(8);
	graph_rect(200.0, 0.0, 300.0, 22.0);
	graph_set_layer(9);
	graph_rect(200.0, 22.0, 300.0, 44.0);
	graph_set_layer(4);
	graph_rect(300.0, 0.0, 400.0, 22.0);
	graph_set_layer(5);
	graph_rect(300.0, 22.0, 400.0, 44.0);
	graph_set_layer(6);
	graph_rect(400.0, 0.0, 500.0, 22.0);
	graph_set_layer(7);
	graph_rect(400.0, 22.0, 500.0, 44.0);
	graph_set_layer(8);
	graph_rect(500.0, 0.0, 600.0, 22.0);
	graph_set_layer(9);
	graph_rect(500.0, 22.0, 600.0, 44.0);

	graph_set_layer(3);
	graph_char_just_set("cc");
	graph_font_set("medium");
	graph_text(50.0, 11.0, "Compare");
	graph_text(50.0, 33.0, "Copy");
	graph_text(150.0, 11.0, "Create");
	graph_text(150.0, 33.0, "Show");
	graph_text(250.0, 11.0, "Delete");
	graph_text(250.0, 33.0, "Combine");
	graph_text(350.0, 11.0, "Shuffle");
	graph_text(350.0, 33.0, "Snapshot");
	graph_text(450.0, 11.0, "Zoom Reset");
	graph_text(450.0, 33.0, "Zoom In");
	graph_text(550.0, 11.0, "Quit Sigview");
	graph_text(550.0, 33.0, "Redraw");
}

vertical_scroll()
{
	graph_set_layer(5);
	graph_rect(0.0, 98.0, 15.0, 500.0);
	graph_set_layer(3);
	if (Sig_axes == NULL)
		return;
	if (Sig_top_axis->previous != NULL){/* there are more axes above screen */
		graph_line(7.5, 400.0, 7.5, 500.0);
		graph_line(7.5, 500.0, 2.0, 475.0);
		graph_line(7.5, 500.0, 13.0, 475.0);
	}
	if (Sig_bottom_axis->next != NULL){/* more axes below */
		graph_line(7.5, 200.0, 7.5, 100.0);
		graph_line(7.5, 100.0, 2.0, 125.0);
		graph_line(7.5, 100.0, 13.0, 125.0);
	}
}

convert(binary_str, num_of_sigs, result, axistype)
	char *binary_str, *result;
	int num_of_sigs, axistype;
{
	switch(axistype) {
		case HEX: 
		case OCTAL: 
		case DECIMAL: 
			convert_general(binary_str, result,num_of_sigs, axistype);
			break;
		case BINARY: 
		default: 
			strcpy(result, binary_str);
			break;
	}
}

/* You'll wonder "why didn't she just use sprintf to convert the
   entire number at once?"  Because of those pesky z's and x's,
   that's why.  Every group of four(hex) or three(octal) which
   has a x or z in it becomes an x or z.  However, that does NOT
   mean that the other grouplets become x or z.  They may be
   an intelligible number.
*/
static
convert_general(bus_str, result, number_of_sigs, axistype)
	char *bus_str, *result;
	int number_of_sigs, axistype;
{	
	int extra_bits, groupsize;
	int bus_str_p, result_p;
	int base;
	int i;

	switch (axistype) {
		case OCTAL:
			groupsize = 3;
			base = 8;
			break;
		case HEX:
			groupsize = 4;
			base = 16;
			break;
		case DECIMAL:
			groupsize = number_of_sigs;
			base = 10;
			break;
	}
	bus_str_p = result_p = 0;

	/* things are in MSB order, so we need to cut off the top bits and
	   lord knows they may not come out evenly. */
	if (extra_bits = number_of_sigs % groupsize) {
		convert2(extra_bits, base, &bus_str[0], &result[0]);
		result_p = 1;
		bus_str_p = extra_bits;
	}

	/* now cycle through the remaining bits */
	for (i = 0 ; i < number_of_sigs/groupsize; i++) {
		convert2(groupsize, base, &bus_str[bus_str_p], &result[result_p]);
		bus_str_p = bus_str_p + groupsize;
		result_p ++;
	}
}

static
convert2(num_bits, base, holder, result)
	int num_bits, base;
	char *holder, *result;
{
	int i;
	unsigned int num;

	num = 0;
	for (i=0; i < num_bits; i++){
		switch (holder[i]) {
			case '0':
				/* shift left, realizing that because num is unsigned,
				   zero will be shifted in */
				num = num << 1;
				break;
			case '1':
				num = num << 1;
				num ++;
				break;
			/* getting x or z is catastrophic, just return the letter */
			case 'x':
				strcpy(result, "x");
				return; 
			case 'z':
				strcpy(result, "z");
				return;
			}
		}
	switch (base) {
		case 10:
			sprintf(result,"%d",num);
			break;
		case 8:
			sprintf(result,"%o",num);
			break;
		case 16:
			sprintf(result,"%x",num);
			break;
		default:
			sprintf(result,"%s",num);
			break;
		}
}

/* collect a linked list of info needed to quickly refresh a vertical.
   Vinfo will be null if we are not to put vals along the vertical. A
   value for an axis is placed on a vertical only if it contains at least 
   one analog signal.  We use the signal of the file to which the vert has 
   been assigned or, failing that, of the file to which the other vert has been
   assigned, or, failing that, to some other analog signal on that axis.
   In addition, if there is more than one signal from the same file on an
   axis, then we don't use them because the viewer could not tell which
   signal's value was being printed.  Are you confused yet?
*/
v_info_t *
collect_vertical_info(which)
	int which;
{
	axis_t *axptr;
	vert_t *vert, *othervert;
	sigT *the_sig, *some_signal_applies();
	double percent, time;
	int i;
	v_info_t *vinfo, *vinfoptr;
	double axspace, axtop;
	double interp_voltage();

	vinfo = NULL;
	if (Sig_axis_ct == 0)
		return(NULL);

	if (which == RIGHT) {
		vert = &(Sig_rightvert);
		othervert = &(Sig_leftvert);
		percent = .7;
	}
	else  {
		vert = &(Sig_leftvert);
		othervert = &(Sig_rightvert);
		percent = .3;
	}

	if (vert->run == NULL)
		return(NULL);
	
	axspace = 395.0 / Sig_axis_ct;
	axtop = 495.0;
	axptr = Sig_top_axis;
	i = 0;
	while (axptr != Sig_bottom_axis->next) {
		if (axptr->axistype == NORMAL) {
			/* search for an appropriate signal for this vertical. */
			the_sig = some_signal_applies(axptr->signals, vert, othervert);
			if (the_sig != NULL) {
				if (vinfo == NULL) {
					vinfo = (v_info_t *) malloc (sizeof(v_info_t));
					vinfo->next = NULL;
					vinfoptr = vinfo;
				}
				else {
					vinfoptr->next = (v_info_t *) malloc (sizeof(v_info_t));
					vinfoptr = vinfoptr->next;
					vinfoptr->next = NULL;
				}

				vinfoptr->signal = the_sig;
				vinfoptr->timestart = axptr->timebegin;

				/* now figure the voltage on the other axis */	
				the_sig = some_signal_applies(axptr->signals, othervert, vert);
				if (the_sig != NULL) {
					time =(((othervert->x - AXISLEFT)/(AXISRIGHT - AXISLEFT)) 
						* Sig_interval) + axptr->timebegin;
					i = hunt(the_sig->run->data.analog[0], 
						the_sig->run->num_X, time, i);

					if (axptr->yaxis.type == LINEAR) {
						vinfoptr->type = LINEAR;
						vinfoptr->othervolt = interp_voltage(time, i, 
							the_sig->run->data.analog[0], the_sig->anal_Y, 
							the_sig->run->num_X, LINEAR);
					}
					else {
						vinfoptr->type = LOG;
						vinfoptr->othervolt = interp_voltage(time, i, 
							the_sig->run->data.analog[0], the_sig->anal_Y, 
							the_sig->run->num_X, LOG);
					}
				}
				else vinfoptr->othervolt = 0.0;
				vinfoptr->ypos = axtop - (percent * axspace);
				vinfoptr->dvypos = axtop - (.7 * axspace);
			}
		}
		axtop = axtop - axspace;
		axptr = axptr->next;
	}
	return(vinfo);
}


double
interp_voltage(time, near_index, X, Y, numx, type)
	double time;
	int near_index, type, numx;
	double *X, *Y;
{
	double timebelow, timeabove;
	double percentdist, voltincr;

	if (X[near_index] >= time)
		return(Y[0]);
	if (near_index == (numx - 1))
		return(Y[numx - 1]);

	/* else we have to interpolate. We know that "near_index" contains the
	   index of the closest time < "time" as passed in here. */
	timeabove = X[near_index + 1];
	timebelow = X[near_index];

	percentdist = (time - timebelow) / (timeabove - timebelow);
	voltincr = (Y[near_index + 1] - Y[near_index]) * percentdist;
	if (type == LINEAR)
		return(Y[near_index] + voltincr);
	else return(LOGTEN(Y[near_index] + voltincr));
}


static sigT *
some_signal_applies(sigptr, vert, othervert)
	sigT *sigptr;
	vert_t *vert, *othervert;
{
	sigT *thisrun_sig, *otherrun_sig, *anon_sig;
	int thisrun_ct, otherrun_ct, anon_ct;

	thisrun_sig = otherrun_sig = anon_sig = NULL;
	thisrun_ct = otherrun_ct = anon_ct = 0;

	while (sigptr != NULL) {
		if ((vert->run->run_type != DIGITL) && (sigptr->run == vert->run)) {
			thisrun_sig = sigptr;
			thisrun_ct++;
		}
		else if ((othervert->run->run_type != DIGITL) && 
			(sigptr->run == othervert->run)) {
			otherrun_sig = sigptr;
			otherrun_ct++;
		}
		else if (sigptr->run->run_type != DIGITL) {
			anon_sig = sigptr;
			anon_ct++;
		}
		sigptr = sigptr->next;
	}
	if (thisrun_ct == 1)
		return(thisrun_sig);
	else if (otherrun_ct == 1)
		return(otherrun_sig);
	else if (anon_ct == 1)
		return(anon_sig);
	else return(NULL);
}

/* If there are more than 5 places either before or 2 after the
   decimal OR Sig_precision, we print values in scientific notation. */
int
time_precision(timeend)
	double timeend;
{
	int precise;

	if (Sig_precision)
		precise = 1;
	else {
		precise = 0;
		if (((timeend * Sig_multiplier)/1e5) >= 1.0)
			precise = 1;
		else if (((timeend * Sig_multiplier)*10) < 1.0) 
			precise = 1;
	}
	return(precise);
}
