/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <math.h>
#include "sigview.h"

extern vert_t Sig_leftvert, Sig_rightvert;
extern double Sig_interval;
extern double Sig_units, Sig_multiplier;
extern int Sig_interactive_mode;

#define S_MAXLIST	128
#define S_PATLEN	512

static axis_t *Last_axis;
static int Last_search_pos;
static int Last_sigct;
static char Last_patterns[S_PATLEN];
static char Last_individual;

/* We assume that a pattern given for a single signal axis has NO SPACES
   within it.  It is does, then the stuff after the first space is
   ignored. */

do_search(axnm, individual, btime, bflag, etime, eflag, pattern)
	char *axnm, *pattern;
	int bflag, eflag, individual;
	double btime, etime;
{
	axis_t *axptr, *find_axis();
	int bindex, eindex, findex;
	sigT *signal;
	int max_index, sigct;
	char *onepattern, *next_token();
	double *X;

	strcpy(Last_patterns, pattern);

	axptr = find_axis(axnm);
	if (axptr == NULL) {
		sig_msg("Search:  ",axnm," is not an axis\n");
		return;
	}
	
	/* check that this axis has only digital signals on it */
	signal = axptr->signals;
	max_index = signal->run->num_X-1; 
	sigct = 0;
	while ((signal != NULL) && (signal->dig_Y != NULL)) {
		max_index = SMIN(max_index, signal->run->num_X-1);
		sigct++;
		signal = signal->next;
	}

	if (signal != NULL) { /* if we didn't get through the list */
		sig_msg("Search: ","Found non-digital signals on axis ",axnm);
		return;	
	}

	X = axptr->signals->run->data.digital.X;
	if (bflag)	/* begintime set in search command */
		bindex = hunt(X, axptr->signals->run->num_X,
			(double)((btime/Sig_multiplier) / Sig_units, 0));
	else bindex = 0;
	
	if (eflag)	/* end time set in search command */
		eindex = hunt(X, axptr->signals->run->num_X,
			(double)((etime/Sig_multiplier) / Sig_units, 0));
	else eindex = max_index;
/*
	if (bindex == eindex)
		return;
*/
	if (sigct == 1) {
		onepattern = next_token(&pattern);
		if (bindex <= eindex)
			findex = search_single_signal(axptr->signals, FORWARDS, 
				bindex, eindex, onepattern);
		else findex = search_single_signal(axptr->signals, BACKWARDS, 
				bindex, eindex, onepattern);
	}
	else {
		if (bindex <= eindex)
			findex =search_multi_signal(sigct, axptr->axistype, 
			axptr->signals, FORWARDS, bindex, eindex, pattern);
		else findex =search_multi_signal(sigct, axptr->axistype, 
			axptr->signals, BACKWARDS, bindex, eindex, pattern);
	}
	if (findex == -1) {
		sig_msg("Search:  pattern not found.","","");
		return;
	}

	adjust_leftvert(findex, X, individual, axnm);
	
	Last_sigct = sigct;
	Last_axis = axptr;
	Last_search_pos = findex;
	Last_individual = individual;
}

adjust_leftvert(findex, X, individual, axnm)
	int findex, individual;
	char *axnm;
	double *X;
{
	double timebegin;

	/* figure new start time such that findex is in screen middle */
/*	if ((bflag) && (X[findex] < ((btime/Sig_multiplier)/Sig_units)))
		timebegin = (btime/Sig_multiplier) - (Sig_interval / 2.0);
	else 
*/
	timebegin = (X[findex] * Sig_units) - (Sig_interval / 2.0);
	timebegin = timebegin * Sig_multiplier;

	/* now bring left change-in-time line to rest over found pattern */
	Sig_leftvert.x = (AXISRIGHT + AXISLEFT) / 2.0;

	/* realign axes--NOTE that start_time redraws the axis area */
	if (individual)
		start_time(timebegin, axnm);
	else start_time(timebegin, ""); /* change all axes */
}

int
search_next()
{
	int start, max_index;
	int findex;
	char pattern[S_PATLEN];

	/* something has happened to the axis we were searching.  Let's leave.*/
	if (Last_axis == NULL)
		return(-1);

	/* else we search forwards for the pattern again */
	max_index =  Last_axis->signals->run->num_X -1;
	if (Last_search_pos == max_index)
		start = 0;
	else start = Last_search_pos + 1;

	strcpy(pattern, Last_patterns);
	if (Last_sigct == 1)
		findex = search_single_signal(Last_axis->signals, FORWARDS, 
				start, max_index, pattern);
	else findex = search_multi_signal(Last_sigct, Last_axis->axistype, 
				Last_axis->signals, FORWARDS, start, 
				max_index, pattern);

	if (findex == -1) {
		Last_search_pos = max_index;
		return(-1);
	}
	Last_search_pos = findex;

	adjust_leftvert(findex, Last_axis->signals->run->data.digital.X, 
		Last_individual, Last_axis->name);

	return(findex);
}

int
search_previous()
{
	int start, max_index;
	int findex;
	char pattern[S_PATLEN];

	/* something has happened to the axis we were searching.  Let's leave.*/
	if (Last_axis == NULL)
		return(-1);

	/* else we search backwards for the pattern again */
	max_index =  Last_axis->signals->run->num_X -1;
	if (Last_search_pos == 0)
		start = max_index;	/*wraparound */	
	else start = Last_search_pos - 1;

	strcpy(pattern, Last_patterns);
	if (Last_sigct == 1)
		findex = search_single_signal(Last_axis->signals, BACKWARDS, 
				start, 0, pattern);
	else findex = search_multi_signal(Last_sigct, Last_axis->axistype, 
				Last_axis->signals, BACKWARDS, start, 0, pattern);

	if (findex == -1) {
		Last_search_pos = 0;
		return(-1);
	}
	Last_search_pos = findex;

	adjust_leftvert(findex, Last_axis->signals->run->data.digital.X, 
		Last_individual, Last_axis->name);
	
	return(findex);
}


/* From the calling proc, starti should always be <= endi, regardless 
   of direction.  If direction is BACKWARDS, then we need to reverse the 
   two values here.  */
int
search_single_signal(signal, direction, starti, endi, pattern)
	int starti, endi, direction;
	sigT *signal;
	char *pattern;
{
	int pos;
	int max_index;

	max_index =  signal->run->num_X -1;
	if (starti < 0) starti = 0; 
	if (endi > max_index) endi = max_index;
		
	if (direction == FORWARDS)
		pos = substr(signal->dig_Y, pattern, starti, endi);
	else
		pos = substr_backwards(signal->dig_Y, pattern, endi, starti);

	return(pos);
}

static
two_least_sig_bits(conversion, pattern, bit0, bit1)
	char *pattern;
	char *bit0, *bit1;
	int conversion;
{
	unsigned int tempint;
	char c;

	switch(conversion) {
		case BINARY:
			/* for now, assuming lsb is at the right of user's pattern*/
			tempint = strlen(pattern);
			*bit0 = pattern[tempint -1];
			*bit1 = pattern[tempint -2];
			break;
		case DECIMAL:
			if (strcmp(pattern, "x") == 0)
				/* set bits to x */
				*bit0 = *bit1 = 'x';
			else if (strcmp(pattern, "z") == 0)
				/* set bits to z */
				*bit0 = *bit1 = 'z';
			else {	/* convert char string to integer, mask out the bits */
				tempint = (unsigned int)atoi(pattern);
				*bit0 = ((0x00000001) & tempint) ? '1' : '0';
				*bit1 = ((0x00000002) & tempint) ? '1' : '0';
				}
			break;
		case OCTAL:
			/* pick out rightmost char of octal number */
			tempint = strlen(pattern);
			c = pattern[tempint - 1];
			switch(c) {
				case 'x':
					*bit0 = *bit1 = 'x';
					break;
				case 'z':
					*bit0 = *bit1 = 'z';
					break;
				case '0': case '4':
					*bit0 = *bit1 = '0';
					break;
				case '1': case '5':
					*bit0 = '1'; *bit1 = '0';
					break;
				case '2': case '6':
					*bit0 = '0'; *bit1 = '1';
					break;
				case '3': case '7':
					*bit0 = *bit1 = '1';
					break;
				default:
					printf("two_least: %s not legal octal number\n",
						pattern);
					break;
			}
			break;
		case HEX:
			/* pick out rightmost char of hex number */
			tempint = strlen(pattern);
			c = pattern[tempint - 1];
			switch(c) {
				case 'x':
					*bit0 = *bit1 = 'x';
					break;
				case 'z':
					*bit0 = *bit1 = 'z';
					break;
				case '0': case '4': case '8': case 'c':
					*bit0 = *bit1 = '0';
					break;
				case '1': case '5': case '9': case 'd':
					*bit0 = '1'; *bit1 = '0';
					break;
				case '2': case '6': case 'a': case 'e':
					*bit0 = '0'; *bit1 = '1';
					break;
				case '3': case '7': case 'b': case 'f':
					*bit0 = *bit1 = '1';
					break;
				default:
					printf("two_least: %s not legal hex number\n",
						pattern);
					break;
			}
			break;
		default:
			break;
	}
}


/* Same rules as preceeding procedure regarding starti, endi.  */
int
search_multi_signal(sigct, conversion, signals, direction, starti, endi, patterns)
	int starti, endi, direction, sigct;
	sigT *signals;
	char *patterns;
	int conversion;
{
	int vectornum;
	int numpats;
	char lsb_list[S_MAXLIST][2];
	char *origpatterns, *onepattern, *next_token();
	char *firstsig, *secsig;
	int patlen, lasti;

	/* fill lsb_list with the two most lsb's of each pattern element*/
	origpatterns = patterns;
	numpats = 0;
	onepattern = next_token(&patterns);
	patlen = strlen(onepattern);
	while (*onepattern != '\0') {
		two_least_sig_bits(conversion, onepattern,
			&lsb_list[numpats][0], &lsb_list[numpats][1]);
		onepattern = next_token(&patterns);
		numpats++;
	}

	firstsig = signals->dig_Y;
	secsig = signals->next->dig_Y;

	if (direction == FORWARDS) {
		vectornum = -1; /* pattern not yet found */
		lasti = endi - numpats + 1;
		while ((vectornum == -1) && (starti <= lasti)) {
			if (lsb_match(firstsig, secsig, numpats, starti, lsb_list)) 
				if (rest_matches(sigct, numpats, patlen, origpatterns, signals, 
						starti, conversion, endi)) 
					vectornum = starti;
				else starti++;
			else starti++;
		}
	}
	else {
		vectornum = -1; /* pattern not yet found */
		/* Point starti at the vector which could be the pattern beginning.  We 
		   do this by moving it back the length of the pattern.  */
		starti = starti - numpats + 1;

		while ((vectornum == -1) && (starti >= endi)) {
			if (lsb_match(firstsig, secsig, numpats, starti, lsb_list)) 
				if (rest_matches(sigct, numpats, patlen, origpatterns, signals, 
						starti, conversion, starti + numpats - 1)) 
					vectornum = starti;
				else starti--;
			else starti--;
		}
	}
	return(vectornum);
}

#define BIT_EQUIV(_a_, _b_)	(((_a_) == (_b_)) || ((_a_) == 'x') || ((_a_) == 'z'))

/* Do the two lsb's  match the two lsb's in every pattern element in 
	the list of patterns? */
static int
lsb_match(firstsig, secsig, numpats, v, lsb_list)
	int v, numpats;
	char *firstsig, *secsig;
	char lsb_list[S_MAXLIST][2];
{
	int pat_elt;
	int okay;

	pat_elt = 0;
	okay = 1;

	while ((pat_elt < numpats) && okay) {
		/* check that pattern lsbs match actual data lsbs */
		if (BIT_EQUIV(lsb_list[pat_elt][0], firstsig[v + pat_elt]) &&
			BIT_EQUIV(lsb_list[pat_elt][1], secsig[v + pat_elt]))
			pat_elt++;
		else okay = 0;
	}
	return(okay);	
}

/* patterns now points to patterns delineated by '\0' instead of blanks */
static int
rest_matches(sigct, numpats, patlen, patterns, signals, starti, conversion, endi)
	int patlen, sigct, starti, numpats, conversion, endi;
	sigT *signals;
	char *patterns;
{
	int i, count;
	char *raw, *converted;
	sigT *sigptr;
	char *patptr;

	raw = (char *)calloc(sigct+1, sizeof(char));
	converted = (char *)calloc(patlen+1, sizeof(char));
	patptr = patterns;
	count = 0;
	while ((starti <= endi) && (count < numpats)) {
		sigptr = signals;
		for (i = sigct-1; i >= 0; i--) {
			raw[i] = sigptr->dig_Y[starti];
			sigptr = sigptr->next;
		}
		raw[sigct] = '\0';
		convert(raw, sigct, converted, conversion);
		if (strcmp(patptr,  converted) != 0) {
			free((char *)raw);
			free((char *)converted);
			return(0);
		}
		starti++;
		count++;
		while (*patptr != '\0')
			patptr++;
		patptr++;
	}
	free((char *)raw);
	free((char *)converted);
	if (count == numpats)
		return(1);
	else return(0);
}
