/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "gp.h"

extern comm_file_t *Sig_files;
extern int Sig_precision;
extern int Sig_decimals;
extern vert_t Sig_leftvert, Sig_rightvert;
extern int Sig_signm_ct;

axis_t *Sig_axes;
int Sig_axis_ct; /* number of axes actually on screen as opposed to scrolled
					off, above or below. */
axis_t *Sig_top_axis, *Sig_bottom_axis;
double Sig_timestart, Sig_timeend, Sig_interval;
double Sig_units; /* multiply digital vector number by this to get seconds */
double Sig_multiplier; /* multiply xvalues by this to get timescale values */
extern int Sig_interactive_mode;

int Sig_notype, Sig_nums;
int Global_quit, Sig_yaxes;
int Sig_main;  /* win ids for the main window of the transient mode */
int GP_mode; /* did the current command come from a GP_win? */
extern gp_win_t *GP_currwin, *GP_wins;
extern int GP_currwin_id;

main(argc, argv)
	char **argv;
	int argc;
{
	int count;
	double maxtime_of_all_runs();

	fprintf(stderr, "Sigview3.1 at your service....\n");

	init_globals();

	for (count = argc; count > 1; count--)
		load_simfile(*++argv);

	determine_mode(); 

	if (! read_cmdfile()) 
		make_default_axes();

	if (Global_quit)
		return;

	if (GP_mode) {
		if (! graph_dev_init("sigview", 0))
			return;
		GP_currwin_id = GP_wins->id = 0;
		Sig_main = -1;
	}
	else {
		if (! graph_dev_init("sigview", 1))
			return;
		Sig_main = 0;
	}

	graph_color_map(); 
	graph_font_set("medium");
	graph_cursor("pointer");
	graph_char_just_set("cc");
	graph_window_set(0, 0.0, 0.0, 600.0, 500.0);

	if (GP_mode) 
		gp_draw_screen(GP_wins->id);
	else draw_screen();

	interact();
}


init_globals()
{
	gp_win_t *define_gp_win();

	Sig_signm_ct = 0;
	Sig_interactive_mode = 0;
	Sig_notype = 0;
	Sig_nums = 1;
	Sig_yaxes = 0;
	Sig_multiplier = 1000000000.0; /* multiply xvalues by this */
	Sig_timestart = 0.0;
	Sig_timeend = maxtime_of_all_runs();
	Sig_decimals=2;
	if (Sig_timeend == 0.0)
		Sig_timeend = Sig_interval = 40.0 / Sig_multiplier;
	else Sig_interval = Sig_timeend - Sig_timestart;

	Sig_units = 0.000000001; /* one ns per digital vector */
	Sig_precision = 0;
	Sig_leftvert.x = AXISLEFT + 70.0;
	Sig_rightvert.x = AXISRIGHT - 70.0;
	if (Sig_files != NULL) {
		Sig_leftvert.run = Sig_files->runs;
		Sig_rightvert.run = Sig_files->runs;
	}
	else Sig_leftvert.run = Sig_rightvert.run = NULL;
	Sig_axis_ct = 0;
	Sig_files = NULL;
	Sig_axes = Sig_top_axis =Sig_bottom_axis = NULL;

	Global_quit = 0;
	GP_mode = 0;
	Sig_main = -1;
	GP_wins = GP_currwin = define_gp_win();
	GP_currwin_id = -1;
}

/* before reading the initial command file, we make a preliminary guess,
   based on the types of files read in at startup, as to what the mode
   may be.  General mode when only AC or TRANSFER runs loaded, else
   transient mode. */

static
determine_mode() 
{
	comm_file_t *file_ptr;
	run_t *run_ptr;

	if (Sig_files == NULL) {
		GP_mode = 0;  /* Sigview mode will be default if no runs loaded */
		return;
	}

	GP_mode = 1; /* assume we have only AC and TRANSFER runs loaded */

	file_ptr = Sig_files;
	while (file_ptr != NULL) {
		run_ptr = file_ptr->runs;
		while (run_ptr != NULL) {
			if ((run_ptr->run_type != AC) &&
				(run_ptr->run_type != TRANSFER) &&
				(run_ptr->run_type != CHAR_CURVES))
				GP_mode = 0;
			run_ptr = run_ptr->next;
		}
		file_ptr = file_ptr->next;
	}
}

double
maxtime_of_all_runs()
{
	comm_file_t *file_ptr;
	run_t *run_ptr;
	double dig_maxtime, anal_maxtime;

	file_ptr = Sig_files;
	dig_maxtime = anal_maxtime = 0.0;

	while (file_ptr != NULL) {
		run_ptr = file_ptr->runs;
		while (run_ptr != NULL) {

			if (run_ptr->run_type == DIGITL) 
				dig_maxtime =  SMAX(dig_maxtime, 
					(double)run_ptr->data.digital.X[run_ptr->num_X-1]);
			else anal_maxtime = SMAX(anal_maxtime,
					run_ptr->data.analog[0][run_ptr->num_X-1]);
			run_ptr = run_ptr->next;
		}
		file_ptr = file_ptr->next;
	}
	dig_maxtime = dig_maxtime * Sig_units;
	return(SMAX(dig_maxtime, anal_maxtime));
}

make_default_axes()
{
	comm_file_t *file_ptr;
	run_t *run_ptr;
	sigT *signal;
	int i;
	double dig_maxtime, anal_maxtime;
	axis_t *add_axis();

	if (GP_mode) {
		gp_make_default_axes();
		return;
	}

	file_ptr = Sig_files;
	Sig_axis_ct = 0;
	Sig_units = 0.000000001;
	Sig_timestart = 0.0;
	dig_maxtime = anal_maxtime = 0.0;

	while (file_ptr != NULL) {
		run_ptr = file_ptr->runs;
		while (run_ptr != NULL) {

			if (run_ptr->run_type == DIGITL) {
				dig_maxtime =  SMAX(dig_maxtime, 
					(double)run_ptr->data.digital.X[run_ptr->num_X-1]);
				for (i = 0; i < run_ptr->num_Y; i++) {
					signal = (sigT *) malloc(sizeof(sigT));
					signal->run = run_ptr;
					signal->dig_Y = run_ptr->data.digital.Y[i];
					signal->anal_Y = NULL;
					signal->next = NULL;
					add_axis(run_ptr->Ynames[i], Sig_timestart, signal, NORMAL);
				}
			}
			else {
				anal_maxtime = SMAX(anal_maxtime,
					run_ptr->data.analog[0][run_ptr->num_X-1]);
				for (i = 1; i < run_ptr->num_Y; i++) {
					signal = (sigT *) malloc(sizeof(sigT));
					signal->run = run_ptr;
					signal->anal_Y = run_ptr->data.analog[i];
					signal->dig_Y = NULL;
					signal->next = NULL;
					add_axis(run_ptr->Ynames[i], Sig_timestart, signal, NORMAL);
				}
			}
			run_ptr = run_ptr->next;
		}
		file_ptr = file_ptr->next;
	}

	dig_maxtime = dig_maxtime * Sig_units;
	Sig_timeend = SMAX(dig_maxtime, anal_maxtime);
	if (Sig_timeend == 0.0)
		Sig_timeend = 40.0/Sig_multiplier;
	Sig_interval = Sig_timeend - Sig_timestart;
}

/* Add this axis to the axis list, and adjust screen pointers such that the
   new axis will be the last visible one at the bottom of the screen. */

axis_t *
add_axis(nm, begintime, siglist, type)
	char *nm;
	double begintime;
	sigT *siglist;
	int type;
{
	axis_t *newaxis;

	newaxis = (axis_t *)malloc(sizeof(axis_t));
	unique_name(nm, newaxis->name);

	newaxis->yaxis.type = LINEAR;
	set_min_max_yscale(siglist, &(newaxis->yaxis.minv), &(newaxis->yaxis.maxv));
	newaxis->yaxis.defawlt = 1;

	newaxis->axistype = type;
	newaxis->timebegin = begintime;
	newaxis->signals = siglist;

	insert_axis(newaxis, Sig_bottom_axis);
	return(newaxis);
}

/* this is called by copy and by add_axis.  Add the new axis right below
   the "above" axis.  If "above" is onscreen, adjust those pointers and 
   Sig_axis_ct.  If "above" is NULL then we must want to insert it at the
   head of Sig_axes.
*/ 
insert_axis(axis, above)
	axis_t *axis, *above;
{
	axis_t *temp;

	axis->next = NULL;
	axis->previous = NULL;

	if (Sig_axes == NULL)  {
		Sig_axes = Sig_top_axis = Sig_bottom_axis = axis;
		Sig_axis_ct = 1;
		return;
	}
	if (above == NULL) {
		axis->previous = NULL;
		axis->next = Sig_axes;
		Sig_axes->previous = axis;
		Sig_axes = axis;
	}
	else {
		axis->previous = above;
		axis->next = above->next;
		if (axis->next != NULL)
			axis->next->previous = axis;
		above->next = axis;
	}
	if (above == Sig_bottom_axis) {
		Sig_bottom_axis = axis;
		Sig_axis_ct++;
		return;
	}
	/* else we must be inserting somewhere in the middle and we don't
	   know if our new axis is onscreen or not */
	Sig_axis_ct=0; temp = Sig_top_axis;
	while (temp != Sig_bottom_axis->next) {
		Sig_axis_ct++;
		temp = temp->next;
	}
}

set_min_max_yscale(sigptr, ax_minv, ax_maxv)
	sigT *sigptr;
	double *ax_minv, *ax_maxv;
{
	int notset;
	double maxv, minv;
	double *voltages;
	int i;

	notset = 1;
	while (sigptr != NULL) {
		 if (sigptr->anal_Y != NULL) {
			if (notset) {
				maxv = minv = sigptr->anal_Y[0];
				notset = 0;
			}
			voltages = sigptr->anal_Y;
			for (i = 1; i < sigptr->run->num_X; i++) {
				maxv = SMAX(maxv, voltages[i]);
				minv = SMIN(minv, voltages[i]);
			}
		}
		sigptr = sigptr->next;
	}
	if (notset) {
		*ax_minv = 0.0;
		*ax_maxv = 5.0;
	}
	else {
		if (minv == maxv)
			maxv = minv + 0.1;
		*ax_minv = minv;
		*ax_maxv = maxv;
	}
}

/* Run through the list of all current axes and make sure that this name
   has never occurred before.  If it has, add UNIQUECHAR and a subscript
   number to distinguish it.  THIS involves finding all axes which have
   the name, e.g., if UNIQUECHAR is -, then q-X */

unique_name(nm, newnm)
	char *nm, *newnm;
{
	char existing_axnm[SIGNMSIZE];
	int dup_ct;
	axis_t *axptr;

	strip_subscript(nm);

	/* count the number of axes which have this same name */
	dup_ct = 0;
	axptr = Sig_axes;
	while (axptr != NULL) {
		if (axptr->name[0] != '\0') {
			strcpy(existing_axnm, axptr->name);
			/* first, strip name of possible -X construction */
			strip_subscript(existing_axnm);

			if (strcmp(existing_axnm, nm) == 0)
				dup_ct++;
		}
		axptr = axptr->next;
	}

	if (dup_ct == 0)
		strcpy(newnm, nm);
	else sprintf(newnm, "%s%c%d", nm, UNIQUECHAR, dup_ct);
}

strip_subscript(nm)
	char nm[];
{
	char *ptr, *rindex();
	int subscript;

	ptr = rindex(nm, UNIQUECHAR); 
	if (ptr == (char *)0)
		return;

	ptr++;
	if (sscanf(ptr, "%d", &subscript) != 1)
		return;
	ptr--;

	*ptr = '\0';
}
