/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.


	JTS AWG -- 10/20/95 --	Changed type sig_t to sigT to do DEC Alpha compile
--------------------------------------------------------------------------- */

#include <stdio.h>
#include <math.h>
#include <string.h>

#define MINUS_INFINITY	-30.0

#define SMAX(__x, __y)	(((__x) > (__y)) ? (__x) : (__y))
#define SMIN(__x, __y)	(((__x) < (__y)) ? (__x) : (__y))
#define ABS(__x)		(((__x) < 0.0) ? (-(__x)) : (__x))
#define NEGLOGTEN(__X)	(((__X) == 0.0) ? (MINUS_INFINITY) : log10(ABS(__X))) 
#define LOGTEN(__x)		(((__x) <= 0.0) ? NEGLOGTEN(__x) : (log10(__x)))

/* below are the different file types */
#define CAZM		1 /* set the zeroth bit to indicate analog files */	
#define SPICE		3	
#define RAW			5
#define HSPICE		7

#define ESIM		2 /* zeroth bit is clear to indicate digital files */
#define HILO		4	
#define RNL			6	

/* and these are the different run types */
#define TRANSIENT	1
#define TRANSFER	3	
#define AC			5	
#define CHAR_CURVES 7
#define DIGITL		2

#define DIGITAL(_t_)	((((_t_) % 2) == 0) ? 1 : 0)
#define LOWERCASE(_c_)	((((_c_) > 64) && ((_c_) < 91)) ? ((_c_) + 32) : (_c_))

#define BUFSIZE		4096	
#define SIGNMSIZE	128	
#define	BLOCKSIZE	1024	

#define	FORWARDS	0
#define BACKWARDS	1
#define	LEFT		0
#define RIGHT		1

#define	UNIQUECHAR	'\\'

typedef struct dig_run_s dig_run_t;
struct dig_run_s {
	double *X;			/* array of timesteps where some voltage changes */
	char **Y;			/* array of array of values for each signal in run */
};

typedef struct run_s run_t;
struct run_s {
	char name[SIGNMSIZE]; /* run name as so: filename_runnumber */
	int num_X;			/* number of elements in a vector */
	int num_Y;			/* number of vectors in this run */
	int run_type;		/* either TRANSIENT,TRANSFER,AC,CHAR_CURVES,DIGITL*/
	int layer;			/* layer number in which run plots are drawn */
	int num_sets;		/* number of sets of data in this CHAR_CURVES run */
	char **Ynames;	    /* list of vector names */
	union {
		dig_run_t digital;
		double **analog;
	} data;
	run_t *next;
};

typedef struct comm_file_s comm_file_t;
struct comm_file_s {
	char name[SIGNMSIZE];	/* name of the file */
	int type;				/* either HILO, RNL,ESIM,CAZM,SPICE */
	run_t *runs;
	comm_file_t *next;
};


#define LINEAR	0
#define LOG	1

#define NORMAL	0
#define HEX		1
#define OCTAL	2
#define DECIMAL 3
#define BINARY  4

#define AXISLEFT	100.0
#define AXISRIGHT	590.0

typedef struct sig_s sigT;
struct sig_s {
	run_t *run;
	/* The next two are pointers to array of pointers to dig or anal data. 
	   One or the other is used, depending on the data type.  */
	char *dig_Y;
	double *anal_Y;
	sigT *next;
};

typedef struct yaxis_s yaxis_t;
struct yaxis_s {
	int type;		/* LINEAR or LOG */
	double minv, maxv;
	int defawlt;	/* have minv, maxv been reset by the user? */
};

typedef struct axis_s axis_t;
struct axis_s {
	char name[64];
	yaxis_t yaxis;
	int axistype;	/* a bus type or NORMAL */
	double timebegin;	/* if -1.0, use screen timebegin, else use this one */
	sigT *signals; /* one of these for every signal on the axis */
	axis_t *next;
	axis_t *previous;
};

typedef struct vert_s vert_t;
struct vert_s {
	double x;
	run_t *run;
};

typedef struct v_info_s v_info_t;
struct v_info_s {
	sigT *signal;
	double timestart;
	double othervolt;
	double ypos;
	double dvypos;
	int type; /*LINEAR OR LOG */
	v_info_t *next;
};

extern char graph_await_event();

