/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include "hardcopy.h"

extern func_t Hard_funcs;
extern int H_lmap[];
extern H_cmap_t H_cmap[];

FILE	*Tek_spool;
char Tek_fonttype[12];
int Tek_color;
int	Tek_strwidth;
int	Tek_strht;
int	Tek_strsp;
int Tek_devtype;
char Tek_cmd[256];
char Tek_filenm[256];

/* Some useful characters. */

#define EC	27
#define FS	28
#define GS	29
#define US	31
#define FF	12

/* Tektronix commands. They must be preceeded by an Escape. */

#define	DRAW			"LG"
#define	DRAW_MARKER		"LH"
#define	DRAW_RECTANGLE		"UR"
#define	GRAPHIC_TEXT		"LT"
#define	HARDCOPY		"KH"
#define	MOVE			"LF"
#define	PAGE			"\014"
#define	SELECT_FILL_PATTERN	"MP"
#define	SET_CHARACTER_PATH	"MN"
#define	SET_GRAPHTEXT_SIZE	"MC"
#define	SET_LINE_INDEX		"ML"
#define	SET_LINE_STYLE		"MV"
#define	SET_MARKER_TYPE		"MM"
#define	SET_SURFACE_COLOR_MAP	"TG"
#define	SET_TEXT_INDEX		"MT"
#define SET_BKGD_COLOR		"TB"
#define SET_COLOR_MODE		"TM"
#define SET_VIEW_ATTRIBUTES	"RA"
#define SET_SURFACE_DEFINITIONS	"RD"
#define SET_WINDOW	"RW"
#define SET_IMAGE_ORIENTATION	"QO"
#define SET_LINE_WIDTH	"MW"

int
tek_init(devtype, astring)
	char *devtype, *astring;
{
	FILE *open_fl();

	/* determine whether we send to a real file or to a temp file to be
	   sent to an actual Tektronix. */
	Tek_devtype = TFILE;

	if (strcmp(devtype, "tek") == 0) {
		unique_tempnm(Tek_filenm, "#Sig_tekXXXXXX");
		if ((Tek_spool = open_fl(Tek_filenm, "w")) == NULL) {
			sig_msg("Cannot open temp file ",Tek_filenm," for Tektronix plot");
			return(0);
		}
		sprintf(Tek_cmd, "%s %s ; rm %s", astring, Tek_filenm, Tek_filenm);
		Tek_devtype = TDIRECT;
	}
	else {
		strip_crap(astring);
		if ((Tek_spool = open_fl(astring, "w")) == NULL) {
			sig_msg("Cannot open ",astring, "for Tektronix dump");
			return(0);
		}
		else strcpy(Tek_filenm, astring);
	}
	/* set the graphics surface to be white after an erase */
	_ztk_cmd(SET_VIEW_ATTRIBUTES);
	_ztk_word(-1);
	_ztk_word(0);
	_ztk_word(0);

	/* Erase graphics area. */
	_ztk_cmd(PAGE);

	/* set line style to solid, just in case it's been mucked up */
	_ztk_cmd(SET_LINE_STYLE);
	_ztk_word(0);
	_ztk_cmd(SET_LINE_WIDTH);
	_ztk_word(1);

	strcpy(Tek_fonttype, "");
	tek_font_set("medium");

	Tek_color = 0;
	return(1);
};

tek_color_map()
{
	int i;
	int red, green, blue;

	/* set the color mode to machine RGB */
	_ztk_cmd(SET_COLOR_MODE);
	_ztk_word(4);
	_ztk_word(0);
	_ztk_word(0);

	_ztk_cmd(SET_SURFACE_COLOR_MAP);
	_ztk_word(-1);
	_ztk_word(H_MAXCOLORS * 4);

	for (i = 0; i < H_MAXCOLORS; i++) {
		_ztk_word(i);
		red = (int) (H_cmap[i].red * 255.0);
		green = (int) (H_cmap[i].green * 255.0);
		blue = (int) (H_cmap[i].blue * 255.0);
		_ztk_word(red);
		_ztk_word(green);
		_ztk_word(blue);
	}
}

static double Gx1, Gy1;		/* current lower left coords of the "world" of
							   the graphics program using graph.c */
static double Gx2, Gy2;
static double GtoX_scalex, GtoX_scaley;
static double XtoG_scalex, XtoG_scaley;
#define TEK_XRES	4095
#define TEK_YRES	3132

int
tek_window_set(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	_ztk_cmd(SET_WINDOW);
	_ztk_xy(0, 0);
	_ztk_xy(TEK_XRES, TEK_YRES);

	_ztk_cmd(SET_IMAGE_ORIENTATION);
	_ztk_word(0);
	
	if (x1 == x2) {
		fprintf(stderr,
			"tek_window_set: not resetting scale -- window has no width\n");
		return(0);
	}
	if (y1 == y2) {
		fprintf(stderr,
			"tek_window_set: not resetting scale -- window has no height\n");
		return(0);
	}
/*
	Gx1 = SMIN(x1,x2);
	Gy1 = SMIN(y1,y2);
	Gx2 = Gx1 + (ABS((x2 - x1)));   
	Gy2 = Gy1 + (ABS((y2 - y1)));
*/
	Gx1 = x1;
	Gx2 = x2;
	Gy1 = y1;
	Gy2 = y2;

	GtoX_scalex = TEK_XRES/(x2 - x1);   
	GtoX_scaley = TEK_YRES/(y2 - y1);
	XtoG_scalex = 1.0 / GtoX_scalex;
	XtoG_scaley = 1.0 / GtoX_scaley;

	return(1);
}

/* user coordinate (tx,ty) to Tek coordinate (x,y) */
GtoTek(x, y, tx, ty)
	double x, y;
	int *tx, *ty;
{
/*
	if (x > Gx2) x = Gx2;
	if (y > Gy2) y = Gy2;
*/
	*tx = (int)((x - Gx1) * GtoX_scalex);
	*ty = (int)((y - Gy1) * GtoX_scaley);
}

static 
TektoG(x, y, tx, ty)
	int x, y;
	double *tx, *ty;
{
	*tx = (double) ((x * XtoG_scalex) + Gx1);
	*ty = (double) ((y * XtoG_scaley) + Gy1);
}

tek_font_set(type)
	char *type;
{
	if ((strcmp(Tek_fonttype, type)) == 0)
		return;

	strcpy(Tek_fonttype, type);
	_ztk_cmd(SET_GRAPHTEXT_SIZE);
	if (strcmp(type, "small")==0) {
		Tek_strwidth = 20;
		Tek_strht = 35;
		Tek_strsp = 10;
	}
	else {
		Tek_strwidth = 30;
		Tek_strht = 42;
		Tek_strsp = 10;
	}
	_ztk_word(Tek_strwidth);
	_ztk_word(Tek_strht);
	_ztk_word(Tek_strsp);
}

#define CENTERCENTER	0
#define LOWERLEFT		2
#define UPPERLEFT		3
#define BOTTOMCENTER	5
#define RIGHTCENTER		6

static int Tek_charjust;

tek_char_just_set(type)
	char *type;
{
	/* tell graph_text where to put text in relation to points.  */
	if (strcmp(type,"cc")== 0)
		Tek_charjust = CENTERCENTER;
	else if (strcmp(type,"rc")== 0)
		Tek_charjust = RIGHTCENTER;
	else if (strcmp(type,"ll")== 0)
		Tek_charjust = LOWERLEFT;
	else if (strcmp(type,"ul")== 0)
		Tek_charjust = UPPERLEFT;
	else if (strcmp(type,"bc")== 0)
		Tek_charjust = BOTTOMCENTER;
}

tek_text(x, y, string)
	double x, y;
	char *string;
{
	int xx, yy;
	int strwidth, strht;
	int len;

	len = strlen(string);
	strwidth = (Tek_strwidth * len) + (Tek_strsp * (len - 1));
	strht = Tek_strht;

	switch (Tek_charjust) {
		case LOWERLEFT:	
			GtoTek(x, y, &xx, &yy);
			break;
		case CENTERCENTER:
			strwidth = strwidth/2; 
			strht = strht / 2;
			GtoTek(x, y, &xx, &yy);
			xx = xx - strwidth;
			yy = yy - strht;
			break;
		case RIGHTCENTER:
			strht = strht / 2;
			GtoTek(x, y, &xx, &yy);
			yy = yy - strht;
			xx = xx - strwidth;
			break;
		case UPPERLEFT:	
			GtoTek(x, y, &xx, &yy);
			yy = yy - strht;
			break;
		case BOTTOMCENTER:
			strwidth = strwidth/2; 
			GtoTek(x, y, &xx, &yy);
			xx = xx - strwidth;
			break;
		default:
			printf("graph_text: invalid char_just set.\n");
			return;
	}

	_ztk_cmd(MOVE);
	_ztk_xy(xx, yy);

	sconv(string);
	_ztk_cmd(GRAPHIC_TEXT);
	_ztk_word(len);
	fputs(string, Tek_spool);
}

static
sconv(p)
register char *p;
{
	for ( ; *p; p++) {
		*p &= 0177;
		if (*p < ' ')
			*p = ' ';
	}
}

double
tek_str_width(str)
	char *str;
{
	int len, strwidth;
	double x,y;

	len = strlen(str);
	strwidth = (Tek_strwidth * len) + (Tek_strsp * (len - 1));
	TektoG(strwidth,0,&x,&y);
	return(x);
}

tek_set_layer(which)
	int which;
{
	if (Tek_color == H_lmap[which])
		return;

	Tek_color = H_lmap[which];
	_ztk_cmd(SET_LINE_INDEX);
	_ztk_word(Tek_color);
	_ztk_cmd(SET_TEXT_INDEX);
	_ztk_word(Tek_color);
}

tek_term()
{
	_ztk_cmd(HARDCOPY);
	_ztk_word(0);

	/* These commands requested to be added (December 18, 1987) so that
	   users of PicSure (a program which does not properly initialize
	   the Tektronix, apparently), will not get our color map.  It
	   resets the rasterizer back to it's default color map. I'm not
	   really sure which subset of these four commands really does the
	   trick, but what the heck.
	*/
	_ztk_cmd(SET_SURFACE_DEFINITIONS);

	/* set the graphics surface to white after an erase */
	_ztk_cmd(SET_VIEW_ATTRIBUTES);
	_ztk_word(-1);
	_ztk_word(0);
	_ztk_word(0);

	/* Erase graphics area. */
	_ztk_cmd(PAGE);

	/* set the color mode to what it is on power-up */
	_ztk_cmd(SET_COLOR_MODE);
	_ztk_word(3);
	_ztk_word(1);
	_ztk_word(1);

	fclose(Tek_spool);
	if (Tek_devtype == TDIRECT) {
		sig_msg("Executing:  ", Tek_cmd, "");
		if (system(Tek_cmd))
			sig_msg("Could not execute: ",Tek_cmd,"");
	}
	else sig_msg("Have written tek commands to ", Tek_filenm, "");
}

tek_line(x1, y1, x2, y2)
	double x1, y1, x2, y2;
{
	int x, y;

	_ztk_cmd(MOVE);
	GtoTek(x1, y1, &x, &y);
	_ztk_xy(x, y);

	_ztk_cmd(DRAW);
	GtoTek(x2, y2, &x, &y);
	_ztk_xy(x, y);
}

tek_connect_dots(Dotlist, Dotpointer)
	XPoint *Dotlist;
	int Dotpointer;
{
	int i;

	fprintf(Tek_spool, "%c", GS); /* get into VECTOR MODE */
	for (i = 0; i < Dotpointer; i++)
		_ztk_xy(Dotlist[i].x, Dotlist[i].y);
	fprintf(Tek_spool, "%c", US); /* get into ALPHA MODE */
}

/* This code was largely stolen from the Z driver for the Tektronix and
   was largely written by Steven Daniel, who may or may not be large, given
   as I've never seen the guy. */

/* Output a command. All commands are preceeded by an escape.  */
_ztk_cmd(s)
	char *s;
{
	fprintf(Tek_spool, "%c%s", EC, s);
}

/* Send out an xy pair */
_ztk_xy(x, y)
	int x, y;
{
	/*
	 * Standard tektronix output of a coordinate.
	 * Five bytes.
	 *	HIY, EB, LOY, HIX, LOX.
	 *
	 * Where hi bytes contain bits 8 - 12,
	 * lo bytes contain bits 3 - 7,
	 * and EB contains  y2,y1,x2,x1.
	 */

	char xy[6];

	xy[5] = '\0';

	/* High bytes */
	xy[0] = 0x20 | ((y>>7) & 0x1f);
	xy[3] = 0x20 | ((x>>7) & 0x1f);

	/* Low bytes */
	xy[2] = 0x60 | ((y>>2) & 0x1f);
	xy[4] = 0x40 | ((x>>2) & 0x1f);

	/* extended byte */
	xy[1] = 0x60 | ((y & 0x3) << 2) | (x & 0x3);

	fputs(xy, Tek_spool);
}

/* Output a word value.  */
_ztk_word(i)
	int i;
{
	/*
	 * Word is 16 bits maximum
	 * Variable number of bytes are to be printed.
	 * The High bytes (all but the last) have 6 data
	 * bits and 1 mark bit (01xxxxxx). The last byte
	 * has 4 data bits, one sign bit, and 1 mark bit (001sxxxx).
	 * Put out the minimum number of high bytes.
	 */

	int more = 0;
	int sign = 0x10;

	if (i < 0) {
		sign = 0;
		i = -i;
	}

	if (i >= (1<<10)) {
		fputc(0x40 | (i >> 10), Tek_spool);
		more++;
	}

	if (more || i >= (1<<4))
		fputc(0x40 | ((i >> 4) & 0x3f), Tek_spool);
	
	fputc(0x20 | sign | (i & 0xf), Tek_spool);
}

connect_to_tek_funcs()
{
	Hard_funcs.init = tek_init;
	Hard_funcs.color_map = tek_color_map;
	Hard_funcs.window_set = tek_window_set;

	Hard_funcs.font_set = tek_font_set;
	Hard_funcs.char_just_set = tek_char_just_set;
	Hard_funcs.text = tek_text;
	Hard_funcs.str_width = tek_str_width;

	Hard_funcs.set_layer = tek_set_layer;

	Hard_funcs.GtoWhatever = GtoTek;
	Hard_funcs.connect_dots = tek_connect_dots;

	Hard_funcs.line = tek_line;

	Hard_funcs.term = tek_term;
}

