/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <stdio.h>
#include <math.h>
#include "hardcopy.h"
#include <X11/Xutil.h>

Display		*Graph_display;
int			Graph_screen;

typedef struct win_list_s win_list_t;
struct win_list_s {
	Window x_id; /* X's identifying number for this window */
	int g_id; /* our identifying number for this window */
	int xres, yres; /* size in pixels of this window */
	double gx1, gy1, gx2, gy2; /* current ll and ur coords of the "world" of
							   the graphics program using graph.c */
	double XtoG_scalex, XtoG_scaley; /* scale factors */
	double GtoX_scalex, GtoX_scaley;

	char font_string[8]; /* asked-for font for this window */
	XFontStruct *xfont; /* current font for this window--may be different
		front font_string if window is wrong size for font_string font. */
	long inputmask; /* X mask representing what events this window wants */

	/* if this is a popup window (i.e. at it's death we will redraw
	   only what it hides AND we don't accept input from any other
	   window while it's alive), we need to keep all this crap */

	win_list_t *my_parent; /* parent of this popup */
	int pu_x1, pu_y1; 	/* upper left corner of popup window relative
									to parent window */
	Pixmap underside;	/* holds the underside of the popup window */

	win_list_t *next;
};

win_list_t *Graph_windows;
win_list_t *Graph_currwin; /* the window to which all X requests are currently
							sent */
win_list_t *Graph_exc_input_win; /* when we are looking for input from
			only one window, this will be pointing to it. */
Window Graph_win_id;	/* X's identifying number for Graph-currwin */

static XFontStruct	*G_fontinfosm;
static XFontStruct	*G_fontinfomd;
static XFontStruct	*G_fontinfo;	/* points to either infosm or infomd */
GC			XGC;

/* these are the values from the current drawing window */
double Gx1, Gy1, Gx2, Gy2;
double GtoX_scalex, GtoX_scaley;
int Yres;

char G_progname[64];

extern int Hardcopy;
extern int Sig_decimals;
extern func_t Hard_funcs;

/* returns 0 or 1 after trying to init an X display, set up cursors
   and fonts, and open the first window. */
int
graph_dev_init(program, big)
	char *program; /* name of program which is calling this thing */
	int big; /* big or small main window? */
{
	extern char *getenv();

	if ((Graph_display = XOpenDisplay (NULL)) == 0) {
		fprintf(stderr,"graph_dev_init: Cannot open X display: %s\n",
			getenv("DISPLAY"));
		return(0);
	}
	Graph_screen = XDefaultScreen(Graph_display);
	Graph_windows = NULL;

	strcpy(G_progname, program);

	if (big) {
		if (graph_create_free_win(0.0, 0.0, 100.0, 100.0, 0.9, 0.8,
				'c', 0, 0, program) == -1)
		return(0);
	}
	else {
		if (graph_create_free_win(0.0, 0.0, 100.0, 100.0, 0.65, 0.65,
				'c', 0, 0, program) == -1)
		return(0);
	}
	Graph_win_id = Graph_windows->x_id;
	Graph_currwin = Graph_windows;

	XGC = XCreateGC(Graph_display, Graph_windows->x_id, 0, (XGCValues *)NULL);

	init_cursors();

	if (setup_fonts(program))
		return(1);
	else return(0);
}

/* defines a new window and maps it to the screen.  Returns its id number. 
   The parent of this new window will be the root window.  This means it
   can be moved and resized by the user.  Will return its non-X id number
   or -1 if it can't be created. */ 
int
graph_create_free_win(wx1, wy1, wx2, wy2, percent_wd, percent_ht,
		where, x, y, program)
	double wx1,wy1,wx2,wy2; /* bounding coords within new window */
	double percent_wd, percent_ht; /* if user doesn't define size in
		.Xdefaults, then set the size of the window by these percents */
	int where; /* if user doesn't define placement in Xdefaults, then
		'c' means center it on screen, 'r' means place it randomly, and
		'a' means place it at the pt indicated by the next two args. */
	int x, y; /* where to place window if "absolute" set */
	char *program; /* identifying string to look for in .Xdefaults */
{
	int display_wd, display_ht;
	int xoffset, yoffset, done;
	int wd, ht;
	char *geom, defaultgeom[64];
	XEvent ev;
	XWMHints myhints;
	XWindowAttributes get_atts;
	XSetWindowAttributes set_atts;
	win_list_t *new_window, *add_new_win();
	Window win_id;
	int garbage;

	display_wd = XDisplayWidth(Graph_display, Graph_screen);
	display_ht = XDisplayHeight(Graph_display, Graph_screen);

	wd = (int) (display_wd * percent_wd);
	ht = (int) (display_ht * percent_ht);

	if (where == 'a') {
		xoffset = x;
		yoffset = y;
	}
	else if (where == 'c') {
		xoffset = (int) ((display_wd - wd)/2); 
		yoffset = (int) ((display_ht - ht)/2); 
	}
	else {
/*		xoffset = random() % (display_wd - wd);
		yoffset = random() % (display_ht - ht);
	*/
	}

	/* let's see if a size has been defined in .Xdefaults */
	sprintf(defaultgeom, "%dx%d+%d+%d", wd, ht, xoffset, yoffset);
	geom = XGetDefault(Graph_display, program, "Geometry");

	if (geom != NULL) {
		garbage =  XGeometry(Graph_display, Graph_screen, 
			geom, defaultgeom, 0, 0, 0, 0,0, &xoffset, &yoffset, &wd, &ht);
		if (garbage)
			set_atts.override_redirect = True;
		else set_atts.override_redirect = False;
	}
	else set_atts.override_redirect = False;

	set_atts.event_mask = ExposureMask | StructureNotifyMask;
	set_atts.border_pixel = (unsigned long) (graph_colormap_begin() + 2);

	win_id = XCreateWindow(Graph_display,
		RootWindow(Graph_display, Graph_screen), xoffset, yoffset, wd, ht, 
		3, CopyFromParent, InputOutput, CopyFromParent,
		CWEventMask | CWOverrideRedirect | CWBorderPixel, &set_atts);
	
	if (win_id == (Window) 0) {
		fprintf(stderr, "graph_create_free_win: XCreateWindow fails\n");
		return(-1);
	}

	new_window = add_new_win();
	new_window->underside = NULL; /* we can let wm handle redrawing
		undersides since this window will have the root as its parent */
	new_window->x_id = win_id;
	new_window->my_parent = NULL;
	new_window->inputmask = ExposureMask | StructureNotifyMask;

	/* tell the window manager a bunch of things (such as name of the
	   program).  This is recommended by the document "Inter-Client
	   Communication Conventions" which I have never seen, but is
	   referred to in some of my conversion-to-X11 writings I've
	   collected. */
	XSetStandardProperties(Graph_display, win_id, program, program, 
		None, NULL, 0, NULL);
	myhints.flags = InputHint | StateHint;
	myhints.input = True;
	myhints.initial_state = NormalState;
	XSetWMHints(Graph_display, win_id, &myhints);

	XSetWindowBackground(Graph_display, win_id, 0);
	XMapWindow(Graph_display, win_id);

	/* now we wait until the user puts the window on the screen,
	   then we discard all events thereunto, setting the scale once we
	   come upon the configure notify event that specifies it. */

	done = 0;
	XWindowEvent(Graph_display, win_id, new_window->inputmask, &ev);
	sleep(1); /* so that all Expose events will have time to get here */
	while (!done) {
		if (ev.type == ConfigureNotify) {		
			/* now let's find out what size he/she/it chose */
			new_window->xres = ev.xconfigure.width;
			new_window->yres = ev.xconfigure.height;
			set_scale(new_window, wx1, wy1, wx2, wy2);
		}
		if (XCheckWindowEvent(Graph_display, win_id, new_window->inputmask,
			&ev) != True) 
			done = 1;
	}
	if (new_window->xres == 0) {
    	XGetWindowAttributes(Graph_display, win_id, &get_atts);
		new_window->xres = get_atts.width;
		new_window->yres = get_atts.height;
		set_scale(new_window, wx1, wy1, wx2, wy2);
	}
	XClearWindow(Graph_display, win_id); 
	return(new_window->g_id);
}

/* defines a new window and maps it to the screen.  Returns its id number. 
   The parent of this new window will be Graph_currwin.  This means it
   CANNOT be moved and resized by the user.  Will return its non-X id number
   or -1 if it can't be created. */ 
int
graph_create_fixed_win(x1, y1, x2, y2, wx1, wy1, wx2, wy2)
	double x1, y1, x2, y2; /* coords of win within its parent */
	double wx1,wy1,wx2,wy2; /* bounding coords within new window */
{
	int px1, py1, px2, py2;
	int pu_x1, pu_y1, pu_wd, pu_ht;
	win_list_t *new_window, *add_new_win();
	XSetWindowAttributes set_atts;
	Window win_id;

	/* check to make sure this window is wholly within parent */

	if ((x1 < Graph_currwin->gx1) || (x2 > Graph_currwin->gx2) || 
		(y1 < Graph_currwin->gy1) || (y2 > Graph_currwin->gy2)) {
		fprintf(stderr,
			"graph_create_fixed_win: window %f %f, %f %f is outside parent\n",
		 	x1, y1, x2, y2);
		return(-1);
	}

	GtoX(x1,y1,&px1,&py1);
	pu_x1 = px1;
	GtoX(x2,y2,&px2,&py2);
	pu_y1 = py2;
	pu_wd = (int) (fabs((double)(px2 - px1)));	
	pu_ht = (int) (fabs((double)(py2 - py1)));

	/* see if we can open a window */
	set_atts.override_redirect = True;
	win_id = XCreateWindow(Graph_display, Graph_win_id, pu_x1, 
		pu_y1, pu_wd, pu_ht, 0, CopyFromParent, InputOutput, 
		CopyFromParent, CWOverrideRedirect, &set_atts);
	
	if (win_id == (Window) 0) {
		fprintf(stderr, "graph_create_window: XCreateWindow fails\n");
		return (-1);
	}

	/* capture the underside of the window to redraw later */
	XSetFunction(Graph_display, XGC, GXcopy);
	XSetPlaneMask(Graph_display, XGC, (unsigned int)
		(pow(2.0, (double)DefaultDepth(Graph_display, Graph_screen))-1));

	new_window = add_new_win();
	new_window->underside = XCreatePixmap(Graph_display, Graph_win_id, 
		pu_wd, pu_ht, DefaultDepth(Graph_display, Graph_screen));

	XCopyArea(Graph_display, Graph_win_id, new_window->underside, XGC, 
		pu_x1, pu_y1, (unsigned int)pu_wd, (unsigned int)pu_ht, 0, 0);

	new_window->pu_x1 = pu_x1;
	new_window->pu_y1 = pu_y1;
	new_window->my_parent = Graph_currwin;
	new_window->xres = pu_wd;
	new_window->yres = pu_ht;
	new_window->x_id = win_id;
	new_window->inputmask = ExposureMask;
	set_scale(new_window, wx1, wy1, wx2, wy2);
	XMapWindow(Graph_display, new_window->x_id);
	return(new_window->g_id);
}

static win_list_t *
add_new_win()
{
	win_list_t *new_window, *ptr;

	new_window = (win_list_t *) malloc (sizeof(win_list_t));

	/* attach it to the end of the list */
	if (Graph_windows == NULL) {
		Graph_windows = new_window;
		new_window->g_id = 0;
	}
	else {
		ptr = Graph_windows;
		while(ptr->next != NULL)
			ptr = ptr->next;
		ptr->next = new_window;
		new_window->g_id = ptr->g_id + 1;
	}
	new_window->xres = new_window->yres = 0;
	new_window->next = NULL;
	return(new_window);
}


/* try to load a small and a medium font.  Return
   0 if no fonts could be loaded, else return 1. */
static int
setup_fonts(program)
	char *program;
{
	XFontStruct *try_loading_default();

	/* load a small font and a med font */
	if ((G_fontinfosm = try_loading_default("SmallFont", program)) == NULL)
		if ((G_fontinfosm = XLoadQueryFont(Graph_display, "6x10")) == NULL)
			G_fontinfosm = XLoadQueryFont(Graph_display, "fixed");

	if ((G_fontinfomd = try_loading_default("MediumFont", program)) == NULL)
		if ((G_fontinfomd = 
			XLoadQueryFont(Graph_display, "*helvetica*bold*14*")) == NULL)
			G_fontinfosm = XLoadQueryFont(Graph_display, "fixed");

	if (G_fontinfomd == NULL) {
		if (G_fontinfosm == NULL) {
			sig_msg("Can't find fonts!","","");
			return(0);
		}
		G_fontinfomd = G_fontinfosm;
	}

	/* if you have a reasonably large window, use the med font as default */
	if ((Graph_currwin->xres < 650)||(Graph_currwin->yres < 350)) {
		XSetFont(Graph_display, XGC, G_fontinfosm->fid);
		G_fontinfo = G_fontinfosm;
	}
	else {
		XSetFont(Graph_display, XGC, G_fontinfomd->fid);
		G_fontinfo = G_fontinfomd;
	}

	return(1);
}

static XFontStruct *
try_loading_default(fieldnm, program)
	char *fieldnm, *program;
{
	char *fontname;
	XFontStruct	*fontptr;

	fontname = XGetDefault(Graph_display, program, fieldnm);

	if (fontname == NULL) {
		return(NULL);
	}
	else if ((fontptr = XLoadQueryFont(Graph_display, fontname)) == NULL) {
		fprintf(stderr,"setup_fonts: Can't load %s which is\n",fontname);
		fprintf(stderr,"  specified as %s.%s in your ~/.Xdefaults file.\n\n",
			program,fieldnm);
		return(NULL);
	}
	return(fontptr);
}


/* graph_term() -- closes windows, etc. */
graph_term()
{
	if (Hardcopy) { (Hard_funcs.term)(); return; }

	XCloseDisplay(Graph_display);
}


/* this sets the font for the Graph_currwin and also sets the global
   G_fontinfo.  It also sets the X server to this font. */

graph_font_set(type)
	char *type;
{
	if (Hardcopy) { (Hard_funcs.font_set)(type); return; }

	if ((strcmp(Graph_currwin->font_string, type)) == 0)
		return;

	/* save what the person asked for, in case window gets resized later */
	strcpy(Graph_currwin->font_string, type);

	/* if you want small, or you have a small window... */
	if ((strcmp(type, "small")==0)||
		(Graph_currwin->xres < 650)||(Graph_currwin->yres < 350)) {
		XSetFont(Graph_display, XGC, G_fontinfosm->fid);
		G_fontinfo = G_fontinfosm;
		Graph_currwin->xfont = G_fontinfosm;
	}
	else {
		XSetFont(Graph_display, XGC, G_fontinfomd->fid);
		G_fontinfo = G_fontinfomd;
		Graph_currwin->xfont = G_fontinfomd;
	}
}


#define CENTERCENTER	0
#define LOWERLEFT	2
#define UPPERLEFT	3
#define BOTTOMCENTER	5
#define RIGHTCENTER	6
#define LOWERRIGHT	7

int Charjust;

graph_char_just_set(type)
	char *type;
{
	/* tell graph_text where to put text in relation to points.  */

	if (Hardcopy) { (Hard_funcs.char_just_set)(type); return; }
	
	/* right now we only use 'cc' and 'lc' and 'll' and 'ul' and 'lr' */
	if (strcmp(type,"cc")== 0)
		Charjust = CENTERCENTER;
	else if (strcmp(type,"rc")== 0)
		Charjust = RIGHTCENTER;
	else if (strcmp(type,"lr")== 0)
		Charjust = LOWERRIGHT;
	else if (strcmp(type,"ll")== 0)
		Charjust = LOWERLEFT;
	else if (strcmp(type,"ul")== 0)
		Charjust = UPPERLEFT;
	else if (strcmp(type,"bc")== 0)
		Charjust = BOTTOMCENTER;
}


/* graph_window_set() -- map user window onto X window */
graph_window_set(win_id, x1,y1,x2,y2)
	int win_id;
	double x1,y1,x2,y2;
{
	win_list_t *win;

	/* graph_window_set() sets up mapping between user window
	 * 	and X window coordinates.
	 * 	(x1,y1) and (x2,y2) must define two opposing corners
	 *	of the window.  The resulting window will be:
	 *
	 *		lower-left corner min(x1,x2),min(y1,y2)
	 *		upper-right corner max(x1,x2),max(y1,y2)
	 */

	if (Hardcopy) { (Hard_funcs.window_set)(x1,y1,x2,y2); return; }

	win = Graph_windows;
	while ((win != NULL) && (win->g_id != win_id))
		win = win->next;

	if (win == NULL) {
		fprintf(stderr,"graph_window_set: window %d doesn't exist\n",win_id);
		return;
	}

	set_scale(win, x1, y1, x2, y2);
}

/* ASSUMED that x1,y1 is the lower left, while x2,y2 is the upper right
   of the rectangle */

graph_rect(x1,y1,x2,y2)
	double x1,y1,x2,y2;
{
	int tx1,ty1,tx2,ty2;

	if (Hardcopy) return;

	GtoX(x1,y1,&tx1,&ty1);
	GtoX(x2,y2,&tx2,&ty2);

	XFillRectangle(Graph_display, Graph_win_id, XGC, tx1, ty2, 
			tx2 - tx1, ty1 - ty2);
}

/* We make no such assumption here as we do in graph_rect.  We do
   assume that we are given two diametrically opposed corners in x1y1x2y2 */

graph_line_rect(x1,y1,x2,y2)
	double x1,y1,x2,y2;
{
	int tx1,ty1,tx2,ty2;

	if (Hardcopy) return;

	GtoX(x1,y1,&tx1,&ty1);
	GtoX(x2,y2,&tx2,&ty2);

	XDrawRectangle(Graph_display, Graph_win_id, XGC, SMIN(tx1, tx2), 
		SMIN(ty1, ty2), ABS((tx2 - tx1)), ABS((ty1 - ty2)));
}

#define MAX_X_POLYLINEPTS  15000

static XPoint *Dotlist;
static int Dotpointer;  /* how many points have been placed in Dotlist */
static int Numdots; /* number of points to be sent from calling proc. */
static double Lastdotx, Lastdoty, Middledotx, Middledoty;
static int twopointsgotten;

graph_start_dots(ndots)
	int ndots;
{
	Dotpointer = 0;
	twopointsgotten = 0;

	if (ndots < 2)  /* need at least two dots to play connect-the-dots */
		return;

 	if (ndots <= Numdots) 
		Numdots = ndots;
	else {
		if (Numdots != 0) 
			free((char *)Dotlist);
		Numdots = ndots;
		if ((Dotlist = (XPoint*) calloc (Numdots, sizeof(XPoint))) == 0) {
			fprintf(stderr,"graph_start_dots: calloc bombed\n");
			return;
		}
	}
}

graph_store_dot(tx, ty)
	double tx, ty;
{
	/* unless we're putting in the first point in our line, don't
	   add this point if it's y value is no different from the last y val */

	if (twopointsgotten) {
		if (! ((Middledoty == Lastdoty) && (Middledoty == ty))) 
			add_dot(Middledotx,Middledoty);
		Lastdoty = Middledoty; Lastdotx = Middledotx;
		Middledoty = ty; Middledotx = tx;
	}
	else if (Dotpointer == 0) { /* first point is always used */
		Lastdoty = ty; Lastdotx = tx;
		add_dot(tx,ty);
	}
	else {	/* second point */
		Middledoty = ty; Middledotx = tx;
		twopointsgotten = 1;
	}
}

graph_connect_dots()
{
	int start, end;

	/* always put in the last dot */
	add_dot(Middledotx, Middledoty);

	if (Hardcopy) { 
		(Hard_funcs.connect_dots)(Dotlist, Dotpointer); 
		return;
	}

	if (Dotpointer <= MAX_X_POLYLINEPTS) 
		XDrawLines(Graph_display, Graph_win_id, XGC, Dotlist, 
			Dotpointer,  CoordModeOrigin);

	else { /* need to send the poly lines in sections 
			which the X server can handle */
		start = 0;
		end = 0;
		while (end < Dotpointer) {
			end = start + MAX_X_POLYLINEPTS;
			if (end >= Dotpointer) 
				end = Dotpointer;
			XDrawLines(Graph_display, Graph_win_id, XGC, &(Dotlist[start]),
				end - start, CoordModeOrigin);
			start = end - 1;
		}	
	}
}


static
add_dot(x,y)
	double x, y;
{
	int tx, ty;

	if (Dotpointer >= Numdots) {
	/*	fprintf(stderr, "graph_store_dot: trying to store more dots \
					than you reserved space for.\n"); */
		return;
	}

	if (Hardcopy) 
		(Hard_funcs.GtoWhatever)(x, y, &tx, &ty); 
	else GtoX(x, y, &tx, &ty);
	Dotlist[Dotpointer].x = (short)tx;
	Dotlist[Dotpointer].y = (short)ty;
	Dotpointer++;
}

graph_fat_line(x1,y1,x2,y2)
	double x1,y1,x2,y2;
{
	int tx1,ty1,tx2,ty2;
	XGCValues vals;

	if (Hardcopy) { (Hard_funcs.line)(x1,y1,x2,y2); return; }
	
	GtoX(x1,y1,&tx1,&ty1);
	GtoX(x2,y2,&tx2,&ty2);

	vals.line_width = 3;
	XChangeGC(Graph_display, XGC, GCLineWidth, &vals);
	XDrawLine(Graph_display, Graph_win_id, XGC, tx1,ty1,tx2,ty2);
	vals.line_width = 0;
	XChangeGC(Graph_display, XGC, GCLineWidth, &vals);
}

graph_line(x1,y1,x2,y2)
	double x1,y1,x2,y2;
{
	int tx1,ty1,tx2,ty2;

	if (Hardcopy) { (Hard_funcs.line)(x1,y1,x2,y2); return; }
	
	GtoX(x1,y1,&tx1,&ty1);
	GtoX(x2,y2,&tx2,&ty2);

	XDrawLine(Graph_display, Graph_win_id, XGC, tx1,ty1,tx2,ty2);
/*
	if (((tx1 < 0.0) && (tx2 < 0.0)) ||
		((tx1 > Graph_xres) && (tx2 > Graph_xres)) ||
		((ty1 < 0.0) && (ty2 < 0.0)) ||
		((ty1 > Graph_yres) && (ty2 > Graph_yres)))
		;
	else XDrawLine(Graph_display, Graph_win_id, XGC, tx1,ty1,tx2,ty2);
*/
}

graph_line_yclip(x0,y0,x1,y1,ymin,ymax)
	double x0,y0,x1,y1,ymin,ymax;
{
	if ((x0 == x1) && (y0 == y1))
		return;

	if (((y0 > ymax) && (y1 > ymax)) ||
		((y0 < ymin) && (y1 < ymin)))
		return;

	if (y0 > ymax) {
		/* line goes through top */
		x0 = x0 + ((x1 - x0) * ((ymax - y0)/(y1 - y0)));
		y0 = ymax;
	}
	else if (y1 > ymax) {
		/* line goes through top */
		x1 = x0 + ((x1 - x0) * ((ymax - y0)/(y1 - y0)));
		y1 = ymax;
	}

	if (y0 < ymin) {
		x0 = x0 + ((x1 - x0) * ((ymin - y0)/(y1 - y0)));
		y0 = ymin;
	}
	else if (y1 < ymin) {
		x1 = x0 + ((x1 - x0) * ((ymin - y0)/(y1 - y0)));
		y1 = ymin;
	}
	graph_line(x0, y0, x1, y1);
}

graph_text(x, y, string)
	double x, y;
	char *string;
{
	int xx, yy;
	int strwidth, strht;

	if (Hardcopy) { (Hard_funcs.text)(x,y,string); return; }

	strwidth = XTextWidth(G_fontinfo, string, strlen(string));
	strht = G_fontinfo->ascent;

	switch (Charjust) {
		case LOWERLEFT:	
			GtoX(x, y, &xx, &yy);
			break;
		case CENTERCENTER:
			strwidth = strwidth/2; 
			strht = strht / 2;
			GtoX(x, y, &xx, &yy);
			xx = xx - strwidth;
			yy = yy + strht;
			break;
		case RIGHTCENTER:
			strht = strht / 2;
			GtoX(x, y, &xx, &yy);
			yy = yy + strht;
			xx = xx - strwidth;
			break;
		case LOWERRIGHT:
			GtoX(x, y, &xx, &yy);
			xx = xx - strwidth;
			break;
		case UPPERLEFT:	
			GtoX(x, y, &xx, &yy);
			yy = yy + strht;
			break;
		case BOTTOMCENTER:
			strwidth = strwidth/2; 
			GtoX(x, y, &xx, &yy);
			xx = xx - strwidth;
			break;
		default:
			printf("graph_text: invalid char_just set.\n");
			return;
	}
	XDrawString(Graph_display, Graph_win_id, XGC, xx, yy,
			string, strlen(string));
}

double
graph_str_width(str)
	char *str;
{
	double wg, dummy;
	int w;

	if (Hardcopy) { return((Hard_funcs.str_width)(str)); }

	if (G_fontinfo == NULL) /* estimate something */
		wg = (double) (strlen(str) * 3);
	else {
		w = XTextWidth(G_fontinfo, str, strlen(str));
		/* transform w (in X pixels) to wg (graphic coords) */
		XtoG(Graph_currwin, w, 0, &wg, &dummy);
	}
	return(wg);
}

double
graph_str_height()
{
	double wg, dummy;
	int w;

	if (Graph_currwin == NULL) 
		return(10.0);

	w = Graph_currwin->xfont->ascent + Graph_currwin->xfont->descent;
	/* transform w (in X pixels) to wg (graphic coords) */
	XtoG(Graph_currwin, w, 0, &wg, &dummy);
	return(wg);
}

graph_text_int(x, y, num)
	double x, y;
	int num;
{
	char holder[20];

	sprintf(holder,"%d",num);
	graph_text(x, y, holder);
}

graph_text_double(x, y, num, precise)
	double x, y;
	double num;
	int precise;
{
	char holder[20], buf[8];
	char decimals[2];

	sprintf(decimals,"%d",Sig_decimals);

	strcpy(buf,"%1.");
	strcat(buf, decimals);
	if (precise)
		strcat(buf, "e");
	else strcat(buf, "f");

/*	if (precise)
		sprintf(holder, "%1.2e", num);
	else sprintf(holder,"%1.2f",num);
*/

	sprintf(holder, buf, num);
	graph_text(x, y, holder);
}

/* set the Graph_currwin global.  This is the var which points to the
   window to which all future X requests will be *sent*.  That is not
   to say anything about which windows we will listen for input *from*.
   See graph_set_input for that.  Returns 0 if no window has this
   win_id, else returns 1. */

int
graph_set_drawing(win_id)
	int win_id;
{
	win_list_t *win;

	if (Hardcopy) return(1);

	win = Graph_windows;
	while ((win != NULL) && (win->g_id != win_id))
		win = win->next;

	if (win == NULL) {
	/*	fprintf(stderr,"graph_set_drawing: window %d doesn't exist\n",win_id);*/
		return(0);
	}

	Graph_win_id = win->x_id;
	Gx1 = win->gx1;
	Gy1 = win->gy1;
	Gx2 = win->gx2;
	Gy2 = win->gy2;
	GtoX_scalex = win->GtoX_scalex;
	GtoX_scaley = win->GtoX_scaley;
	Yres = win->yres;

	Graph_currwin = win;

	if ((strcmp(Graph_currwin->font_string, "small")==0)||
		(Graph_currwin->xres < 650)||(Graph_currwin->yres < 350)) {
		XSetFont(Graph_display, XGC, G_fontinfosm->fid);
		G_fontinfo = G_fontinfosm;
		Graph_currwin->xfont = G_fontinfosm;
	}
	else {
		XSetFont(Graph_display, XGC, G_fontinfomd->fid);
		G_fontinfo = G_fontinfomd;
		Graph_currwin->xfont = G_fontinfomd;
	}
	return(1);
}

/* if win_id is a valid id (that is, we have created an X window and
   assigned this number to represent it to the outside world) then
   tell X to listen for certain event types.  Note that we always
   listen for window exposure and window resize. */

graph_set_input(win_id, exclusive, s)
	int win_id, exclusive;
	char *s;
{
	win_list_t *win, *winptr, *which_win();

	if ((win = which_win(win_id)) == NULL)
		return;

	if(strcmp(s,"") == 0)
		win->inputmask = 
			ExposureMask | StructureNotifyMask;
	else if(strcmp(s,"b") == 0)
		win->inputmask = 
			ExposureMask | StructureNotifyMask | ButtonPressMask;
	else if(strcmp(s, "bk") == 0)
		win->inputmask = 
			ExposureMask | StructureNotifyMask |ButtonPressMask| KeyPressMask;
	else if(strcmp(s, "bm") == 0)
		win->inputmask = 
			ExposureMask|StructureNotifyMask|ButtonPressMask|PointerMotionMask;
	else if(strcmp(s, "k") == 0)
		win->inputmask = 
			ExposureMask | StructureNotifyMask | KeyPressMask;
	else if(strcmp(s, "m") == 0)
		win->inputmask = 
			ExposureMask | StructureNotifyMask | PointerMotionMask;
	else {
		fprintf(stderr, "Graph_set_input: %s not recognized\n",s);
		return;
	}

	if (!exclusive) {
		/* make sure listening is turned on for all windows */
		winptr = Graph_windows;
		while (winptr != NULL) {
			if (winptr != win)
				XSelectInput(Graph_display, winptr->x_id, winptr->inputmask);
			winptr = winptr->next;
		}
		Graph_exc_input_win = NULL;
	}
	else { 
		Graph_exc_input_win = win;
		winptr = Graph_windows;
		/* shut off input listening to all other windows (except for
		   the ever-present exposure and resize events */
		while (winptr != NULL) {
			if (winptr != win)
				XSelectInput(Graph_display, winptr->x_id, 
					ExposureMask | StructureNotifyMask);
			winptr = winptr->next;
		}
	}

	XSelectInput(Graph_display, win->x_id, win->inputmask); 
}


/* Await an event from any one of our windows, according to our previous
   calls to XSelectInput.  Note that a resize event is not reported, but
   is handled here. */

char
graph_await_event(win_id, val, x, y)
	double *x, *y;
	int *val;
	int *win_id;
{
	XEvent event;
	int done;
	win_list_t *win;

	/* XNextEvent blocks until an event matching the type we have selected 
	   via call to XSelectInput is received.  */

	done = 0;
	while(!done) {
		XNextEvent (Graph_display, &event);
		if (Graph_exc_input_win == NULL) {
			win = Graph_windows;
			while ((win != NULL) && (win->x_id != event.xany.window))
				win = win->next;
		}
		else win = Graph_exc_input_win;

		if (win != NULL) {
			done = 1;		
			*win_id = win->g_id; 

			switch (event.type) {
				case MotionNotify:
					if (move_event(win, x, y, event))
						return('m');
					else done = 0;
					break;	
				case ButtonPress:
					if (button_event(win, x, y, val, event))
						return('b');
					else done = 0;
					break;
				case ConfigureNotify:
				case Expose:
					if (screenscrewed_event(win_id, event))
						return('s');
					else done = 0; /* didn't really resize or expose anything*/
					break;
				case KeyPress:
					if (key_event(val, event))
						return('k');
					else done = 0; /* got garbage keyhit */
					break;	
				default:
					done = 0;
					break;
			}
		}
		else done = 0;
	}
	return('\0');
}

/*  returns "yes" if a control c has been found in the input queue, "no" if
    there is not. Assumes that we "graph_set_input(k)" before calling it. */
/*
#define	CONTROLC	3

char *
graph_hold_everything()
{
	XEvent	event;
	int i;
	int numevents;
	int yes;
	char c[1]; 

	yes = 0;

	if ((numevents = XPending(Graph_display)) > 0) {
		for (i=0; i< numevents; i++) {
			XNextEvent (Graph_display, &event);
			if ((int)event.type == KeyPress) {
				XLookupString(&event.xkey, c, 1, (KeySym *)NULL, NULL);
				if ((int)c[0] == CONTROLC)
		 			yes = 1;
			}	
		}
	}
	if (yes)
		return("yes");
	else return("no");
}
*/

static int
move_event(win, x, y, event)
	double *x, *y;
	XEvent	event;
	win_list_t *win;
{
	Window this_window;

	/* we want to return the latest position of the cursor, so loop
	   through and trash any other motion events that may be hanging
	   around, reporting only the last one. And, yes, I checked, we
	   really do need this.  It seems to throw out two or three reports
	   for every one we send back. */

	this_window = win->x_id;
	while (XCheckWindowEvent(Graph_display, this_window, 
		PointerMotionMask, &event) == True);

	XtoG(win, event.xmotion.x, event.xmotion.y, x, y);
	return(1);
}

static int
button_event(win, x, y, button, event)
	double *x, *y;
	int *button;
	win_list_t *win;
	XEvent	event;
{
	/* Translate to our button number convention */
	if (event.xbutton.button == Button1) /* left */
		*button = 1;
	else if (event.xbutton.button == Button2) /* middle */
		*button = 0;
	else if (event.xbutton.button == Button3) /* right */
		*button = 3;
	else { 
		printf("Graph_await_event: undefined button");
		return(0);
		}

	XtoG(win, event.xbutton.x, event.xbutton.y, x, y);
	return(1);
}

static int
screenscrewed_event(win_g_id, event)
	int *win_g_id; /* user's id of window that's having the problem. */
	XEvent	event;
{
	Window win_x_id;
	win_list_t *win;
	int need_to_redraw;

	/* note that we don't use our predetermined window as found in await_event.
	   That's because regardless of how Graph_exc_input_window is set, we may 
	   need to resize any other window that's currently alive. */

	win_x_id = event.xany.window;
	win = Graph_windows;
	while ((win != NULL) && (win->x_id != win_x_id))
		win = win->next;

	if (win == NULL) 
		return(0);

	*win_g_id = win->g_id;

	/* now we want to trash any multiple expose events for this window.  I
	   have tried innumerable times to keep this from delivering multiple
	   expose events using the method of checking the "count" field in the
	   xexpose event.  This does not work since count is often zero even
	   when there are expose events to follow.  I've tried XCheckWindowEvent
	   by itself, but often the events just haven't gotten to the queue
	   yet.  So, my only solution is to wait a second and then trash events.
	   It works but it forces the program to pause.  */

	sleep(1); /* wait for the events to get to the queue */

	need_to_redraw = 0;
	do {
		if (event.type == Expose)  
			need_to_redraw = 1;
		else if (event.type == ConfigureNotify) {
			if ((event.xconfigure.width == 0) ||
				(event.xconfigure.height == 0))
				;
			else if ((win->xres != event.xconfigure.width) || 
				(win->yres != event.xconfigure.height)) {
				need_to_redraw = 1;
				win->xres = event.xconfigure.width;
				win->yres = event.xconfigure.height;

		/* resize text: if window has become small, make sure text is small.
	  	 If window has become large and we wanted medium text, make sure 
	  	 text is medium */

				if ((win->xres < 650) || (win->yres < 350)) 
					win->xfont = G_fontinfosm;
				else if (strcmp(win->font_string, "medium") == 0)
					win->xfont = G_fontinfomd;

				if (Graph_currwin == win) {
					G_fontinfo = win->xfont;
					XSetFont(Graph_display, XGC, G_fontinfo->fid);
				}

				set_scale(win, win->gx1, win->gy1, win->gx2, win->gy2);
			}
		}
	}
	while (XCheckWindowEvent(Graph_display, win_x_id, 
		ExposureMask | StructureNotifyMask, &event) == True);

	return(need_to_redraw);
}

static int
key_event(val, event)
	int *val;
	XEvent event;
{
	char c[1]; 

	/* Translate key via XKeymap.  This will return 0 if a weird,
	   non-printing key was hit.  */
	if (! XLookupString(&event.xkey, c, 1, (KeySym *)NULL, NULL))
		return(0);

	if (c[0] == 13)
		*val = 10;	/* map carriage return onto Newline */
	else *val = (int)c[0];
	return(1);
}


/*static */
set_scale(win, gx1, gy1, gx2, gy2)
	win_list_t *win;
	double gx1, gy1, gx2, gy2;	/* coords of window corners in user's world */
{
	if (win->xres == 0) {
		fprintf(stderr,
			"set_scale: not resetting scale -- X window has no width\n");
		return;
	}
	if (win->yres == 0) {
		fprintf(stderr,
			"set_scale: not resetting scale -- X window has no height\n");
		return;
	}

	win->gx1 = gx1; win->gy1 = gy1;
	win->gx2 = gx2; win->gy2 = gy2;

	win->XtoG_scalex = (win->gx2 - win->gx1) / win->xres;
	win->XtoG_scaley = (win->gy2 - win->gy1) / win->yres;

	win->GtoX_scalex = 1.0 / win->XtoG_scalex;
	win->GtoX_scaley = 1.0 / win->XtoG_scaley;

	/* if win is the current drawing win, then reset the drawing globals */
	if (win == Graph_currwin) {
		Gx1 = win->gx1;
		Gy1 = win->gy1;
		Gx2 = win->gx2;
		Gy2 = win->gy2;
		GtoX_scalex = win->GtoX_scalex;
		GtoX_scaley = win->GtoX_scaley;
		Yres = win->yres;
	}
}

/* X coordinate (x,y) to user coordinate (tx,ty) */
static 
XtoG(win, x, y, tx, ty)
	int x, y;
	win_list_t *win;
	double *tx, *ty;
{
	*tx = (x * win->XtoG_scalex) + win->gx1;
	*ty = ((win->yres - y) * win->XtoG_scaley) + win->gy1;
}

/* user coordinate (tx,ty) to X coordinate (x,y) */
static 
GtoX(x, y, tx, ty)
	double x, y;
	int *tx, *ty;
{
/*	*tx = ((ABS((x - Gx1))) * GtoX_scalex);  */
	*tx = (int)((x - Gx1) * GtoX_scalex); 
	*ty = (int)(Yres - ((y - Gy1) * GtoX_scaley));
}

graph_end_win(win_id)
	int win_id;
{
	win_list_t *win, *this, *previous, *which_win();
	XEvent ev;
	int done;

	if ((win = which_win(win_id)) == NULL) 
		return;

	/* unshow the window.  Will generate a UnmapNotify event--not that I care.
	   However, will also generate an Expose event for the parent.  This must
	   be trashed here else we will get a reported event later which can
	   royally screw up everything to follow.  */
	XUnmapWindow(Graph_display, win->x_id);
	XFlush(Graph_display);

	if (win->underside != NULL) {
		/* first, trash all events in the queue, up to and including the 
		   Expose Event for killing this popup window...else we'd end up
 		   redrawing the parent which isn't necessary since we've saved 
		   the underside */
		done = 0;
		while (!done) {
			XNextEvent(Graph_display, &ev);
			if (ev.type == Expose)
				done = 1;
		}

		XSetFunction(Graph_display, XGC, GXcopy);
		XSetPlaneMask(Graph_display, XGC, (unsigned int)
			(pow(2.0, (double)DefaultDepth(Graph_display, Graph_screen))-1));
	
		XCopyArea(Graph_display, win->underside, win->my_parent->x_id, XGC, 
			0, 0, win->xres, win->yres, win->pu_x1, win->pu_y1); 
	
		XFlush(Graph_display);
		XFreePixmap(Graph_display, win->underside);
	}

	/* unhook win from the list of wins and say bye bye */
	previous = NULL;
	this = Graph_windows;
	while (this != win) {
		previous = this;
		this = this->next;
	}
	if (previous != NULL)
		previous->next = this->next;
	
	free ((char *)win);
}

static win_list_t *
which_win(win_id)
	int win_id;
{
	win_list_t *win;

	win = Graph_windows;
	while ((win != NULL) && (win->g_id != win_id))
		win = win->next;

	if (win == NULL) {
		return(NULL);
	}
	else return(win);
}

/* Within the given bounding box (given in user's coords), what is the
   shortest dimension?  This is used so that the application can determine
   a square aspect ratio for the window, even though it doesn't have
   direct access to the size in pixels. */
/*
#define IABS(__x)		(((__x) < 0) ? (-(__x)) : (__x))

graph_shortest_dim(x1, y1, x2, y2, x, y)
	double x1, y1, x2, y2;
	double *x, *y;
{
	int tx1, ty1, tx2, ty2;
	int distx, disty;

	GtoX(x1, y1, &tx1, &ty1);
	GtoX(x2, y2, &tx2, &ty2);

	distx = IABS(tx2 - tx1);
	disty = IABS(ty2 - ty1);
	if (disty < distx)
		XtoG(Graph_currwin, disty, disty, x, y);
	else XtoG(Graph_currwin, distx, distx, x, y);
	*y = (y1 > y2 ? y1 : y2) - *y;
}
*/
graph_flush()
{
	XFlush(Graph_display);
}
