/*---------------------------------------------------------------------------
  #######      #     #########   #          #   #    ########     #    #    #
  #            #     #            #        #    #    #            #    #    #
   ###         #    #              #      #     #    #####         #  # #  #
      ###      #    #    #####     #     #      #    #             #  # #  #
         #     #    #        #      #   #       #    #              # #  # #
  ##     #     #     ##      #       # #        #    #              ##   ##
    ####       #       ######         #         #    ########        #    #

    Designed and written by
    Lisa Pickel - pickel@mcnc.org 

    Copyright 1990, 1991 by MCNC.
    This program may be freely distributed provided that the
	copyright notice and author identification are retained.
--------------------------------------------------------------------------- */
#include <stdio.h>
#include <math.h>
#include "candl.h"
#include "hardcopy.h"

/* This file, created by user of these routines, contains these three
   structures, filled and ready to go:

	#define G_MAXCOLORS	
	#define G_MAXLAYERS
	#define G_PLANES
	#define G_COLORS
	#define G_HILITE_BIT
	#define G_LINESTYLES
	
	struct G_cmap_t {
		double red;
		double green;
		double blue;
	}  G_cmap[];
	
	struct G_lmap_t { 
		short color,mask,bw,linestyle;
	} G_lmap[];
	
	struct G_linestyle_t {
		int n_elems;
		char dash_list[8];
	} G_linestyle[G_LINESTYLES];
*/
	
#define x_MV		65535.0 /* max value for red, green, blue values */
#define DRAW_MODE	0
#define UNDRAW_MODE	1
#define HIGHLIGHT_MODE	2
#define UNHIGHLIGHT_MODE	3

extern GC XGC;

/* The following can be changed in two ways:  either the user of these
   routines changes them by calling graph_drawmode or by graph_set_layer
   which adjusts these guys as necessary.
*/
static int G_drawmode;		/* current drawmode as set by user */
static int G_layer;			/* position of current layer in G_lmap */
static int X_colormapstart; /* beginning of colormap which X gave us */
static int X_maskmask;		/* bits we always want to write in */

int G_blackwhite;			/* couldn't allocate wanted colors, so treat
								this device as a black and white one */
int X_zero, X_one;			/* where we store this device's black and 
								white pixel numbers */

extern Window Graph_win_id;
extern Display	*Graph_display;
extern int	Graph_screen;

extern int Hardcopy;
extern func_t Hard_funcs;

/* IMPLIED USAGE NOTE:  there are no DEFAULT COLORS loaded.  You must
   call graph_color_map before starting to use any drawing functions
   like graph_rect.  Relying on this, the global vars are initialized
   in here too.

   ANOTHER USAGE NOTE:  It is assumed that you have defined a color map
   IN candl.h which makes sense.
*/

/* This is the routine I call to do all the color map setup. 
   USING DEC'S X11: Note that you cannot use colors 254 and
   255 because they are for the foreground and background of the display
   (usually black and white, or, in X10, blue and white).  
  
   USING MIT'S X11: You cannot use the highest colors (e.g. the 250's)
   since that is where the xterm window colors are stored.

   To use all
   256 colors you will need to allocate your own colormap (XCreateColormap)
   instead of borrowing spots in the default map.  You can do this, but
   beware:  as soon as the user moves his cursor into your application's
   window, ALL THE COLORS ON THE SCREEN (including Xterm windows and such)
   will be defined according to your colormap.  This produces an annoying
   flashing unless you know what you are doing.  Personally, I gave up on
   that whole approach, but then I only need 64 colors.
 */ 

graph_color_map()
{
	XColor cmap_entry;
	int i;
	Colormap Xcolormap, XDefaultColormap();
	unsigned long planesmasks[G_PLANES], pixels[G_COLORS];

	if (Hardcopy) { (Hard_funcs.color_map)(); return; }

	G_drawmode = DRAW_MODE;
	G_layer = -1;
	Xcolormap = XDefaultColormap(Graph_display, Graph_screen);

	if (XAllocColorCells(Graph_display, Xcolormap, True, planesmasks,
		G_PLANES, pixels, G_COLORS) == 0) { 

		G_blackwhite = 1;
		X_zero = BlackPixel(Graph_display, Graph_screen);
		X_one = WhitePixel(Graph_display, Graph_screen);
		XSetState(Graph_display, XGC, 1, 0, GXcopy, AllPlanes);
		return;
	}

	G_blackwhite = 0;
	X_colormapstart = pixels[0];
	cmap_entry.flags = (char) DoRed | DoGreen | DoBlue;
	for (i= 0; i < G_MAXCOLORS; i++) {
		/* load the color */
		cmap_entry.pixel = X_colormapstart + i;
		cmap_entry.red = (short) (G_cmap[i].red * x_MV);
		cmap_entry.green = (short) (G_cmap[i].green * x_MV); 
		cmap_entry.blue = (short) (G_cmap[i].blue * x_MV);
		XStoreColor(Graph_display, Xcolormap, &cmap_entry);
	}

	X_maskmask = (int)(pow(2.0, (double)XDisplayPlanes(Graph_display, 
		Graph_screen)) - G_MAXCOLORS);
	XSetState(Graph_display, XGC, 1, 0, GXcopy, X_maskmask);
}

/* This function changes the current layer. */
graph_set_layer(layernum)
	int layernum;
{
	if (Hardcopy) { (Hard_funcs.set_layer)(layernum); return; }

	if ((layernum < 0) || (layernum >= G_MAXLAYERS)) {
		fprintf(stderr, "graph_set_layer: %d is not a layer\n",layernum);
		return; 
	} 
	G_layer = layernum; 
	adjust_layer();
} 

static 
adjust_layer() 
{ 
	int linestyle;

	if (G_blackwhite) {
		linestyle = G_lmap[G_layer].linestyle;

		if (linestyle == -2)
			XSetFunction(Graph_display, XGC, GXinvert);
		else XSetFunction(Graph_display, XGC, GXcopy);

		if (linestyle < 0) {
			 XSetLineAttributes(Graph_display, XGC, 1, LineSolid, 
					CapNotLast, JoinMiter);
		}
		else {
			XSetLineAttributes(Graph_display, XGC, 1, LineOnOffDash, 
					CapNotLast, JoinMiter);
			XSetDashes(Graph_display, XGC, 0, G_linestyle[linestyle].dash_list,
				G_linestyle[linestyle].n_elems);
		}

		if (G_drawmode == UNDRAW_MODE) 
			XSetForeground(Graph_display, XGC, X_zero); 
		else if (G_drawmode == UNHIGHLIGHT_MODE)
			XSetFunction(Graph_display, XGC, GXnoop);
		else {
			if (G_lmap[G_layer].bw == 1)
				XSetForeground(Graph_display, XGC, X_one);
			else XSetForeground(Graph_display, XGC, X_zero);
		}

		return;
	}				

	/* Draw mode is easy, just use everything as it is found. */ 
	if (G_drawmode == DRAW_MODE) { 
		XSetForeground(Graph_display, XGC, 
			X_colormapstart + G_lmap[G_layer].color); 
		XSetPlaneMask(Graph_display, XGC, X_maskmask | G_lmap[G_layer].mask);  
	} 
	/* Undraw is like draw except that we set the color to background color.  
	   Using the user's writemask, we will then set the enabled bits to zero 
	   (since background color is 00000..), thus undrawing
	   the color.  Why not set the writemask to 11111...?  Because
	   undrawing may be a matter of removing a layer (poly over metal).
	   We do not want to draw background color, but to unset bits
	   involved in the color as specified by the writemask.
	*/
	else if (G_drawmode == UNDRAW_MODE) {
		XSetForeground(Graph_display, XGC, X_colormapstart);
		XSetPlaneMask(Graph_display, XGC, X_maskmask | G_lmap[G_layer].mask);
	}

	/* Unhighlight involves  resetting the highlight bit back to zero.
	   This should return a pixel to it's original, unhilited color.
	*/
	else if (G_drawmode == UNHIGHLIGHT_MODE) {
		XSetForeground(Graph_display, XGC, X_colormapstart);
		XSetPlaneMask(Graph_display, XGC, X_maskmask | G_HILITE_BIT);
	}

	/* Highlight involves setting color to user's requested color with
	   a one put into the highlight bit.  Then we use user's writemask
	   and put a one in it's highlight bit.  This insures that we will
	   select the highlight counterpart of a color in the color map.
	*/
	else if (G_drawmode == HIGHLIGHT_MODE) {
		XSetForeground(Graph_display, XGC, X_colormapstart + 
			G_lmap[G_layer].color | G_HILITE_BIT);
		XSetPlaneMask(Graph_display, XGC, X_maskmask | G_lmap[G_layer].mask 
			| G_HILITE_BIT);
	}
}


graph_draw_mode(mode)
	int mode;	/* 0,1,2,3 */
{
	if (Hardcopy) return; 

	if ((mode < 0) || (mode > 3)) {
		fprintf(stderr,"graph_draw_mode: mode %d out of range 0 to 3\n",mode);
		return;
	}

	/* if no layer has been set, don't change color, writemask-- 
	   just set the drawmode global for later use. */
	if (G_layer == -1)
		G_drawmode = mode;
	else {	
		G_drawmode = mode;
		adjust_layer();
	}
}

#include <X11/cursorfont.h>

#define C_POINTER	0
#define C_WATCH 1	
#define C_LEFTCORNER	2
#define C_RIGHTCORNER	3	
#define C_INVISIBLE     4

#define G_FONTCURSORS	4

int G_cursormasks[G_FONTCURSORS] = {
	XC_arrow, /* pointer */ 
	XC_watch, /* watch */
	XC_top_left_corner, /* left corner */
	XC_bottom_right_corner, /* right corner */
};

static Cursor G_cursors[G_FONTCURSORS];
static Cursor G_invisible_cursor;

/* called by graph_dev_init to inform X about all our cursors, 
   as defined above */
init_cursors()
{
	int i;
	Pixmap pmap;
	XColor fcolor, bcolor;
 	GC tempGC;

	/* load the font cursors */
	for (i = 0; i < G_FONTCURSORS; i++) 
		G_cursors[i] = XCreateFontCursor(Graph_display, G_cursormasks[i]);

	/* now set up the invisible cursor */
	bcolor.flags = fcolor.flags = (char) DoRed | DoGreen | DoBlue;
	fcolor.red = fcolor.green = fcolor.blue = 0;
	bcolor.red = bcolor.green = bcolor.blue = 0;
 
	pmap = XCreatePixmap(Graph_display, Graph_win_id, 16, 16, 1);
	tempGC = XCreateGC(Graph_display, pmap, 0, 0);
	XSetPlaneMask(Graph_display, tempGC, 1);
	XSetForeground(Graph_display, tempGC, 0);

	XFillRectangle(Graph_display, pmap, tempGC, 0,0,16,16);
	G_invisible_cursor = XCreatePixmapCursor(Graph_display, pmap,
				 pmap, &fcolor, &bcolor, 8, 8);

	XFreeGC(Graph_display, tempGC);
	XFreePixmap(Graph_display, pmap);
}

/* relate this cursor to the current window */
graph_cursor(cursorstyle)
	char *cursorstyle;
{
	XUndefineCursor(Graph_display, Graph_win_id);

	if (strcmp(cursorstyle, "pointer") == 0)
		XDefineCursor(Graph_display, Graph_win_id, G_cursors[C_POINTER]);
	else if (strcmp(cursorstyle, "upperleft") == 0)
		XDefineCursor(Graph_display, Graph_win_id, G_cursors[C_LEFTCORNER]);
	else if (strcmp(cursorstyle, "lowerright") == 0)
		XDefineCursor(Graph_display, Graph_win_id, G_cursors[C_RIGHTCORNER]);
	else if (strcmp(cursorstyle, "watch") == 0)
		XDefineCursor(Graph_display, Graph_win_id, G_cursors[C_WATCH]);
	else if (strcmp(cursorstyle, "invisible") == 0)
		XDefineCursor(Graph_display, Graph_win_id, G_invisible_cursor);
	else fprintf(stderr,"graph_cursor: do not have style %s\n",cursorstyle);	
	XFlush(Graph_display);
}

int
graph_colormap_begin()
{
	return(X_colormapstart);
}

int
graph_set_line_width( line_width )
int line_width;
{
/* no effect for X11 displays */

	if (Hardcopy) { (Hard_funcs.set_line_width)(line_width); return; }
}
